/******************************************************************************/
/* exemple5a.c                       09/04/96 - Edouard.Thiel@lim.univ-mrs.fr */
/*                                                                            */
/* 5. BORD EXTERIEUR DES IMAGES   a. Etiquetage d'objets                      */
/******************************************************************************/

/* 
 * Le include suivant est OBLIGATOIRE dans tous vos .c 
*/ 
#include "../IpsDir/include/tel_oper.h"


/*
 * Declaration des 4 voisins directs puis des 4 voisins indirects.
 *
 * Attention a la dualite des connexites entre le fond et la forme.
*/
static int v8x[8] = { 1, 0,-1, 0, 1,-1,-1, 1 },
           v8y[8] = { 0,-1, 0, 1,-1,-1, 1, 1 },
           v8t[8],
           v8m = 4;

/*
 * Fonctions du module
*/
void ex5a_erosion ();
void ex5a_label_object ();
void ex5a_cadre ();
int  ex5a_fill ();


/******************************************************************************
 * EFFET : erosion des objets en connexite c_b ;
 *         on supprime les points de la frontiere des objets, i.e les points 
 *         objet ayant un voisin du fond pour la connexite du fond.
 *
 *****************************************************************************/
void ex5a_erosion (ima1, ima2, cmin, xmax, ymax, xtot, xytot, c_b)
Unt16 *ima1, *ima2;
int cmin, xmax, ymax, xtot, xytot, c_b;
{
    int x, y, t, i, hop;

    /* preparation de v8t */
    for (i = 0; i < 8; i++) v8t[i] = v8x[i] + v8y[i] * xtot;
    v8m = (c_b == 4) ? 8 : 4; /* connexite du fond */

    /* parcours de la partie visible de l'image */
    for (y = cmin; y < ymax; y++)
    for (x = cmin; x < xmax; x++) {
	t = y*xtot+x;

	if (ima1[t] != 0) {

	    for (i = 0; i < v8m; i++)
		if (hop = (ima1[t + v8t[i]] == 0)) break;
	    ima2[t] = (hop) ? 0 : 255;

	} else ima2[t] = 0;
    }

    /* Ce balayage de l'image est la facon classique de proceder ;
     * comme les points du bord sont a 0, on aurait pu faire a la place
     *     for (t = 0; t < xytot; t++) if (ima1[t] != 0) { .. }
     * ce qui est plus efficace et demande moins de parametres a transmettre.
     * (cf entete de la fonction ex5b_calc_dt() dans exemple5b.c)
    */
}


/******************************************************************************
 * EFFET : Etiquetage de chaque objet en connexite c_b,
 *         avec label <= 65535.
 *
 *****************************************************************************/
void ex5a_label_object (ima1, ima2, cmin, xmax, ymax, xtot, xytot, c_b)
Unt16 *ima1, *ima2;
int cmin, xmax, ymax, xtot, xytot, c_b;
{
    int	t, ta, i, label, aire;

    /* init image 2 a 0 */
    for (t = 0; t < xytot; t++) ima2[t] = 0;

    /* Cadre a une valeur != de 0, ici par exemple 3 
     * Remarque : n'importe quelle autre valeur != 0 ferait l'affaire !
    */
    ex5a_cadre (ima2, cmin, xmax, ymax, xtot, xytot, 3);

    /* preparation de v8t */
    for (i = 0; i < 8; i++) v8t[i] = v8x[i] + v8y[i] * xtot;
    v8m = c_b; label = 1;

    /* parcours et detection objets image */
    for (ta = 0; ta < xytot; ta++)
    if (ima1[ta] != 0 && ima2[ta] == 0) {

	aire = ex5a_fill (ima1, ima2, ta, ima1[ta], 0, label);
	printf ("Aire objet %d = %d\n", label, aire);

	if (++label > 65535) label = 1;
    }

    /* on remet le cadre a 0 */
    ex5a_cadre (ima2, cmin, xmax, ymax, xtot, xytot, 0);
}


/******************************************************************************
 * EFFET : Entoure l'image par un cadre de couleur v
 *
 * Le cadre est situe dans le bord exterieur de l'image,
 * mais est "contre" l'image (cmin peut etre > 1).
 *****************************************************************************/
void ex5a_cadre (ima1, cmin, xmax, ymax, xtot, xytot, v)
Unt16 *ima1;
int cmin, xmax, ymax, xtot, xytot, v;
{
    int t, cmin1 = cmin - 1;

    for (t = cmin1; t <= xmax; t++)
	ima1[t + cmin1*xtot] = ima1[t + ymax*xtot] = v;
    for (t = cmin1; t <= ymax; t++)
	ima1[t*xtot + cmin1] = ima1[t*xtot + xmax] = v;
}


/******************************************************************************
 * EFFET : Parcourt en connexite v8m l'objet de ima1, de couleur c1, 
 *         et remplis ima2, avec la trace c2 --> c3.
 *
 * RENVOIE : l'aire (i.e le nombre de pixels de l'objet)
 *
 * NOTE : Cet algorithme de propagation empile les voisins a colorier,
 *        et vide periodiquement la pile. La methode est optimale car on
 *        ne teste JAMAIS si les points sont dans l'image : c'est la 
 *        presence du cadre qui contient la propagation.
 *****************************************************************************/
int ex5a_fill (ima1, ima2, ta, c1, c2, c3)
Unt16 *ima1, *ima2;
int ta, c1, c2, c3;
{
    int pilt[10000], pilp, i, j, s, t, test, aire;

    /* initialisation */
    aire = 1;
    pilt[0] = ta; pilp = 1; 
    ima2[pilt[0]] = c3;

    do {

	t = pilt[--pilp];

	for (i = 0; i < v8m; i++) { /* on teste les voisins */
	    s = t + v8t[i];
	    if (ima1[s] == c1 && ima2[s] == c2) { /* on empile */
		ima2[s] = c3;
		aire++;
		pilt[pilp++] = s;
	    }
	}

	if (pilp >= 9990) { /* on vide la pile */

	    j = 0;
	    while (j < pilp) {
		t = pilt[j];
		for (i = 0; i < v8m; i++) {
		    s = t + v8t[i];
		    if (test = (ima1[s] == c1 && ima2[s] == c2)) break;
		}
		if (test) j++;  /* rien a faire, on passe au suivant */
		else pilt[j] = pilt[--pilp]; /* on depile */
	    }
	}
    }
    while (0 < pilp && pilp < 9990);

    return aire;
}


