/******************************************************************************/
/* exemple5b.c                       09/04/96 - Edouard.Thiel@lim.univ-mrs.fr */
/*                                                                            */
/* 5. BORD EXTERIEUR DES IMAGES   b. Image de distances                       */
/******************************************************************************/

/* 
 * Le include suivant est OBLIGATOIRE dans tous vos .c 
*/ 
#include "../IpsDir/include/tel_oper.h"


/*
 * Le masque de chanfrein
*/
#define MASKNB 32

typedef struct {
	int 	x[MASKNB], y[MASKNB],	/* coordonnees  */
		t[MASKNB],		/* t = y*xtot+x */
		p[MASKNB],		/* ponderation  */
		nb;
	} ChamferMask;

static ChamferMask mask;

/*
 * Fonctions du module
*/
void ex5b_set_mask ();
void ex5b_calc_dt ();



/******************************************************************************
 * EFFET : Fixe les coefficients du masque de chanfrein.
 *
 * 0: d4  -  1: d8  -  2: chanfrein 3,4  -  3: chanfrein 5,7,11
 *****************************************************************************/
void ex5b_set_mask (num, xtot)
int num, xtot;
{
    int i, a, b, c;

    /* Ponderations et nb de points du masque */
    switch (num) {
	case 0 : a = 1;                mask.nb = 2; break;
	case 1 : a = 1; b = 1;         mask.nb = 4; break;
	case 2 : a = 3; b = 4;         mask.nb = 4; break;
	case 3 : a = 5; b = 7; c = 11; mask.nb = 8; break;
    }

    /* Coordonnees x,y des points et ponderation a,b,c */
    switch (num) {
	case 3 : 
	    mask.x[7] = -2; mask.y[7] =  1;  mask.p[7] =  c;
	    mask.x[6] = -1; mask.y[6] =  2;  mask.p[6] =  c;
	    mask.x[5] =  1; mask.y[5] =  2;  mask.p[5] =  c;
	    mask.x[4] =  2; mask.y[4] =  1;  mask.p[4] =  c;
	case 2 : 
	case 1 : 
	    mask.x[3] = -1; mask.y[3] =  1;  mask.p[3] =  b;
	    mask.x[2] =  1; mask.y[2] =  1;  mask.p[2] =  b;
	case 0 : 
	    mask.x[1] =  1; mask.y[1] =  0;  mask.p[1] =  a;
	    mask.x[0] =  0; mask.y[0] =  1;  mask.p[0] =  a;
    }

    /* Preparation de t = y*xtot+x */
    for (i = 0; i < mask.nb; i++) 
	mask.t[i] = mask.y[i] * xtot + mask.x[i];
}


/******************************************************************************
 * EFFET : A partir d'une image binaire, calcule en chaque point sa
 *         distance (de chanfrein) au complementaire.
 *
 * NB : Il faut absolument que l'image soit entouree par un bord ayant
 *      l'epaisseur suffisante pour le masque.
 *
 * On devrait normalement realiser un balayage du type
 *     for (y = cmin; y < ymax; y++)
 *     for (x = cmin; x < xmax; x++)
 *     if (ima1[ t = y*xtot+x ] != 0) {
 *         ima2[t] =
 *     }
 *
 * Or ici les points du bord sont a 0, et on fait des calculs sur les 
 * points non nuls de l'image ; il est donc beaucoup plus habile de faire
 *     for (t = 0; t < xytot; t++) 
 *     if (ima1[t] != 0) { 
 *         ima2[t] = .. 
 *     }
 *****************************************************************************/
void ex5b_calc_dt (ima1, ima2, xytot)
Unt16 *ima1, *ima2;
int xytot;
{
    int  t, i, mini, v;

    /* 
     * Premier balayage 
    */
    for (t = 0; t < xytot; t++)
    if (ima1[t] != 0) {

	mini = ima2[t - mask.t[0]] + mask.p[0];
	for (i = 1; i < mask.nb; i++) {
	    v = ima2[t - mask.t[i]] + mask.p[i];
	    if (v < mini) mini = v;
	}
	ima2[t] = mini;

    } else ima2[t] = 0; /* car ima2 n'a pas ete initialise'e a 0 */

    /* 
     * Second balayage 
    */
    for (t = xytot-1; t >= 0; t--)
    if (ima2[t] != 0) {

	mini = ima2[t];
	for (i = 0; i < mask.nb; i++) {
	    v = ima2[t + mask.t[i]] + mask.p[i];
	    if (v < mini) mini = v;
	}
	ima2[t] = mini;
    }
}

