/******************************************************************************/
/* exemple7a.c                       03/05/96 - Edouard.Thiel@lim.univ-mrs.fr */
/*                                                                            */
/* TRACE' D'HISTOGRAMMES dans la section FIG et EXE                           */
/******************************************************************************/

/* 
 * Le include suivant est OBLIGATOIRE dans tous vos .c 
*/ 
#include "../IpsDir/include/tel_oper.h"


/* Definitions */
#define LG_LIGNE 256

/* Fonctions du module */
void ex7a_fig_profils ();
void ex7a_calc_histog ();
void ex7a_appel_gnuplot ();
void ex7a_bresenham ();


/* Donnees devant exister en permanence */
static double S_x1 = 0, S_y1 = 0, S_x2 = 0, S_y2 = 0;
static int S_choix;


/*****************************************************************************
 * Callback pour la section FIG.
 *
 * fig1 est le numero de la figure
 * 
 *****************************************************************************/
void ex7a_fig_profils (p, fig1)
Tel_oper_p *p;
int fig1;
{
    switch (p->fig_event) {

	case FIG_PUSH : switch (S_choix = p->fig_butt) {
	    case 1 :
		S_x1 = S_x2 = floor(p->fig_x)+0.5;
		S_y1 = S_y2 = floor(p->fig_y)+0.5;
		CO_DRAW_MODE_XOR;
		CO_DRAW_LINE (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	    case 2 :
		S_x1 = S_x2 = rint (p->fig_x);
		S_y1 = S_y2 = rint (p->fig_y);
		CO_DRAW_MODE_XOR;
		CO_DRAW_RECT (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	  } break;

	case FIG_DRAG : switch (p->fig_butt) {
	    case 1 :
		CO_DRAW_LINE (fig1, S_x1, S_y1, S_x2, S_y2);
		S_x2 = floor(p->fig_x)+0.5; S_y2 = floor(p->fig_y)+0.5;
		CO_DRAW_LINE (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	    case 2 :
		CO_DRAW_RECT (fig1, S_x1, S_y1, S_x2, S_y2);
		S_x2 = rint (p->fig_x); S_y2 = rint (p->fig_y);
		CO_DRAW_RECT (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	  } break;

	case FIG_RELEASE : switch (S_choix) {
	    case 1 :
		CO_DRAW_LINE (fig1, S_x1, S_y1, S_x2, S_y2);
		CO_FIG_DO_RUN;
		break;
	    case 2 :
		CO_DRAW_RECT (fig1, S_x1, S_y1, S_x2, S_y2);
		CO_FIG_DO_RUN;
		break;
	  } break;

	case FIG_CANCEL : switch (S_choix) {
	    case 1 :
		CO_DRAW_LINE (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	    case 2 :
		CO_DRAW_RECT (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	  } break;

	case FIG_REFRESH : switch (S_choix) {
	    case 1 :
		CO_DRAW_MODE_XOR;
		CO_DRAW_LINE (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	    case 2 :
		CO_DRAW_MODE_XOR;
		CO_DRAW_RECT (fig1, S_x1, S_y1, S_x2, S_y2);
		break;
	  } break;
    }
}


/******************************************************************************
 * EFFET : Calcule l'histogramme des niveaux de gris de ima1
 *         Ecrit cet histogramme dans fichier_data
 *
 *****************************************************************************/
void ex7a_calc_histog (ima1, xsiz, ysiz, fichier_data)
Unt16 *ima1;
int xsiz, ysiz;
char *fichier_data;
{
    FILE *f;

    switch (S_choix) {

	case 1 : { /* Profil sur une ligne */

	    int x1 = min (max ((int) floor(S_x1), 0), xsiz),
		x2 = min (max ((int) floor(S_x2), 0), xsiz),
		y1 = min (max ((int) floor(S_y1), 0), ysiz),
		y2 = min (max ((int) floor(S_y2), 0), ysiz);

	    /* Ouverture de fichier_data */
	    f = fopen (fichier_data, "w");
	    if (f == NULL) {
		printf ("Can't write to file %s\n", fichier_data);
		return;
	    }

	    /* Parcours de la droite et ecriture des valerus dans f */
	    ex7a_bresenham (f, ima1, xsiz, x1, y1, x2, y2);

	    /* Fermeture de fichier_data */
	    fclose (f);
	} break;

	case 2 : { /* Zone rectangulaire */

	    int x, y, t, v[256], i,
		x1 = max ((int) min (S_x1, S_x2), 0),
		x2 = min ((int) max (S_x1, S_x2), xsiz),
		y1 = max ((int) min (S_y1, S_y2), 0),
		y2 = min ((int) max (S_y1, S_y2), ysiz);

	    /* init vecteur des niveaux de gris a 0 */
	    for (i = 0; i < 256; i++) v[i] = 0;

	    /* Parcours l'image et incremente v[] */
	    for (y = y1; y < y2; y++)
	    for (x = x1; x < x2; x++) 
		if (ima1[t = y*xsiz+x] < 256) v[ima1[t]]++;

	    /* Ouverture de fichier_data */
	    f = fopen (fichier_data, "w");
	    if (f == NULL) {
		printf ("Can't write to file %s\n", fichier_data);
		return;
	    }

	    /* Ecriture dans fichier_data sous la forme "i v[i]" */
	    fprintf (f, "# Fichier des niveaux de gris\n");
	    for (i = 0; i < 256; i++) fprintf (f, "%d %d\n", i, v[i]);

	    /* Fermeture de fichier_data */
	    fclose (f);
	} break;

    } /* switch */
}


/******************************************************************************
 * EFFET : Appel de gnuplot, qui execute le fichier de commandes f_commandes,
 *         dans lequel on demande de tracer le fichier de donnees f_data,
 *         en precisant le display.
 *
 *****************************************************************************/
void ex7a_appel_gnuplot (f_data, f_commandes, display, appel_gnuplot)
char *f_data, *f_commandes, *display, *appel_gnuplot;
{
    char appel_complet[200];
    FILE *f;

    /* On ouvre le fichier de commandes pour gnuplot */
    f = fopen (f_commandes, "w");
    if (f == NULL) {
	printf ("Can't write to file %s\n", f_commandes);
	return;
    }

    /* 
     * On ecrit les commandes pour gnuplot, avec des commentaires
    */
    fprintf (f, "# Commandes gnuplot pour le tracage de %s\n", f_data);
    fprintf (f, "set term x11\n");
    fprintf (f, "set title '%s - %s'\n", (S_choix == 1) ? 
	"Profil sur la droite" : "Histogramme sur le rectangle", f_data);
    fprintf (f, "set nokey\n");
    fprintf (f, "set grid\n");
    if (S_choix == 1) fprintf (f, "set noxtics\n");
    fprintf (f, "plot '%s' with %s\n", f_data, 
	(S_choix == 1) ? "lines":"impulses");
    fprintf (f, "pause 120\n");

    /* Fermeture du fichier de commandes */
    fclose (f);

    /* 
     * Appel de gnuplot. Par defaut, on a :
     * 
     *     appel_gnuplot = "gnuplot -display %s:0 %s &"
     *     display = "localhost"
     *     f_commandes = "histog.gnu"
     *
     * Le sprintf fera que
     * 
     *     appel_complet = "gnuplot -display localhost:0 histog.gnu &"
    */
    sprintf (appel_complet, appel_gnuplot, display, f_commandes);
    system (appel_complet);
}


/******************************************************************************
 * affichage des valeurs sur le fichier f
 * d'un segment de droite par bresenham 
 *
 *****************************************************************************/

void ex7a_bresenham (f, ima1, xsiz, x1, y1, x2, y2)
FILE *f;
Unt16 *ima1;
int xsiz, x1, y1, x2, y2;
{
    int  a, x, y, dx, dy, px, py, incr1, incr2, d, num;
    char test;

    /* initialisations */
    dx = x2 - x1; dy = y2 - y1;
    test = (abs(dy) <= abs(dx));  /* pente <= 1 */
    if ((test && dx < 0) || (!test && dy < 0)) {
	a = x1; x1 = x2; x2 = a;
	a = y1; y1 = y2; y2 = a;
	dx = -dx; dy = -dy;
    }

    x = x1; y = y1; num = 0;
    fprintf (f, "%d %d\n", num++, ima1[y*xsiz+x]);

    /* trace de la droite */
    if (test) { /* pente <= 1 */

	if (dy >= 0) py = 1; else { py = -1; dy = -dy; }
	incr1 = 2*dy; d = incr1 - dx; incr2 = d - dx;

	while (x < x2) {
	    x += 1;
	    if (d < 0) d += incr1; else { d += incr2; y += py; }
	    fprintf (f, "%d %d\n", num++, ima1[y*xsiz+x]);
	}

    } else { /* pente > 1 */

	if (dx >= 0) px = 1; else { px = -1; dx = -dx; }
	incr1 = 2*dx; d = incr1 - dy; incr2 = d - dy;

	while (y < y2) {
	    y += 1;
	    if (d < 0) d += incr1; else { d += incr2; x += px; }
	    fprintf (f, "%d %d\n", num++, ima1[y*xsiz+x]);
	}
    }
}





