/******************************************************************************/
/* exemple7b.c                       30/04/96 - Edouard.Thiel@lim.univ-mrs.fr */
/*                                                                            */
/* GESTION D'UNE LISTE DE RECTANGLES dans la section FIG                      */
/******************************************************************************/

/* 
 * Le include suivant est OBLIGATOIRE dans tous vos .c 
*/ 
#include "../IpsDir/include/tel_oper.h"


/* Definitions */
#define LG_LIGNE 256

/* Fonctions du module */
void ex7b_fig_rects ();
int ex7b_cherche_clic ();
int ex7b_ajoute_rect ();
int ex7b_deplace_rect ();
int ex7b_supprime_rect ();
int ex7b_redessine ();


/*****************************************************************************
 * Callback pour la section FIG.
 *
 * fig1 est le numero de la figure
 * par1 est le numero du parametre TEXT pour le nom du fichier de points
 * par2 est le numero du parametre INT pour la couleur du trace' *****************************************************************************/
void ex7b_fig_rects (p, fig1, par1, par2)
Tel_oper_p *p;
int fig1, par1, par2;
{
    /* Donnees devant exister en permanence */
    enum { E_None, E_Add, E_Suppr, E_Move, E_Size };
    static int S_etat, S_n;
    static double S_xs, S_ys, S_xa, S_ya, S_xb, S_yb;

    /* Variables */
    char *nom_f = CO_GET_TEXT (par1);
    int coin;

    switch (p->fig_event) {

	case FIG_PUSH :
	    /* Cherche la ligne du point clique' */
	    S_n = ex7b_cherche_clic (nom_f, p->fig_x, p->fig_y, p->fig_zoom,
		&coin, &S_xa, &S_ya, &S_xb, &S_yb);

	    /* En deduit S_etat */
	    S_etat = (p->fig_butt == 1)
		? (S_n < 0) ? E_Add  : (coin) ? E_Size : E_Move
		: (S_n < 0) ? E_None : E_Suppr;

	    /* Memorise coordonnees du clic, arrondies sur la grille entiere */
	    S_xs = rint(p->fig_x); S_ys = rint(p->fig_y);

	    /* Action selon S_etat */
	    switch (S_etat) {
		case E_Add :
		    S_xa = S_xs; S_ya = S_ys;
		case E_Size :
		    CO_DRAW_MODE_XOR;
		    CO_DRAW_RECT (fig1, S_xa, S_ya, S_xs, S_ys);
		    break;
		case E_Move :
		    S_xa -= S_xs; S_ya -= S_ys;
		    S_xb -= S_xs; S_yb -= S_ys;
		    CO_DRAW_MODE_XOR;
		    CO_DRAW_RECT (fig1, S_xa+S_xs, S_ya+S_ys, 
			S_xb+S_xs, S_yb+S_ys);
		    break;
		case E_Suppr :
		    ex7b_supprime_rect (nom_f, fig1, S_n);
		    break;
		case E_None :
		    CO_FIG_CENTER (fig1, S_xs, S_ys);
		    break;
	    }
	    break;

	case FIG_DRAG : switch (S_etat) {
		case E_Add :
		case E_Size :
		    CO_DRAW_RECT (fig1, S_xa, S_ya, S_xs, S_ys);
		    S_xs = rint(p->fig_x); S_ys = rint(p->fig_y);
		    CO_DRAW_RECT (fig1, S_xa, S_ya, S_xs, S_ys);
		    break;
		case E_Move :
		    CO_DRAW_RECT (fig1, S_xa+S_xs, S_ya+S_ys, 
			S_xb+S_xs, S_yb+S_ys);
		    S_xs = rint(p->fig_x); S_ys = rint(p->fig_y);
		    CO_DRAW_RECT (fig1, S_xa+S_xs, S_ya+S_ys, 
			S_xb+S_xs, S_yb+S_ys);
		    break;
	    }
	    break;

	case FIG_RELEASE : switch (S_etat) {
		case E_Add :
		    CO_DRAW_RECT (fig1, S_xa, S_ya, S_xs, S_ys);
		    if (fabs(S_xa-S_xs) * (double) p->fig_zoom >= 1 &&
			fabs(S_ya-S_ys) * (double) p->fig_zoom >= 1 )
		    ex7b_ajoute_rect (nom_f, fig1, CO_GET_INT (par2), 
			S_xa, S_ya, S_xs, S_ys);
		    break;
		case E_Size :
		    CO_DRAW_RECT (fig1, S_xa, S_ya, S_xs, S_ys);
		    if (fabs(S_xa-S_xs) * (double) p->fig_zoom >= 1 &&
			fabs(S_ya-S_ys) * (double) p->fig_zoom >= 1 )
		    ex7b_deplace_rect (nom_f, fig1, 
			S_xa, S_ya, S_xs, S_ys, S_n);
		    break;
		case E_Move :
		    CO_DRAW_RECT (fig1, S_xa+S_xs, S_ya+S_ys, 
			S_xb+S_xs, S_yb+S_ys);
		    ex7b_deplace_rect (nom_f, fig1, S_xa+S_xs, S_ya+S_ys, 
			S_xb+S_xs, S_yb+S_ys, S_n);
		    break;
	    }
	    break;

	case FIG_CANCEL : switch (S_etat) {
		case E_Add :
		case E_Size :
		    CO_DRAW_RECT (fig1, S_xa, S_ya, S_xs, S_ys);
		    break;
		case E_Move :
		    CO_DRAW_RECT (fig1, S_xa+S_xs, S_ya+S_ys, 
			S_xb+S_xs, S_yb+S_ys);
		    break;
	    }
	    break;

	case FIG_REFRESH :
	    ex7b_redessine (nom_f, fig1, CO_GET_INT(par2));
	    break;
    }
}


/*****************************************************************************
 * Cherche dans le fichier nom_f le premier rectangle "proche" de cx,cy
 *
 * Renvoie -1 en cas d'erreur, >=0 la ligne du rectangle trouve'.
 *****************************************************************************/
int ex7b_cherche_clic (nom_f, cx, cy, zoom, coin, xa, ya, xb, yb)
char *nom_f;
double cx, cy;
int zoom, *coin;
double *xa, *ya, *xb, *yb;
{
    FILE *f;
    char ligne[LG_LIGNE];
    double x1, y1, x2, y2, d = 5.0 / (double) zoom;
    int n = -1;

    /* Ouverture du fichier nom_f */
    if ((f = fopen (nom_f, "r")) == NULL) {
	printf ("File %s does not exist.\n", nom_f);
	return -1;
    }

    /* Parcours du fichier nom_f */
    while ( fgets (ligne, LG_LIGNE, f) != NULL ) {
	n++;
	if (sscanf (ligne, "%lf %lf %lf %lf", &x1, &y1, &x2, &y2) < 4) continue;
	if ( (x1-d < cx && cx < x2+d && y1-d < cy && cy < y2+d) &&
	     (cx < x1+d || x2-d < cx || cy < y1+d || y2-d < cy) ) {

	    /* On a trouve le rect, on cherche le coin */
	    if (cx <= x1 && cy <= y1) {
		*xa = x2; *ya = y2; *xb = x1; *yb = y1; *coin = VRAI;
	    } else if (cx <= x1 && y2 <= cy) {
		*xa = x2; *ya = y1; *xb = x1; *yb = y2; *coin = VRAI;
	    } else if (x2 <= cx && cy <= y1) {
		*xa = x1; *ya = y2; *xb = x2; *yb = y1; *coin = VRAI;
	    } else if (x2 <= cx && y2 <= cy) {
		*xa = x1; *ya = y1; *xb = x2; *yb = y2; *coin = VRAI;
	    } else {
		*xa = x1; *ya = y1; *xb = x2; *yb = y2; *coin = FAUX;
	    }

	    fclose (f);
	    return n; /* Le rect est sur la ligne n */
	}
    }

    /* Rect non trouve' */
    fclose (f);
    return -1;
}


/*****************************************************************************
 * Ajoute dans le fichier nom_f le rectangle x1,y1,x2,y2.
 *
 * Renvoie -1 en cas d'erreur, sinon 0 pour succes.
 *****************************************************************************/
int ex7b_ajoute_rect (nom_f, fig1, coul, x1, y1, x2, y2)
char *nom_f;
int fig1, coul;
double x1, y1, x2, y2;
{
    FILE *f;

    /* Operations sur le fichier nom_f */
    if ((f = fopen (nom_f, "a")) == NULL) {
	printf ("Can't open file %s for appending.\n", nom_f);
	return -1;
    }

    fprintf (f, "%.2lf %.2lf %.2lf %.2lf\n", 
	min(x1,x2), min(y1,y2), max(x1,x2), max(y1,y2));

    CO_DRAW_MODE_CPY;
    CO_DRAW_COLOR (coul);
    CO_DRAW_RECT (fig1, x1, y1, x2, y2);

    /* Fermeture */
    fclose (f);
    return 0;
}


/*****************************************************************************
 * Deplace dans le fichier nom_f le rect sur la ligne cn 
 * aux nouvelles coordonnees x1, y1, x2, y2.
 *
 * Renvoie -1 en cas d'erreur, 0 en cas de succes.
 *****************************************************************************/
int ex7b_deplace_rect (nom_f, fig1, x1, y1, x2, y2, cn)
char *nom_f;
int fig1;
double x1, y1, x2, y2;
int cn;
{
    FILE *f, *g;
    char ligne[LG_LIGNE], nom_g[256];
    int n = -1;

    /* Operations sur fichiers */
    sprintf (nom_g, "%s%%", nom_f);
    if (rename (nom_f, nom_g) != 0) {
	printf ("Can't rename file %s as %s.\n", nom_f, nom_g);
	return -1;
    }
    if ((g = fopen (nom_g, "r")) == NULL) {
	printf ("Can't open file %s for reading.\n", nom_g);
	return -1;
    }
    if ((f = fopen (nom_f, "w")) == NULL) {
	printf ("Can't open file %s for writing.\n", nom_f);
	fclose (g);
	return -1;
    }

    /* Recopie des points de g dans f, sauf le point cn qui est modifie' */
    while ( fgets (ligne, LG_LIGNE, g) != NULL ) {
	if (++n != cn)
	     fputs (ligne, f); 
	else fprintf (f, "%.2lf %.2lf %.2lf %.2lf\n", 
		min(x1,x2), min(y1,y2), max(x1,x2), max(y1,y2));
    }

    /* m.a.j et fermetures */
    CO_FIG_REFRESH (fig1);
    fclose (f); fclose (g);
    return 0;
}

/*****************************************************************************
 * Supprime dans le fichier nom_f le rectangle de la ligne cn.
 *
 * Renvoie -1 en cas d'erreur, 0 en cas de succes.
 *****************************************************************************/
int ex7b_supprime_rect (nom_f, fig1, cn)
char *nom_f;
int fig1;
int cn;
{
    FILE *f, *g;
    char ligne[LG_LIGNE], nom_g[256];
    int n = -1;

    /* Operations sur fichiers */
    sprintf (nom_g, "%s%%", nom_f);
    if (rename (nom_f, nom_g) != 0) {
	printf ("Can't rename file %s as %s.\n", nom_f, nom_g);
	return -1;
    }
    if ((g = fopen (nom_g, "r")) == NULL) {
	printf ("Can't open file %s for reading.\n", nom_g);
	return -1;
    }
    if ((f = fopen (nom_f, "w")) == NULL) {
	printf ("Can't open file %s for writing.\n", nom_f);
	fclose (g);
	return -1;
    }

    /* Recopie des points de g dans f, suppression de la ligne cn */
    while ( fgets (ligne, LG_LIGNE, g) != NULL ) {
	if (++n != cn)
	    fputs (ligne, f); 
    }

    /* m.a.j et fermetures */
    CO_FIG_REFRESH (fig1);
    fclose (f); fclose (g);
    return 0;
}


/*****************************************************************************
 * Redessine l'ensemble des rectangles du fichier nom_f.
 *
 * Renvoie -1 en cas d'erreur, 0 en cas de succes.
 *****************************************************************************/
int ex7b_redessine (nom_f, fig1, coul)
char *nom_f;
int fig1, coul;
{
    FILE *f;
    char ligne[LG_LIGNE];
    double x1, y1, x2, y2;

    /* Ouverture de nom_f */
    if ((f = fopen (nom_f, "r")) == NULL) {
	printf ("Can't open file %s for reading.\n", nom_f);
	/* Creation de nom_f */
	f = fopen (nom_f, "w");
	printf ("Trying to create %s ... %s\n", nom_f, 
	    (f == NULL) ? "failed" : "success");
	fclose (f);
	return -1;
    }

    CO_DRAW_MODE_CPY;
    CO_DRAW_COLOR (coul);

    /* Redessine les rectangles */
    while ( fgets (ligne, LG_LIGNE, f) != NULL ) {
	if (sscanf (ligne, "%lf %lf %lf %lf", &x1, &y1, &x2, &y2) < 4) continue;
	CO_DRAW_RECT (fig1, x1, y1, x2, y2);
    }

    /* fermeture */
    fclose (f); 
    return 0;
}




