/******************************************************************************/
/* exemple7c.c                       30/04/96 - Edouard.Thiel@lim.univ-mrs.fr */
/*                                                                            */
/* GESTION D'UNE LISTE DE POINTS NUMEROTES dans la section FIG                */
/******************************************************************************/

/* 
 * Le include suivant est OBLIGATOIRE dans tous vos .c 
*/ 
#include "../IpsDir/include/tel_oper.h"


/* Definitions */
#define LG_LIGNE 256

/* Fonctions du module */
void ex7c_fig_points ();
int ex7c_cherche_clic ();
int ex7c_ajoute_point ();
int ex7c_deplace_point ();
int ex7c_supprime_point ();
int ex7c_redessine ();


/*****************************************************************************
 * Callback pour la section FIG.
 *
 * fig1 est le numero de la figure
 * par1 est le numero du parametre TEXT pour le nom du fichier de points
 * par2 est le numero du parametre INT pour le label du point a inserer.
 *****************************************************************************/
void ex7c_fig_points (p, fig1, par1, par2)
Tel_oper_p *p;
int fig1, par1, par2;
{
    /* Donnees devant exister en permanence */
    enum { E_None, E_Add, E_Suppr, E_Move };
    static int S_etat, S_n;
    static double S_x, S_y;

    /* Variables */
    char *nom_f = CO_GET_TEXT (par1);

    switch (p->fig_event) {

	case FIG_PUSH :
	    /* Memorise coordonnees du clic */
	    S_x = p->fig_x; S_y = p->fig_y;

	    /* Cherche le numero du point clique' */
	    S_n = ex7c_cherche_clic (nom_f, S_x, S_y, p->fig_zoom);

	    /* En deduit S_etat */
	    S_etat = (p->fig_butt == 1)
		? (S_n < 0) ? E_Add  : E_Move
		: (S_n < 0) ? E_None : E_Suppr;

	    /* Action selon S_etat */
	    switch (S_etat) {
		case E_Move :
		    CO_DRAW_MODE_XOR;
		    CO_DRAW_CURSOR (fig1, S_x, S_y);
		    break;
		case E_Suppr :
		    ex7c_supprime_point (nom_f, fig1, S_n);
		    CO_SET_INT (par2, S_n);
		    break;
		case E_Add : {
		    int n = CO_GET_INT(par2), m;
		    m = ex7c_ajoute_point (nom_f, fig1, S_x, S_y, n);
		    if (m >= 0) CO_SET_INT (par2, m+1);
		  } break;
		case E_None :
		    CO_FIG_CENTER (fig1, S_x, S_y);
		    break;
	    }
	    break;

	case FIG_DRAG : if (S_etat == E_Move) {
	    CO_DRAW_CURSOR (fig1, S_x, S_y);
	    S_x = p->fig_x; S_y = p->fig_y;
	    CO_DRAW_CURSOR (fig1, S_x, S_y);
	  } break;

	case FIG_RELEASE : if (S_etat == E_Move) {
	    CO_DRAW_CURSOR (fig1, S_x, S_y);
	    ex7c_deplace_point (nom_f, fig1, S_x, S_y, S_n);
	  } break;

	case FIG_CANCEL : if (S_etat == E_Move) {
	    CO_DRAW_CURSOR (fig1, S_x, S_y);
	  } break;

	case FIG_REFRESH :
	    ex7c_redessine (nom_f, fig1);
	    break;
    }
}


/*****************************************************************************
 * Cherche dans le fichier nom_f le premier point "proche" de cx,cy
 *
 * Renvoie -1 en cas d'erreur, >=0 le label du point trouve'.
 *****************************************************************************/
int ex7c_cherche_clic (nom_f, cx, cy, zoom)
char *nom_f;
double cx, cy;
int zoom;
{
    FILE *f;
    char ligne[LG_LIGNE];
    double x, y, d = 5.0 / (double) zoom;
    int n;

    /* Ouverture du fichier nom_f */
    if ((f = fopen (nom_f, "r")) == NULL) {
	printf ("File %s does not exist.\n", nom_f);
	return -1;
    }

    /* Parcours du fichier nom_f */
    while ( fgets (ligne, LG_LIGNE, f) != NULL ) {
	if (sscanf (ligne, "%lf %lf %d", &x, &y, &n) < 3) continue;
	if (fabs(cx - x) < d && fabs(cy - y) < d) {
	    fclose (f);
	    return n; /* On a trouve le point, son label est n */
	}
    }

    /* Point non trouve' */
    fclose (f);
    return -1;
}


/*****************************************************************************
 * Ajoute dans le fichier nom_f le point cx,cy de label cn.
 * Si cn est inferieur au plus grand label trouve dans le fichier, alors
 * tous les points superieurs sont renumerotes.
 * Sinon, cn est abaisse' au premier label libre (i.e le suivant du + grand).
 *
 * Le fichier nom_f est donc toujours ordonne sur les labels, et ces labels
 * se suivent en commencant par 0.
 *
 * Renvoie -1 en cas d'erreur, sinon le label effectivement inse're'.
 *****************************************************************************/
int ex7c_ajoute_point (nom_f, fig1, cx, cy, cn)
char *nom_f;
int fig1;
double cx, cy;
int cn;
{
    FILE *f, *g;
    char ligne[LG_LIGNE], nom_g[256];
    double x, y;
    int n, res, k = -1, test = FAUX;

    /* Operations sur fichiers */
    sprintf (nom_g, "%s%%", nom_f);
    if (rename (nom_f, nom_g) != 0) {
	printf ("Can't rename file %s as %s.\n", nom_f, nom_g);
	return -1;
    }
    if ((g = fopen (nom_g, "r")) == NULL) {
	printf ("Can't open file %s for reading.\n", nom_g);
	return -1;
    }
    if ((f = fopen (nom_f, "w")) == NULL) {
	printf ("Can't open file %s for writing.\n", nom_f);
	fclose (g);
	return -1;
    }

    /* Recopie de g sur f des labels < cn */
    while ( fgets (ligne, LG_LIGNE, g) != NULL ) {
	if (sscanf (ligne, "%lf %lf %d", &x, &y, &n) < 3) continue;
	if (n < cn)
	     { fputs (ligne, f); k = n; }
	else { test = VRAI; break; }
    }

    /* Insertion du point cx,cy avec le label res */
    res = ++k;
    fprintf (f, "%.2lf %.2lf %d\n", cx, cy, k);
    if (test) {
	fprintf (f, "%.2lf %.2lf %d\n", x, y, ++k);
	CO_FIG_REFRESH (fig1);
    }
    else CO_DRAW_LABEL (fig1, cx, cy, k);

    /* Recopie du reste de g dans f, avec m.a.j des labels */
    while ( fgets (ligne, LG_LIGNE, g) != NULL ) {
	if (sscanf (ligne, "%lf %lf", &x, &y) < 2) continue;
	fprintf (f, "%.2lf %.2lf %d\n", x, y, ++k);
    }

    /* Fermetures */
    fclose (f); fclose (g);
    return res;
}


/*****************************************************************************
 * Deplace dans le fichier nom_f le point de label cn 
 * aux nouvelles coordonnees cx,cy.
 *
 * Renvoie -1 en cas d'erreur, 0 en cas de succes.
 *****************************************************************************/
int ex7c_deplace_point (nom_f, fig1, cx, cy, cn)
char *nom_f;
int fig1;
double cx, cy;
int cn;
{
    FILE *f, *g;
    char ligne[LG_LIGNE], nom_g[256];
    double x, y;
    int n;

    /* Operations sur fichiers */
    sprintf (nom_g, "%s%%", nom_f);
    if (rename (nom_f, nom_g) != 0) {
	printf ("Can't rename file %s as %s.\n", nom_f, nom_g);
	return -1;
    }
    if ((g = fopen (nom_g, "r")) == NULL) {
	printf ("Can't open file %s for reading.\n", nom_g);
	return -1;
    }
    if ((f = fopen (nom_f, "w")) == NULL) {
	printf ("Can't open file %s for writing.\n", nom_f);
	fclose (g);
	return -1;
    }

    /* Recopie des points de g dans f, sauf le point cn qui est modifie' */
    while ( fgets (ligne, LG_LIGNE, g) != NULL ) {
	if (sscanf (ligne, "%lf %lf %d", &x, &y, &n) < 3) continue;
	if (n != cn)
	     fputs (ligne, f); 
	else fprintf (f, "%.2lf %.2lf %d\n", cx, cy, cn);
    }

    /* m.a.j et fermetures */
    CO_FIG_REFRESH (fig1);
    fclose (f); fclose (g);
    return 0;
}

/*****************************************************************************
 * Supprime dans le fichier nom_f le point de label cn.
 * Les autres points sont renumerotes.
 *
 * Renvoie -1 en cas d'erreur, 0 en cas de succes.
 *****************************************************************************/
int ex7c_supprime_point (nom_f, fig1, cn)
char *nom_f;
int fig1;
int cn;
{
    FILE *f, *g;
    char ligne[LG_LIGNE], nom_g[256];
    double x, y;
    int n;

    /* Operations sur fichiers */
    sprintf (nom_g, "%s%%", nom_f);
    if (rename (nom_f, nom_g) != 0) {
	printf ("Can't rename file %s as %s.\n", nom_f, nom_g);
	return -1;
    }
    if ((g = fopen (nom_g, "r")) == NULL) {
	printf ("Can't open file %s for reading.\n", nom_g);
	return -1;
    }
    if ((f = fopen (nom_f, "w")) == NULL) {
	printf ("Can't open file %s for writing.\n", nom_f);
	fclose (g);
	return -1;
    }

    /* Recopie des points de g dans f, suppression de cn, renumerotation */
    while ( fgets (ligne, LG_LIGNE, g) != NULL ) {
	if (sscanf (ligne, "%lf %lf %d", &x, &y, &n) < 3) continue;
	if (n < cn)
	     fputs (ligne, f); 
	else if (n > cn)
	    fprintf (f, "%.2lf %.2lf %d\n", x, y, n-1);
    }

    /* m.a.j et fermetures */
    CO_FIG_REFRESH (fig1);
    fclose (f); fclose (g);
    return 0;
}


/*****************************************************************************
 * Redessine l'ensemble des points du fichier nom_f avec leurs labels.
 *
 * Renvoie -1 en cas d'erreur, 0 en cas de succes.
 *****************************************************************************/
int ex7c_redessine (nom_f, fig1)
char *nom_f;
int fig1;
{
    FILE *f;
    char ligne[LG_LIGNE];
    double x, y;
    int n;

    /* Ouverture de nom_f */
    if ((f = fopen (nom_f, "r")) == NULL) {
	printf ("Can't open file %s for reading.\n", nom_f);
	/* Creation de nom_f */
	f = fopen (nom_f, "w");
	printf ("Trying to create %s ... %s\n", nom_f, 
	    (f == NULL) ? "failed" : "success");
	fclose (f);
	return -1;
    }

    /* Redessine les points */
    while ( fgets (ligne, LG_LIGNE, f) != NULL ) {
	if (sscanf (ligne, "%lf %lf %d", &x, &y, &n) < 3) continue;
	CO_DRAW_LABEL (fig1, x, y, n);
    }

    /* fermeture */
    fclose (f); 
    return 0;
}

