/*****************************************************************************/
/* tel_oper.c                                      I.P.S  v 3.8a  27/06/1996 */
/*                                                                           */
/*                                             Edouard.Thiel@lim.univ-mrs.fr */
/*****************************************************************************/
/* MODULE D'APPEL DES OPERATEURS                                             */
/*                                                Lecons pour le menu d'aide */
/*****************************************************************************/

/*
 * Ce include appelle tous les .h qu'il faut, y compris math.h.
 * Toutes les macro-commandes telles que CO_GET_INT y sont definies.
 * N'hesitez pas a le lire pour avoir la liste et la syntaxe des macros.
*/
#include "../IpsDir/include/tel_oper.h"


CO_BEGIN_CALL_OPER

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExCopie")

  /* 
   *	1. IMAGES EN ENTREE ET SORTIE
   *       --------------------------
   *
   * Operateur "ExCopie" : 1 image en entree, 1 image en sortie
   *
   *   L'operateur de copie est d'abord declare' dans son menu par la ligne 
   *   suivante dans le fichier tel_oper.menu :
   *     OPER  ExCopie  "Copie"
   *   La chaine "Copie" est le nom de l'operateur dans le menu ; ExCopie est 
   *   l'identificateur de l'operateur, en argument de CO_BEGIN_CONNECT.
   *
   *   La vie de l'operateur se joue ici, entre CO_BEGIN_CONNECT ("ExCopie")
   *   et CO_END_CONNECT.
   *   La boite de parametres est declaree dans CO_BEGIN_DEF .. CO_END_DEF.
   *   L'appel de la fonction ex1_copie se fait dans CO_BEGIN_EXE .. CO_END_EXE.
  */

      CO_BEGIN_DEF
      /* 
       * Declaration de la boite de parametres : l'icone s'appelle "Copie" ;
       * il y a 1 image en entree , l'original, et 1 en sortie, la copie.
       *
       * REGLES :
       * - les images sont numerotees de 1 a 31, A PARTIR DE 1.
       * - seules les images en sortie (_OUT) sont modifiables par l'operateur.
       * - les images en sortie heritent des legende ; ici "Une copie".
      */
	IPS_NAME,	"Copie",
	IPS_IMAGE_IN,	1, "Original",
	IPS_IMAGE_OUT,	2, "Une copie",
      CO_END_DEF

      CO_BEGIN_EXE
      /*
       * Partie appelee quand on fait "Run" sur l'icone.
       * La fonction ex1_copie est dans exemple1.c ; voir les commen-
       * taires dans ce fichier. Les 2 images et leur taille sont transmises.
      */
	ex1_copie (p->pix[1], p->pix[2], p->xsiz, p->ysiz );
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExSymVert") 

  /*
   * Operateur "ExSymVert" : 1 entree, 1 sortie
  */

      CO_BEGIN_DEF
	IPS_NAME,	"Symetrie Verticale",
	IPS_IMAGE_IN,	1, "Original",
	IPS_IMAGE_OUT,	2, "Sym Vert.",
      CO_END_DEF

      CO_BEGIN_EXE
	ex1_sym_vert (p->pix[1], p->pix[2], p->xsiz, p->ysiz );
      CO_END_EXE

  CO_END_CONNECT


  /*
   * EXERCICE : realiser une symetrie horizontale
   *
   *		ATTENTION les exercices sont a realiser
   *		  dans VOTRE repertoire ~/pips/perso1
   *
   * - Editer votre tel_oper.menu, declarer l'operateur ExSymHoriz
   * - Faire [Extra] [Reread menus] pour mettre a jour les menus de IPS
   * - Dans votre tel_oper.c, recopier le bloc 
   *     CO_BEGIN_CONNECT ("ExSymHoriz") ... CO_END_CONNECT
   * - Dans votre perso1/ creer le fichier my_symhoriz.c a partir de exemple1.c
   * - Declarer my_symhoriz.o dans votre Makefile
   * - Compiler en faisant : make comp
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExCompar") 

  /*
   * Operateur "ExCompar" : 2 entrees, 1 sortie
  */

      CO_BEGIN_DEF
	IPS_NAME,	"Comparaison",
	IPS_IMAGE_IN,	1, "",
	IPS_IMAGE_IN,	2, "",
	IPS_IMAGE_OUT,	3, "Compar",
      CO_END_DEF

      CO_BEGIN_EXE
	ex1_compar (p->pix[1], p->pix[2], p->pix[3], p->xsiz, p->ysiz );
      CO_END_EXE

  CO_END_CONNECT


  /*
   * EXERCICE : realiser ExComparBool avec les specifications suivantes. 
   *            Soit p1 = ima1(x,y) , p2 = ima2(x,y) , p3 = ima3(x,y)
   *
   * - si p1 = 0 et p2 = 0, alors p3 = 0
   * - si p1 > 0 et p2 > 0, alors p3 = 7
   * - si p1 > 0 et p2 = 0, alors p3 = 10
   * - si p1 = 0 et p2 > 0, alors p3 = 12
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExMinMax") 

  /*
   * Operateur "ExMinMax" : 2 entrees, 2 sorties
  */

      CO_BEGIN_DEF
	IPS_NAME,	"Min Max",
	IPS_IMAGE_IN,	1, "",
	IPS_IMAGE_IN,	2, "",
	IPS_IMAGE_OUT,	3, "Min",
	IPS_IMAGE_OUT,	4, "Max",
      CO_END_DEF

      CO_BEGIN_EXE
	ex1_minmax (p->pix[1], p->pix[2], p->pix[3], p->pix[4], 
		p->xsiz, p->ysiz ); 
      CO_END_EXE

  CO_END_CONNECT


  /*
   * EXERCICE : faire moyenne, median ((max-min)/2).
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExSeuil") 

  /*
   *	2. PARAMETRES
   *       ----------
   *
   * Operateur "ExSeuil" : 1 entree, 1 sortie, 1 parametre entier
  */

      CO_BEGIN_DEF
      /* 
       * Declaration du parametre entier numero 1, de legende "Seuil :",
       * de valeur min = 0, max = 65535, defaut = 127.
       *
       * REGLES :
       * - les parametres sont numerotes de 1 a 31, A PARTIR DE 1.
      */
	IPS_NAME,	"Seuillage",
	IPS_IMAGE_IN,	1, "Original",
	IPS_IMAGE_OUT,	2, "Seuille'",
	IPS_PAR_INT,    1, "seuil :", 0, 65535, 127,
      CO_END_DEF

      CO_BEGIN_EXE
      /* 
       * On peut declarer des variables dans _EXE ; ici seuil.
       * Le parametre entier 1 est retrouve avec CO_GET_INT (1).
       *
       * REGLES :
       * - les parametres sont numerotes a partir de 1
      */
	int seuil = CO_GET_INT (1);
	printf ("Le seuil est %d\n", seuil);
	ex2_seuil (p->pix[1], p->pix[2], p->xsiz, p->ysiz, seuil );
      CO_END_EXE

  CO_END_CONNECT


  /*
   * EXERCICE : mettre un slider (potentiometre)
   *
   * - remplacer IPS_PAR_INT par IPS_PAR_SLIDER
   * - c'est tout !
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExShift") 

  /* 
   * Operateur "ExShift" : 1 entree, 1 sortie, 2 parametres entiers
  */

      CO_BEGIN_DEF
	IPS_NAME,	"Shift",
	IPS_IMAGE_IN,	1, "Original",
	IPS_IMAGE_OUT,	2, "Shift",
	IPS_PAR_INT,    1, "decalage en x :", 0, 16384, 10,
	IPS_PAR_INT,    2, "decalage en y :", 0, 16384, 20,
      CO_END_DEF

      CO_BEGIN_EXE
	int dx = CO_GET_INT (1);
	int dy = CO_GET_INT (2);
	ex2_shift (p->pix[1], p->pix[2], p->xsiz, p->ysiz, dx, dy );
      CO_END_EXE

  CO_END_CONNECT


  /*
   * EXERCICE : realiser un 'bas relief'
   *
   * - Faire la difference entre l'image origine et l'image decalee.
   *   La translation doit etre petite : choisissez (1,2) par defaut.
   * - Rehausser cette difference en ajoutant 127
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExAffine") 

  /* 
   * Operateur "ExAffine" : 1 entree, 1 sortie, 1 par. entier, 1 par. reel
  */

      CO_BEGIN_DEF
      /* 
       * Declaration du parametre reel numero 1, de legende "MUL :",
       * de valeur min = -10000, max = 10000, defaut = 1.5.
       *
       * REGLE : les valeurs pour IPS_PAR_REAL sont mis entre ""
      */
	IPS_NAME,	"Trans Affine",
	IPS_IMAGE_IN,	1, "Original",
	IPS_IMAGE_OUT,	2, "Affine",
	IPS_PAR_REAL,   1, "MUL :", "-10000", "10000", "1.5",
	IPS_PAR_INT,    2, "ADD", -65535, 65535, 5,
      CO_END_DEF

      CO_BEGIN_EXE
	/*
         * CO_GET_REAL() renvoie un double.
        */
	double fois = CO_GET_REAL (1);
	int    plus = CO_GET_INT (2);
	ex2_affine (p->pix[1], p->pix[2], p->xsiz, p->ysiz, fois, plus );
      CO_END_EXE

  CO_END_CONNECT


  /*
   * EXERCICE : faire une transformation de type a * ima1(x,y) / b + c
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExAllZeParams") 

  /* 
   * Tous les parametres disponibles : declaration et lecture.
   *
  */

     CO_BEGIN_DEF
      /* 
       * La syntaxe depend de la categorie de parametre ; num commence a 1.
       *
       * IPS_PAR_INT, num, "legende", min, max, defaut,
       *     idem pour IPS_PAR_SLIDER.
       *
       * IPS_PAR_REAL, num, "legende", "min", "max", "defaut",
       *     attention les valeurs sont entre ""
       *
       * IPS_PAR_TEXT, num, "legende", longueur_max, "defaut",
       *     prevoir longueur_max suffisante pour chemins de fichiers (> 80).
       *
       * IPS_PAR_CHOICE, num, "legende", defaut, "mot0", "mot1", ..., NULL,
       *     idem pour IPS_PAR_CH_ABR ; la valeur defaut commence a 0 ;
       *     la liste de mots est terminee par NULL.
       *
       * IPS_PAR_CHK_BOX, num, "legende", defaut,
       *     defaut vaut 0 ou 1.
      */
	IPS_NAME,	"Tous les parametres",
	IPS_PAR_INT,    1, "Valeur entiere :", 0, 1024, 256,
	IPS_PAR_REAL,   2, "Valeur reelle :", "-6.3", "6.3", "3.14159",
	IPS_PAR_TEXT,   3, "Chaine de car :", 40, "noname.txt",
	IPS_PAR_CHOICE, 4, "Boutons Radio :", 0, "Europe 1", "RTL", "RMC", NULL,
	IPS_PAR_CH_ABR, 5, "Choix Abrege :", 2, "bijou", "caillou", "chou", 
				"genou", "hibou", NULL,
	IPS_PAR_SLIDER, 6, "Potentiometre :", 1, 9, 5,
	IPS_PAR_CHK_BOX,7, "Case a cocher :", 1,
      CO_END_DEF

      CO_BEGIN_EXE
	printf ("Voici les parametres\n");
	printf ("Valeur #1 = %d\n", CO_GET_INT     (1));
	printf ("Valeur #2 = %f\n", CO_GET_REAL    (2));
	printf ("Valeur #3 = %s\n", CO_GET_TEXT    (3));
	printf ("Valeur #4 = %d\n", CO_GET_CHOICE  (4));
	printf ("Valeur #5 = %d\n", CO_GET_CH_ABR  (5));
	printf ("Valeur #6 = %d\n", CO_GET_SLIDER  (6));
	printf ("Valeur #7 = %d\n", CO_GET_CHK_BOX (7));
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExFibonacci") 

  /* 
   * Modification de la valeur de parametres depuis la section EXE.
   *
   * Cet operateur calcule la suite de Fibonacci, qui tend vers le 
   * nombre d'or (1 + sqrt(5)) / 2.
  */

     CO_BEGIN_DEF
	IPS_NAME,	"Fibonnaci nombre d'or",
	IPS_PAR_INT,    1, "Rang de la suite :", 1, 1024, 1,
	IPS_PAR_INT,    2, "U(i) =", 1, 1000000000, 1,
	IPS_PAR_INT,    3, "U(i+1) =", 1, 1000000000, 1,
	IPS_PAR_TEXT,	4, "Fraction :", 40, "1 / 1",
	IPS_PAR_REAL,   5, "Nombre d'or ~", "0", "10", "1",
      CO_END_DEF

      CO_BEGIN_EXE
	/*
	 * On recupere tous les parametres
	*/
	int    rang  = CO_GET_INT  (1);
	int    ui    = CO_GET_INT  (2);
	int    uii   = CO_GET_INT  (3);
	char   *fra  = CO_GET_TEXT (4), blabla[40];
	double nb_or = CO_GET_REAL (5);

	/* 
	 * L'objectif est de montrer comment on peut modifier les parametres
	 * depuis la section EXE.
	 *
	 * La syntaxe est : CO_SET_XXX (num, valeur);
	 * avec XXX = REAL, TEXT, INT, SLIDER, CHK_BOX, CH_ABR, CHOICE. 
	 * num est le numero du parametre.
	 * valeur est la nouvelle valeur a assigner, de type dependant de XXX :
	 * double pour REAL, char* pour TEXT, int pour les autres.
	*/
	CO_SET_INT  (1, rang+1);
	CO_SET_INT  (2, uii);
	CO_SET_INT  (3, ui+uii);
	sprintf (blabla, "%d / %d", CO_GET_INT(3), CO_GET_INT(2) );
	CO_SET_TEXT (4, blabla);
	CO_SET_REAL (5, (double) CO_GET_INT(3) / (double) CO_GET_INT(2) );

	/*
	 * Remarque : CO_SET_ (n,v) change IMMEDIATEMENT la valeur du param n,
	 * qui est stocke' LOCALEMENT (quelque part dans p-> ).
	 * Par contre, la mise a jour au niveau de IPS est DIFFEREE, elle se
	 * fait en une seule fois APRES la fin de la section EXE.
	*/
      CO_END_EXE

  CO_END_CONNECT


  /*
   * EXERCICE : calculer la somme (1 / n^2) qui tend vers (PI^2 / 6).
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExSecChoice") 

  /* 
   *	3. ACTIONS DANS LES BOITES DE PARAMETRES
   *       -------------------------------------
   * 
   *  D'ordinaire il faut faire un [Apply] ou un [Apply & Run] dans une boite
   *  de parametres d'IPS pour que les nouvelles valeurs des parametres soient
   *  prises en compte par IPS, si vous les avez change' a la main.
   * 
   *  Les parametres menus et cases a cocher respectent ce principe, mais 
   *  offrent une possibilite supplementaire : traiter votre action AU VOL, 
   *  par exemple pour griser des parametres selon les valeurs d'un menu.
   * 
   *  Pour chaque parametre menu/case a cocher, a l'initialisation de la boite,
   *  et pour chaque action dans la boite, la section CHOICE est appelee
   *  (au besoin en relancant l'operateur si le process est mort).
   *
   *  Bien entendu, la section CHOICE est OPTIONNELLE !
  */

     CO_BEGIN_DEF
	IPS_NAME,	"Action choice",
	IPS_PAR_CHOICE,  1, "Boutons Radio :", 0, 
				"Europe 1", "RTL", "RMC", NULL,
	IPS_PAR_CH_ABR,  2, "Choix Abrege :", 2, "bijou", "caillou", "chou", 
				"genou", "hibou", NULL,
	IPS_PAR_CHK_BOX, 3, "Case a cocher :", 1,
      CO_END_DEF

      CO_BEGIN_CHOICE
	/*
	 * Appele pour chaque action dans IPS_PAR_CHOICE, _CH_ABR, _CHK_BOX.
	 *
	 * Les 3 valeurs p->chce_num, p->chce_val et p->chce_ini decrivent
	 * l'evenement qui a eu lieu :
	 *
	 * p->chce_num est le numero du parametre sur lequel a lieu l'action.
	 *
	 * p->chce_val est la nouvelle valeur que prend ce parametre a cause
	 * de l'action (mais elle n'a pas ete valide'e par un [APPLY]).
	 *
	 * p->chce_ini est un booleen, qui vaut 1 pendant la phase 
	 * d'initialisation de l'operateur, et 0 quand une action a lieu 
	 * dans la boite de parametres.
	 * En effet, lors de la section DEF, chaque parametre de type CHOICE,
	 * CH_ABR ou CHK_BOX genere un appel de la section CHOICE pour initi-
	 * aliser ce qu'on veut (on verra l'utilite de ce mecanisme plus loin).
	 */
	printf ("-- CHOICE  ini = %d  num = %d  val = %d\n", 
	    p->chce_ini, p->chce_num, p->chce_val);
      CO_END_CHOICE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExGriser") 

  /* 
   * Activation/inactivation de parametres en les grisant au vol.
  */

     CO_BEGIN_DEF
	IPS_NAME,	"Griser",
	IPS_PAR_INT,     1, "entier 1 :", -100, 100, 5,
	IPS_PAR_TEXT,	 2, "texte 2 :", 40, "Bonjour",
	IPS_PAR_REAL,    3, "reel 3 :", "0", "10", "3.14159",
	IPS_PAR_CHOICE,  4, "Cliquez moi :", 2, "degrise", "grise 1", "2", "3", 
				NULL,
	IPS_PAR_CHK_BOX, 5, "Montrer menu :", 1,
      CO_END_DEF

      CO_BEGIN_CHOICE
	/*
	 * Pour griser/degriser un parametre, la syntaxe est
	 * 
	 *    CO_SET_ACTIVE (num, activer)
	 * 
	 * ou num est le numero du parametre a griser/degriser, et
	 * activer est un booleen : 
	 *   VRAI pour rendre le parametre visible, FAUX pour le griser.
	 */
	int x;
	switch (p->chce_num) {
	    case 4 :
		for (x = 1; x <= 3; x++)
		    CO_SET_ACTIVE (x, x != p->chce_val);
		break;
	    case 5 :
		CO_SET_ACTIVE (4, p->chce_val);
		break;
	}
      CO_END_CHOICE

  CO_END_CONNECT


  /*
   * EXERCICE : faire un operateur qui calcule A*ima1(x,y) + B*ima2(x,y)
   *
   * - 2 images en entree, 1 en sortie
   * - 2 parametres reels A et B
   * - 1 parametre IPS_PAR_CHOICE valant "B = 1-A" ou "B libre"
   * - Si "B = 1-A" est choisi, le parametre B doit etre grise'.
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExModifCho")

  /* 
   * Lecture / modification de parametres au vol.
  */

     CO_BEGIN_DEF
	IPS_NAME,	"Modif au vol",
	IPS_PAR_SLIDER,  1, "Valeur :", 1, 20, 10,
	IPS_PAR_CHOICE,  2, "Operation :", 3, 
		"= 1", "-5", "-1", "+1", "+5", "= 20", NULL,
      CO_END_DEF

      CO_BEGIN_CHOICE
	/*
	 * Dans la section CHOICE, on a acces aux valeurs (validees par 
	 * un [Apply]) de tous les parametres, que l'on peut lire avec 
	 * CO_GET_ (num), exactement comme dans la section EXE.
	 *
	 * De meme, on peut modifier ces valeurs avec un CO_SET_ (num,val).
	 *
	 * On s'amuse donc ici a faire bouger un potentiometre :
	 */

	/* on recupere la valeur actuelle du potentiometre */
	int v = CO_GET_SLIDER (1);

	/* on modifie v en fonction de p->chce_val */
	switch (p->chce_val) {
	    case 0 : v = 1; break;
	    case 1 : v -= 5; if (v < 1) v = 1; break;
	    case 2 : v -= 1; if (v < 1) v = 1; break;
	    case 3 : v += 1; if (v > 20) v = 20; break;
	    case 4 : v += 5; if (v > 20) v = 20; break;
	    case 5 : v = 20; break;
	    default : printf ("Zut j'ai oublie' le cas %d\n", p->chce_val);
	}

	/* on donne une nouvelle valeur au potentiometre */
	CO_SET_SLIDER (1, v);

      CO_END_CHOICE

  CO_END_CONNECT


  /*
   * EXERCICE : variante de l'exercice de ExGriser. 
   *            Somme pondere'e A*ima1(x,y) + (1-A)*ima2(x,y)
   *
   * - 2 images en entree, 1 en sortie
   * - 1 parametres reel A
   * - 1 parametre slider de 0 a 100
   * - Lorsque le slider est actionne', il change A ( = slider / 100)
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExSetMask")

  /* 
   * Lecture / ecriture d'un masque dans un fichier.
   *
   * Il peut etre utile d'avoir un operateur specialise' qui ecrit un masque
   * dans un fichier, de telle sorte que d'autres operateurs lisent le masque
   * depuis ce fichier (au lieu de dupliquer le parametre "masque" partout).
  */

      CO_BEGIN_DEF
	IPS_NAME, "Fixer le masque" , 
	IPS_PAR_CH_ABR, 1, "Masque :", 3, 
		"d4", "d8", "chamfer 2-3", "chamfer 3-4", "chamfer 5-7-11",
		NULL,
      CO_END_DEF

      CO_BEGIN_CHOICE
	/*
	 * On veut eviter d'avoir a valider le choix du masque par un [Apply],
	 * c'est pourquoi on utilise la section CHOICE.
	 *
	 * On va lire ou ecrire le numero du masque dans un fichier. Si cette 
	 * valeur etait deja fixee dans un fichier, on veut la recuperer pour
	 * initialiser correctement le param 1.
	 * 
	 * C'est pourquoi on teste p->chce_ini :
	 * - si VRAI, c'est que on est pendant la phase d'initialisation de la
	 *   boite de params, et donc on lit dans le fichier et on met a jour
	 *   le param 1.
	 * - si FAUX, une action vient d'avoir lieu dans la boite, et on
	 *   ecrit dans le fichier la valeur p->chce_val.
	 */
	if (p->chce_ini)
	     CO_SET_CH_ABR (1, ex3_lire_masque());
	else ex3_fixer_masque (p->chce_val);
      CO_END_CHOICE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExLireMask")

      CO_BEGIN_DEF
	IPS_NAME, "Lire le masque" , 
      CO_END_DEF

      CO_BEGIN_EXE
	/*
	 * Cet operateur se contente de lire dans 1 fichier le numero de masque 
	 * qui a ete fixe' par l'operateur ExSetMask, et de l'afficher lorsque
	 * l'on fait un [Run].
	 */
	char noms[][20] = { "d4", "d8", "chamfer 2-3", 
				"chamfer 3-4", "chamfer 5-7-11" };
	int v = ex3_lire_masque();
	printf ("Le masque actif est le numero %d : %s\n", v, noms[v] );
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExTranspo") 

  /*
   *	4. TAILLE DES IMAGES
   *       -----------------
   * 
   *  Par defaut, les proprietes de taille (p->xsiz, p->ysiz) sont heritees de 
   *  la premiere image. 
   *
   *  Autrement dit, lorsqu'un operateur agit sur une selection d'images, 
   *  IPS va considerer la premiere image, et supposera que toutes les autres
   *  images doivent avoir la meme taille. 
   *  Si une image en entree (!= de la premiere) n'a pas cette taille, 
   *  IPS refusera d'executer l'operateur.
   *  Si une image en sortie n'a pas cette taille, il la modifiera 
   *  (en conservant le contenu dans la mesure du possible), ou la creera.
   * 
   *  On peut s'affranchir de ce mecanisme avec la section DIM.
   *
   *  Voici une illustration avec un operateur de transposition, qui pour 
   *  une image en entree de taille (a,b) a besoin d'une image en sortie 
   *  de taille (b,a).
  */

     CO_BEGIN_DEF
	IPS_NAME,	"Transpo sition",
	IPS_IMAGE_IN,	1, "Quelconque",
	IPS_IMAGE_OUT,	2, "Transposee",
      CO_END_DEF

      CO_BEGIN_DIM
	/*
	 *  La section DIM est appelee juste avant la section EXE, elle sert a
	 *  preparer les donnees de la section EXE. De la sorte, lorsqu'on 
	 *  arrive dans la section EXE, les images ont deja les bonnes tailles.
	 *
	 *  Les tailles des images sont rangees dans 2 vecteurs : 
	 *  p->ima_x[1..31] et p->ima_y[1..31]. 
	 *  L'image num. i a pour taille (p->ima_x[i], p->ima_y[i]).
	 *  
	 *  IPS regarde la premiere image selectionnee, et remplit ces vecteurs
	 *  avec p->ima_x[1] et p->ima_y[1].
	 *  
	 *  Si il n'y a pas de section DIM, ces tableaux sont inchanges, et 
	 *  donc les images devront avoir la meme taille (cas par defaut).
	 *  
	 *  Si il y a une section DIM, dans laquelle (p->ima_x[i], p->ima_y[i])
	 *  est modifie (par exemple p->ima_x[i] = p->ima_y[i] = 256), alors au
	 *  retour de la section DIM, IPS appliquera ce changement.
	*/
	p->ima_x[2] = p->ima_y[1];
	p->ima_y[2] = p->ima_x[1];
      CO_END_DIM

      CO_BEGIN_EXE
	/*
	 * On a TOUJOURS p->xsiz = p->ima_x[1] et p->ysiz = p->ima_y[1].
	 *
	 * La section DIM fait que p->ima_x[2] = p->ysiz et 
	 * p->ima_y[2] = p->xsiz.
	 * Dans cet operateur de transposition, il est inutile de tout passer
	 * a ex4_transpose, un couple de tailles suffit.
	*/
	ex4_transpose (p->pix[1], p->pix[2], p->xsiz, p->ysiz );
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExChgeDim") 

  /*
   *  Lecture / modification de parametres dans la section DIM
   */

     CO_BEGIN_DEF
	IPS_NAME,	"Change taille",
	IPS_IMAGE_OUT,	 1, "Coucou",
	IPS_PAR_INT,     1, "Taille x :", 8, 8192, 300,
	IPS_PAR_INT,     2, "Taille y :", 8, 8192, 200,
	IPS_PAR_TEXT,	 3, "Resultat :", 40, "Image y,x",
      CO_END_DEF

      CO_BEGIN_DIM
	/*
	 *  Cet operateur cree une image en sortie, de taille specifiee
	 *  avec les parametres 1 et 2. Le param 3 contient un message
	 *  apres execution.
	*/
	char bla[40];
	p->ima_x[1] = CO_GET_INT(1);
	p->ima_y[1] = CO_GET_INT(2);
	sprintf (bla, "Image %d,%d", CO_GET_INT(2), CO_GET_INT(1));
	CO_SET_TEXT (3, bla);
      CO_END_DIM

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExDesHerit") 

  /*
   *  Arriver dans la section DIM avec les tailles reelles des 
   *  images selectionnees
   */

     CO_BEGIN_DEF
	/*
	 *  Par defaut IPS fait heriter la taille de la premiere image aux 
	 *  autres images de la selection ; en fait IPS recopie 
	 *  (p->ima_x[1], p->ima_y[1]) dans tout (p->ima_x[], p->ima_y[]), 
	 *  puis va dans la section DIM.
	 *  
	 *  On peut desactiver ce mecanisme en rajoutant dans la section DEF la 
	 *  ligne :
	 *		IPS_SAME_SIZE, FAUX,
	*/
	IPS_NAME,	"Tailles reelles",
	IPS_IMAGE_IN,	 1, "Rectangulaire",
	IPS_IMAGE_IN,	 2, "Autre taille",
	IPS_IMAGE_OUT,	 3, "Min des tailles des 2 images",
	IPS_SAME_SIZE,   FAUX,
      CO_END_DEF

      CO_BEGIN_DIM
	/*
	 *  On arrive donc ici avec (p->ima_x[], p->ima_y[]) qui contient les
	 *  vraies tailles des images de la selection (tant pour les images
	 *  IN que pour les images OUT ; toutefois les images OUT qui sont crees
	 *  heritent quand meme de l'image 1).
	 *  
	 *  Cet operateur prend en entree 2 images rectangulaires de tailles
	 *  differentes (x1,y1) et (x2,y2), et fabrique en sortie une image 
	 *  de taille (x3,y3), avec x3 = MIN(x1,x2) et y3 = MIN (y1,y2).
	 *  
	 *  Si on n'avait pas mis IPS_SAME_SIZE a FAUX, on serait arrive' dans
	 *  DIM avec x2=x1 et y2=y1, et comme les 2 images IN avaient des 
	 *  tailles differentes, IPS aurait refuse' l'execution !
	*/
	p->ima_x[3] = (p->ima_x[1] < p->ima_x[2]) ? p->ima_x[1] : p->ima_x[2];
	p->ima_y[3] = (p->ima_y[1] < p->ima_y[2]) ? p->ima_y[1] : p->ima_y[2];
      CO_END_DIM

      CO_BEGIN_EXE
	printf ("x1 = %3d   y1 = %3d\n",   p->ima_x[1], p->ima_y[1]);
	printf ("x2 = %3d   y2 = %3d\n",   p->ima_x[2], p->ima_y[2]);
	printf ("x3 = %3d   y3 = %3d\n\n", p->ima_x[3], p->ima_y[3]);
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExSuperPos") 

  /*
   *  Utilisation de section DIM + section CHOICE + IPS_SAME_SIZE.
   *  
   *  Cet operateur permet de superposer ima1 sur ima2 avec transparence.
   *  
   *  ima1 et ima2 on des tailles quelconques ; le contenu de ima1 est
   *  superpose' au contenu actuel de ima2, a la position donnee (le coin 
   *  superieur gauche de ima1 est positionne' dans ima2).
   *  On peut conserver la taille actuelle de ima2, ou la fixer par parametres.
   *  On peut fixer la valeur de transparence pour ima1, ou supprimer la
   *  transparence.
   */

      CO_BEGIN_DEF
        IPS_NAME,       "Super position",
        IPS_IMAGE_IN,   1, "To be superposed over image 2",
        IPS_IMAGE_OUT,  2, "Superposition",
        IPS_PAR_INT,    1, "top left x coord", -8192, 16384, 0,
        IPS_PAR_INT,    2, "top left y coord", -8192, 16384, 0,
	IPS_PAR_CHOICE,	3, "Image 2 size", 0, "Keep", "Modify", NULL,
        IPS_PAR_INT,    4, "image 2 width ", 8, 8192, 512,
        IPS_PAR_INT,    5, "image 2 height", 8, 8192, 512,
	IPS_PAR_CHK_BOX,6, "Transparency", 0,
        IPS_PAR_INT,    7, "Transp. value", 0, 65535, 0,
	IPS_SAME_SIZE,	FAUX,  /* cf operateur precedent ExDesHerit */
      CO_END_DEF

      CO_BEGIN_CHOICE
	/*
	 *  Selon la case a cocher (param 6), on grise ou degrise le param 7
	 *  pour la valeur de transparence.
	 *  De meme, le param 3 dit si on modifie ou pas la taille de ima2 ;
	 *  on grise/degrise en consequence les params de taille 4 et 5.
	*/
	if (p->chce_num == 6)
	    CO_SET_ACTIVE (7, p->chce_val);
	else if (p->chce_num == 3) {
	    CO_SET_ACTIVE (4, p->chce_val);
	    CO_SET_ACTIVE (5, p->chce_val);
	}
      CO_END_CHOICE

      CO_BEGIN_DIM
	/*
	 *  Si le param 3 vaut "Modify", alors
	 *  on fixe la taille de ima2 a partir des params 4 et 5.
	*/
	if (CO_GET_CHOICE(3)) {
            p->ima_x[2] = CO_GET_INT(4);
            p->ima_y[2] = CO_GET_INT(5);
	}
      CO_END_DIM

      CO_BEGIN_EXE
        int xm1 = p->ima_x[1],
	    ym1 = p->ima_y[1],
	    xm2 = p->ima_x[2],
	    ym2 = p->ima_y[2],
            dx  = CO_GET_INT(1),
            dy  = CO_GET_INT(2),
	    tr  = (CO_GET_CHK_BOX(6)) ? CO_GET_INT(7) : -1;

	ex4_superpose (p->pix[1], p->pix[2], xm1, ym1, xm2, ym2, dx, dy, tr);
      CO_END_EXE
  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExLutEtNom") 

  /*
   *  Modification des tables de couleur (LUT) et des noms des images
   *  en sortie (IPS_IMAGE_OUT) dans la section DIM.
   *
   *  Le changement de LUT est tres utile par exemple quand vous avez en
   *  entree une image en niveaux de gris (64G), et que vous voulez visualiser
   *  un resultat en fausses couleurs (16C) tel que contour, squelette,
   *  ou par des couleurs en sur-impression (BWC).
   *
   *  Le changement de nom est utile par exemple quand vous travaillez sur
   *  un fichier de donnees et que vous voulez montrer son nom ; un autre
   *  exemple est un operateur qui prend des parametres numeriques, et vous
   *  voulez faire apparaitre ces parametres avec l'image du resultat.
  */

      CO_BEGIN_DEF
        IPS_NAME,       "Change LUT et nom",
        IPS_IMAGE_IN,   1, "Quelconque",
        IPS_IMAGE_OUT,  2, "LUT et nom modifies",
	IPS_PAR_CH_ABR, 1, "LUT :", 0, "16 couleurs", "niv gris",
		"16 coul + gris", NULL,
	IPS_PAR_TEXT,   2, "Nom :", 64, "Coucou",
      CO_END_DEF

      CO_BEGIN_DIM
	/*
	 *  Les images en sortie heritent 
	 *   - de la table de couleur de l'image IN 1
	 *   - les legendes des images comme nouveau nom
	 *  
	 *  On peut court-circuiter ce mecanisme dans la section DIM :
	 *  Pour changer la LUT de l'image OUT num : CO_SET_xxx (num) 
	 *  avec xxx = 16C, 64G ou BWC.
	 *  Pour changer le nom de l'image OUT num : CO_SET_NAME (num, nom).
	*/
	switch (CO_GET_CH_ABR (1)) {
	    case 0 : CO_SET_16C (2); break;
	    case 1 : CO_SET_64G (2); break;
	    case 2 : CO_SET_BWC (2); break;
	}
	CO_SET_NAME (2, CO_GET_TEXT (2));
      CO_END_DIM

      CO_BEGIN_EXE
	/* On fait une simple copie pour illustrer les effets de section DIM */
	ex1_copie (p->pix[1], p->pix[2], p->xsiz, p->ysiz );
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExErosion") 

  /*
   *	5. BORD EXTERIEUR DES IMAGES
   *       -------------------------
   * 
   *  Vous avez la possibilite de demander a IPS de rajouter un bord autour 
   *  des images ; la documentation est accessible par le premier bouton
   *  du menu "Le bord exterieur des images".
   *
   *  Voici une illustration avec un operateur d'erosion. La fonction a
   *  besoin d'un bord d'epaisseur 1, on met donc dans la section DEF 
   *  la ligne :  IPS_BORDER, 1,
  */

      CO_BEGIN_DEF
	IPS_NAME, "Erosion" , 
	IPS_IMAGE_IN , 1 , "Binary" , 
	IPS_IMAGE_OUT, 2 , "Erosion" , 
	IPS_PAR_CHOICE, 1, "Foreground", 1, "4 connect", "8 connect", NULL,
	IPS_BORDER, 1,
      CO_END_DEF

      CO_BEGIN_EXE
	int c = (CO_GET_CHOICE(1) == 0) ? 4 : 8;  /* connexite */

	ex5a_erosion ( p->pix[1], p->pix[2],
	    p->cmin, p->xmax, p->ymax, p->xtot, p->xytot, c);
       CO_END_EXE

  CO_END_CONNECT


  /*
   *  EXERCICE : faire une dilatation (i.e une erosion du complementaire).
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExObjectLabel") 

  /*
   *  Prend en entree une image binaire, et donne une couleur differente 
   *  a chaque objet.
   *
   *  La fonction d'etiquetage a besoin d'un bord d'epaisseur quelconque >= 1.
   *  On met donc dans la section DEF :     IPS_BORDER_POSIT,
   *  NB. On aurait pu mettre a la place :  IPS_BORDER, 1, 
  */

      CO_BEGIN_DEF
	IPS_NAME, "Object Label" , 
	IPS_IMAGE_IN , 1 , "Binary" , 
	IPS_IMAGE_OUT, 2 , "Object labeled" , 
	IPS_PAR_CHOICE, 1, "Foreground", 1, "4 connect", "8 connect", NULL,
	IPS_BORDER_POSIT,
      CO_END_DEF

      CO_BEGIN_DIM
	CO_SET_16C(2);  /* On met l'image 2 en couleurs */
      CO_END_DIM

      CO_BEGIN_EXE
	int c = (CO_GET_CHOICE(1) == 0) ? 4 : 8;  /* connexite */

	ex5a_label_object ( p->pix[1], p->pix[2],
	    p->cmin, p->xmax, p->ymax, p->xtot, p->xytot, c);

	/*  On passe en argument de cette fonction les bornes dont on a 
	 *  vraiment besoin ; parfois il suffit de passer xytot,
	 *  pour des algorithmes de balayage, du type
	 *      for (t = 0; t < xytot; t++) { ima[t] = .. }
	 *  Pour d'autres algorithmes on a besoin de toutes les bornes.
	 *  Les trois parametres essentiels sont xsiz, ysiz, cmin ;
	 *  les autres bornes peuvent en etre deduits.
	*/
       CO_END_EXE

  CO_END_CONNECT


  /*
   *  EXERCICE : etiqueter chaque objet avec son aire comme couleur.
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExDistMap") 

  /*
   *  Calcul d'une image de distance, avec un bord d'epaisseur 2.
  */

      CO_BEGIN_DEF
	IPS_NAME,	"Distance chanfrein au fond",
	IPS_IMAGE_IN,	1, "Binaire",
	IPS_IMAGE_OUT,	2, "Dist Map",
	IPS_PAR_CH_ABR, 1, "Distance :", 2, "d4", "d8", "3,4", "5,7,11", NULL, 
	IPS_BORDER,	2,
      CO_END_DEF

      CO_BEGIN_DIM
	CO_SET_16C(2);  /* On met l'image 2 en couleurs */
      CO_END_DIM

      CO_BEGIN_EXE
	ex5b_set_mask (CO_GET_CH_ABR (1), p->xtot);
	ex5b_calc_dt (p->pix[1], p->pix[2], p->xytot);
      CO_END_EXE

  CO_END_CONNECT

  /*
   *  EXERCICE : faire une transformation de distance inverse.
  */

/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExErodeMulti") 

  /*
   *	6.  COMPLEMENTS
   *        -----------
   * 
   *    - L'INDICATEUR D'ETAT
   *  
   *  L'indicateur d'etat est un petit trait jaune sur l'icone, qui indique
   *  le pourcentage realise' quand on fait un [Run].
   *  Il est particulierement recommande' pour des operateurs qui font de
   *  longs calculs ; cela aide l'utilisateur a garder patience, sinon il
   *  croira que l'operateur est plante' !
   *
   *  Comment faire ? Tout ce passe dans la section EXE :
   *   - declarer un entier etape, que l'on initialise immediatement.
   *   - appeler le timer avec  CO_SPY_INT (&etape, debut, fin);
   *   - changer autant de fois que l'on veut la valeur de etape.
   *  
   *  En fait CO_SPY_INT va mettre en route le timer, avec l'ordre de
   *  regarder chaque seconde la valeur de etape. Sachant que etape va
   *  de debut a fin, un pourcentage est calcule' et transmis a IPS.
   *  Le timer s'arrete automatiquement a la fin de la section EXE.
   *
   *
   *    - IMAGE TEMPORAIRE
   *  
   *  Certains operateurs ont besoins d'images temporaires :
   *  - on les cree avec un malloc
   *  - on teste si le malloc a reussi
   *  - on les detruit apres utilisation.
   *  
   *
   *  Voici une illustration de l'indicateur d'etat et d'une image temporaire
   *  avec un enchainement d'erosions.
  */

      CO_BEGIN_DEF
	IPS_NAME, "Erode Multiple" , 
	IPS_IMAGE_IN,	1, "Binaire",
	IPS_IMAGE_OUT,	2, "Erode multiple",
	IPS_PAR_INT,    1, "Nombre d'iterations :", 0, 1024, 100,
	IPS_PAR_CHOICE, 2, "Foreground", 1, "4 connect", "8 connect", NULL,
	IPS_BORDER, 1,
      CO_END_DEF

      CO_BEGIN_DIM
	CO_SET_16C(2);  /* On met l'image 2 en couleurs */
      CO_END_DIM

      CO_BEGIN_EXE
	int nb = CO_GET_INT(1), c = (CO_GET_CHOICE(2) == 0) ? 4 : 8;

	/*
	 * On cree une image temporaire
	*/
	Unt16 *ima3 = (Unt16 *) malloc (p->xytot * sizeof (Unt16));

	/*
	 * Le malloc a pu echouer
	*/
	if (ima3 == NULL) {
	    printf ("Malloc error for oper ExErodeMulti\n");

	} else {
	    /* C'est ok, ima3 est bien alloue'e. On peut maintenant appeler
	     * la fonction ex6_erode_multi, qui utilise CO_SPY_INT.
            */
	    ex6_erode_multi (p->pix[1], p->pix[2], ima3, nb, c, 
		p->cmin, p->xmax, p->ymax, p->xtot, p->ytot, p->xytot);
	}

	/*
	 * On libere ima3
        */
	free (ima3);

      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExGnuPlot") 

  /*
   * Creation d'un fichier de donnees et visualisation par gnuplot.
  */

      CO_BEGIN_DEF
	IPS_NAME, "Histogramme" , 
	IPS_IMAGE_IN,	1, "Quelconque en niveaux de gris",
	IPS_PAR_TEXT,   1, "Fichier donnees :", 80, "histog.dat",
	IPS_PAR_TEXT,   2, "Fichier commandes :", 80, "histog.gnu",
	IPS_PAR_TEXT,   3, "Display :", 80, "localhost",
	IPS_PAR_TEXT,   4, "Appel gnuplot :", 80, "gnuplot -display %s:0 %s &",
	IPS_PAR_CH_ABR, 5, "Dessin :", 2, "points", "lines", "steps", 
				"impulses", "boxes", NULL,
      CO_END_DEF

      CO_BEGIN_EXE
	char *fd = CO_GET_TEXT (1), *fc = CO_GET_TEXT (2),
	     *dp = CO_GET_TEXT (3), *ap = CO_GET_TEXT (4);
	char liste[][16] = {"points", "lines", "steps", "impulses", "boxes" };
	char *dessin = liste[ CO_GET_CHOICE(5) ];

	ex6_calc_histog (p->pix[1], p->xytot, fd);
	ex6_appel_gnuplot (fd, fc, dp, ap, dessin);
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExDemoParBox") 

  /*
   * Boite de demonstration : rappel des parametres et des sections
  */

      CO_BEGIN_DEF
	/* 
	 *  Declaration de la boite de parametres
	 */
	IPS_NAME,	"Demo ParamBox",
	IPS_IMAGE_IN,	1, "Emplacement des legendes",
	IPS_IMAGE_IN,	2, "des images en entree",
	IPS_IMAGE_IN,	3, "( ici il y en a 3 )",
	IPS_IMAGE_OUT,	4, "Emplacement des legendes",
	IPS_IMAGE_OUT,	5, "des images en sortie",
	IPS_PAR_REAL,   1, "Valeur reelle :", "-6.3", "6.3", "3.14159",
	IPS_PAR_INT,    2, "Valeur entiere :", 0, 1024, 256,
	IPS_PAR_TEXT,   3, "Chaine de car :", 40, "noname.txt",
	IPS_PAR_CHOICE, 4, "Boutons Radio :", 3, "grise 1", "2", "3", 
				"degrise", NULL,
	IPS_PAR_CH_ABR, 5, "Choix Abrege :", 2, "bijou", "caillou", "chou", 
				"genou", "hibou", NULL,
	IPS_PAR_SLIDER, 6, "Potentiometre :", 1, 9, 5,
	IPS_PAR_CHK_BOX,7, "Je suis on ou off :", 1,
	IPS_PAR_CHK_BOX,8, "et moi off ou on :", 0,
      CO_END_DEF

      CO_BEGIN_CHOICE
	/*
	 *  Appele pour chaque action dans PAR_CHOICE, PAR_CH_ABR, CHK_BOX
	 */
	int x;
	printf ("-- CHOICE  ini = %d  num = %d  val = %d\n", 
	    p->chce_ini, p->chce_num, p->chce_val);

	/* on grise ici des parametres */
	if (p->chce_num == 4)
	  for (x = 1; x < 4; x++)
	    CO_SET_ACTIVE (x, x-1 != p->chce_val);

	/* on modifie le param 6 (un potentiometre) */
	x = CO_GET_SLIDER(6) + 1;
	if (x > 9) x = 1;
	CO_SET_SLIDER (6, x);
      CO_END_CHOICE

      CO_BEGIN_DIM
	/* 
	 *  Appele juste au debut d'un Run :
	 *  Les tailles des images deja selectionnes sont dans les tableaux
	 *  p->ima_x[1..31] et p->ima_y[1..31]
	 *  Cet emplacement sert a lire et a modifier la taille des images.
	 */
	p->ima_x[2] = p->ima_y[1];
	p->ima_y[2] = p->ima_x[1];
	p->ima_x[3] = p->ima_y[3] = CO_GET_INT(2);
      CO_END_DIM

      CO_BEGIN_EXE
	/* 
	 *  Appele pour l'execution
	 */

	/* declaration de variables */
	int x, y;

	/* exemple de calcul sur une image */
	for (y = 0; y < p->ysiz; y++)
	  for (x = 0; x < p->xsiz; x++)
	    p->pix[1][y * p->xsiz + x] = 0;
	/* Si les images ont des tailles differentes on peut utiliser */
	/* p->ima_x[], p->ima_y[] a la place de p->xsiz, p->ysiz      */

	/* on lit les parametres */
	printf ("Valeur #1 = %f\n", CO_GET_REAL    (1));
	printf ("Valeur #2 = %d\n", CO_GET_INT     (2));
	printf ("Valeur #3 = %s\n", CO_GET_TEXT    (3));
	printf ("Valeur #4 = %d\n", CO_GET_CHOICE  (4));
	printf ("Valeur #5 = %d\n", CO_GET_CH_ABR  (5));
	printf ("Valeur #6 = %d\n", CO_GET_SLIDER  (6));
	printf ("Valeur #7 = %d\n", CO_GET_CHK_BOX (7));
	printf ("Valeur #8 = %d\n", CO_GET_CHK_BOX (8));
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExLesDessins") 

  /*
   *	7.  FIGURES INTERACTIVES
   *        --------------------
   * 
   * 
  */

      CO_BEGIN_DEF
	IPS_NAME, "Les dessins", 
	IPS_IMAGE_IN,   1, "Background",
	IPS_FIG_IMAGE,	1, 1, "Dessins disponibles",
      CO_END_DEF

      CO_BEGIN_FIG
	switch (p->fig_event) {
	    case FIG_REFRESH :

		CO_DRAW_MODE_CPY;
		CO_DRAW_COLOR(9);
		CO_DRAW_POINT(1,10,10);
		CO_DRAW_LINE(1,15,10,35,20);
		CO_DRAW_RECT(1,40,10,60,30);
		CO_FILL_RECT(1,40,40,60,60);
		CO_DRAW_VALUE(1,60,60,4567);
		CO_DRAW_COLOR(12);
		CO_DRAW_CIRC(1,83,30,18);
		CO_DRAW_ELLIP(1,110,10,140,30);
		CO_DRAW_ARC(1,10,80,40,100,0,64*300);
		CO_FILL_ARC(1,10,120,40,140,0,64*300);
		CO_DRAW_COLOR(10);
		CO_DRAW_TRIANG(1,60,80,90,90,70,105);
		CO_FILL_TRIANG(1,100,80,130,90,110,105);
		CO_DRAW_CROSS(1,60,160);
		CO_DRAW_CURSOR(1,80,160);
		CO_DRAW_MARK(1,100,160);
		CO_DRAW_LABEL(1,120,160,123);

		break;
	}
      CO_END_FIG

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExLesEvents") 

  /*
   * 
  */

      CO_BEGIN_DEF
	IPS_NAME, "Les evenements", 
	IPS_IMAGE_IN,   1, "Background",
	IPS_FIG_IMAGE,	1, 1, "Figure 1",
	IPS_FIG_IMAGE,	2, 1, "Figure 2",
      CO_END_DEF

      CO_BEGIN_FIG

	switch (p->fig_event) {
	    case FIG_REFRESH :
		printf ("FIG_REFRESH : tout redessiner\n");
		break;
	    case FIG_DONE :
		printf ("FIG_DONE : fenetre fermee\n");
		break;
	    case FIG_NONE :
		printf ("FIG_NONE : evenement vide\n");
		break;
	    case FIG_PUSH :
		printf ("FIG_PUSH : bouton souris enfonce'\n");
		break;
	    case FIG_DRAG :
		printf ("FIG_DRAG : souris tire'e\n");
		break;
	    case FIG_RELEASE :
		printf ("FIG_RELEASE : bouton souris relache'\n");
		break;
	    case FIG_CANCEL :
		printf ("FIG_CANCEL : DRAG annule', sortie de la fenetre\n");
		break;
	}

	printf ("Fenetre %d  Bouton %d  X = %.2lf Y = %.2lf  Zoom = %d\n\n",
	    p->fig_num, p->fig_butt, p->fig_x, p->fig_y, p->fig_zoom);

      CO_END_FIG

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExProfils") 

  /*
   * 
  */

      CO_BEGIN_DEF
	IPS_NAME, "Profils" , 
	IPS_IMAGE_IN,   1, "Background",
	IPS_FIG_IMAGE,	1, 1, "Left = Line, Middle = Rect",
	IPS_PAR_TEXT,   1, "Fichier donnees :", 80, "histog.dat",
	IPS_PAR_TEXT,   2, "Fichier commandes :", 80, "histog.gnu",
	IPS_PAR_TEXT,   3, "Display :", 80, "localhost",
	IPS_PAR_TEXT,   4, "Appel gnuplot :", 80, "gnuplot -display %s:0 %s &",
      CO_END_DEF

      CO_BEGIN_FIG
	ex7a_fig_profils (p, 1);
      CO_END_FIG

      CO_BEGIN_EXE
	char *fd = CO_GET_TEXT (1), *fc = CO_GET_TEXT (2),
	     *dp = CO_GET_TEXT (3), *ap = CO_GET_TEXT (4);

	ex7a_calc_histog (p->pix[1], p->xsiz, p->ysiz, fd);
	ex7a_appel_gnuplot (fd, fc, dp, ap);
      CO_END_EXE

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExInsereRec") 

  /*
   * 
  */

      CO_BEGIN_DEF
	IPS_NAME, "Rectangles" , 
	IPS_IMAGE_IN,   1, "Background",
	IPS_PAR_TEXT,   1, "Fichier de rectangles :", 80, "rects.tmp",
	IPS_PAR_INT,	2, "Couleur :", 0, 255, 12,
	IPS_FIG_IMAGE,	1, 1, "Left = Add/Move, Middle = Suppr/Center",
      CO_END_DEF

      CO_BEGIN_FIG
	ex7b_fig_rects (p, 1, 1, 2);
      CO_END_FIG

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExInserePts") 

  /*
   * 
  */

      CO_BEGIN_DEF
	IPS_NAME, "Points numerotes" , 
	IPS_IMAGE_IN,   1, "Background",
	IPS_PAR_TEXT,   1, "Fichier de points :", 80, "points.tmp",
	IPS_PAR_INT,    2, "Label :", 0, 10000, 0,
	IPS_FIG_IMAGE,	1, 1, "Left = Add/Move, Middle = Suppr/Center",
      CO_END_DEF

      CO_BEGIN_FIG
	ex7c_fig_points (p, 1, 1, 2);
      CO_END_FIG

  CO_END_CONNECT


/* ========================================================================= */

  CO_BEGIN_CONNECT ("ExInserePts2") 

  /*
   * 
  */

      CO_BEGIN_DEF
	IPS_NAME, "Points numerotes 2 figures" , 
	IPS_IMAGE_IN,   1, "Image A",
	IPS_IMAGE_IN,   2, "Image B",
	IPS_PAR_TEXT,   1, "Fichier de points A :", 80, "pointsA.tmp",
	IPS_PAR_TEXT,   2, "Fichier de points B :", 80, "pointsB.tmp",
	IPS_PAR_INT,    3, "Label A :", 0, 10000, 0,
	IPS_PAR_INT,    4, "Label B :", 0, 10000, 0,
	IPS_FIG_IMAGE,	1, 1, "Figure A",
	IPS_FIG_IMAGE,	2, 2, "Figure B",
      CO_END_DEF

      CO_BEGIN_FIG
	if (p->fig_num == 1)
	     ex7c_fig_points (p, 1, 1, 3);
	else ex7c_fig_points (p, 2, 2, 4);
      CO_END_FIG

  CO_END_CONNECT


/* ========================================================================= */

CO_END_CALL_OPER



