/***************************************************************************************/
/* chaussette.c                                               I.P.S  v 3.7  14/03/1996 */
/*                                                                                     */
/* module de communication par socket                    Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#define CS_NUM_PROTOCOL "1.0"


#include "inclus.h"
#include <errno.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>

#define DEBUG_

#define CS_CTX_TAB	64	/* Nombre de contextes */
#define CS_BUFSIZE	2048	/* Taille systeme des buffers des socket */
#define CS_GROSBUF	10240	/* Taille buffer reception pour envoi par blocs */
#define CS_GRAIN	1728	/* Taille d'un bloc (ancien=1792 plante Linux 2.0.0) */
#define CS_ENTETE	32	/* Taille entete message */
#define CS_ORDBLOC	32	/* Position bloc apres la commande d'envoi */


typedef struct {
	int	libre,			/* booleen : contexte libre/occupe' */
		pid,			/* le pid du processus distant */
		working,		/* booleen : une requete est en cours */
		vieux,			/* index pour cs_renvoie_ctx_libre() */
		ord_suiv,		/* ordre suivant, pour envoi de buffers */
		d_nb, d_tb, 		/* Infos bloc : nombre blocs, taille bloc, */
		d_tt, d_nd,		/* 	taille totale, numero data */
		nm_in, nm_out,		/* num dernier mess recu et envoye' */
		sendto_len;		/* taille dernier mess envoye' */
	Ico	*ico;			/* l'icone du telop */
	void	*data1,			/* champs de donnees : malloc / free */
		*adr1,			/* adr non libere, pour REP/SEND/BLOC/DONE */
		*data2;			/* liste des icones selectionnes */
	char	bufSnd[CS_BUFSIZE],	/* buffer pr envoi */
		*mSnd;			/* pointeur apres entete dans bufSnd */
	struct sockaddr adrFar;		/* adresse distante */
	} Cs_ctx_tab;


/*
 * Variables globales
*/

static int	cs_sLoc, 			/* socket locale */
		cs_pLoc, cs_pFar,		/* port local et distant */
		cs_sizeSnd, cs_sizeRcv,		/* taille buffer systeme pr socket */
		cs_ctx,				/* numero contexte */
		cs_lgFar,			/* longueur adrFar */
		cs_vieux_ctx;			/* fournir des "vieux" ctx inutilise's */
static struct sockaddr_in
	 	cs_adrFar;			/* adresse distante */
static char 	cs_bufRcv[CS_BUFSIZE],		/* buffer pr reception */
		*cs_mRcv;			/* pointeur apres entete dans buffer */
static Cs_ctx_tab
		cs_ctx_tab[CS_CTX_TAB];		/* table de contextes */

/*
 * Declaration fonctions
*/

void cs_init ();
int cs_creer_socket ();
void cs_quit ();
void cs_footer ();
int cs_recvfrom ();
int cs_sendto ();
int cs_sendto_str ();
void cs_change_dest ();
int cs_renvoie_ctx_libre ();
void cs_init_ctx ();
void cs_free_ctx ();
void cs_kill_ctx ();
void cs_set_data2 ();
void *cs_get_data2 ();
int cs_is_working ();
int cs_execute ();
int cs_requete ();
Notify_value cs_input ();



/***************************************************************************************
 * Initialisation generale
 *
 ***************************************************************************************/
void cs_init()
{
    int i;

    if (cs_creer_socket () < 0) exit (0);

    notify_set_input_func (
	(Notify_client) 10203,	/* arbitraire */
	cs_input,		/* callback */
	cs_sLoc);		/* socket */

    cs_lgFar = sizeof(cs_adrFar);
    cs_adrFar.sin_family = AF_INET;
    cs_mRcv = cs_bufRcv+CS_ENTETE;

    cs_vieux_ctx = 0;
    for (i = 0; i < CS_CTX_TAB; i++) cs_init_ctx (i);
}



/***************************************************************************************
 * Creation de la socket
 *
 ***************************************************************************************/
int cs_creer_socket ()
{
    int lg;
    struct sockaddr_in addr;

    /* Creation de la socket */
    if ((cs_sLoc = socket (AF_INET, SOCK_DGRAM, 0)) == -1) {
	perror ("Can't create socket");
	return -1;
    }

    /* Preparation de l'adresse */
    memset ((char *) &addr, 0, sizeof (addr));
    addr.sin_port = 0;
    addr.sin_addr.s_addr = INADDR_ANY;
    addr.sin_family = AF_INET;

    if (bind (cs_sLoc, (struct sockaddr *) &addr, sizeof(addr))) {
	perror ("Can't bind socket");
	return -1;
    }

    lg = sizeof (addr);
    if (getsockname (cs_sLoc, (struct sockaddr *) &addr, &lg)) {
	perror ("Can't get socket name");
	return -1;
    }

    cs_pLoc = ntohs (addr.sin_port);

    /* propriete socket : Rifflet p.287 */
    cs_sizeSnd = cs_sizeRcv = CS_BUFSIZE;
    lg = sizeof (cs_sizeSnd);
    setsockopt (cs_sLoc, SOL_SOCKET, SO_SNDBUF, &cs_sizeSnd, lg);
    setsockopt (cs_sLoc, SOL_SOCKET, SO_RCVBUF, &cs_sizeRcv, lg);
    getsockopt (cs_sLoc, SOL_SOCKET, SO_SNDBUF, &cs_sizeSnd, &lg);
    getsockopt (cs_sLoc, SOL_SOCKET, SO_RCVBUF, &cs_sizeRcv, &lg);

#   ifdef DEBUG
    printf ("Numero du port = %d\n", cs_pLoc);
    printf ("SO_RCVBUF = %d\n", cs_sizeRcv);
    printf ("SO_SNDBUF = %d\n", cs_sizeSnd);
#   endif

    return 0;
}


/***************************************************************************************
 * Suppression de la socket
 *
 ***************************************************************************************/
void cs_quit ()
{
    int i;

    for (i = 0; i < CS_CTX_TAB; i++) cs_kill_ctx (i);

    close (cs_sLoc); 
#   ifdef DEBUG
    printf ("Fermeture socket %d\n", cs_pLoc);
#   endif
}


/***************************************************************************************
 * Message dans footer
 *
 ***************************************************************************************/
void cs_footer (ctx, mess)
int ctx;
char *mess;
{
    char s[32];
    sprintf (s, "tel%d : ", ctx);
    pcp_footer (s, mess, NULL);
}

/***************************************************************************************
 * Reception message : normalement bloquante, mais on ne lit que depuis 
 * la callback cs_input quand il y a qqchose.
 *
 * Dans l'entete du message il doit y avoir :
 *	TEL ctx size num_mess
 * Le num_mess lu sert a ge'rer les cas AGAIN.
 * Modifie le contexte global : cs_ctx
 ***************************************************************************************/
int cs_recvfrom ()
{
    int n, k, nm_lu;

    n = recvfrom (cs_sLoc, cs_bufRcv, CS_BUFSIZE, 0,
	(struct sockaddr *) &cs_adrFar, &cs_lgFar);

#   ifdef DEBUG
    printf ("IPS < tel%d : %s\n", cs_ctx, cs_mRcv);
#   endif

    if (n < 0) {
	perror ("ERROR recvfrom");
    } else if (sscanf (cs_bufRcv, "TEL %d %d %d", &cs_ctx, &k, &nm_lu) != 3) {
	printf ("ERROR recvfrom, bad header\n");
    } else if (n != k) {
	printf ("ERROR recvfrom, bad message size\n");
    } else if (cs_ctx < 0 || cs_ctx >= CS_CTX_TAB) {
	printf ("ERROR recvfrom, bad context\n");
    } else if (cs_ctx_tab[cs_ctx].libre) {
	/* Test si le contexte est attribue' (sinon : zombie, erreur).
	 * Attention, il faut tester .libre avant .nm_in, pour ne pas modifier
	 * .nm_in par un contexte non attribue' (ou venant d'etre libe're').
	*/
	sprintf (cs_ctx_tab[cs_ctx].mSnd, "34"); /* QUIT */
	cs_sendto_str (); /* ca modifie .nm_out mais c'est pas grave */	
    } else if (nm_lu > cs_ctx_tab[cs_ctx].nm_in) {
	/* Ce message n'a jamais ete recu */
	cs_ctx_tab[cs_ctx].nm_in = nm_lu;
	memcpy (&cs_ctx_tab[cs_ctx].adrFar, &cs_adrFar, sizeof (cs_adrFar));
	return n; /* ok */
    } else {
	/* Ce message a deja ete recu.
	 * Si (nm_lu == nm_in) on re'envoie la derniere re'ponse.
	 * sinon, c'est un vieux message dont on ne connait plus la reponse.
	 * Dans les 2 cas, on renvoie -1 pour ne pas traiter ce message.
	 * LE BUT : le cas AGAIN est traite' ici ; lorsque cs_recvfrom renvoie >= 0
	 * dans cs_input, le message n'a JAMAIS ete traite'.
	*/
#	ifdef DEBUG
	printf ("IPS: BINGO message ancien (%d <= %d) ctx = %d\n", 
	    nm_lu, cs_ctx_tab[cs_ctx].nm_in, cs_ctx);
#	endif
	if (nm_lu == cs_ctx_tab[cs_ctx].nm_in) cs_sendto_again ();
	return -1;
    }
    return -1;
}


/***************************************************************************************
 * Reponse message (non bloquante)
 *
 * Le destinataire est le dernier recvfrom ; pour le modifier, cf cs_change_dest ().
 * Utilise le contexte global : cs_ctx
 ***************************************************************************************/
int cs_sendto (len)
int len;
{
    int n;
    Cs_ctx_tab *tab = &cs_ctx_tab[cs_ctx];

#   ifdef DEBUG
    printf ("IPS > tel%d : %s\n", cs_ctx, tab->mSnd);
#   endif

    tab->sendto_len = len += CS_ENTETE;
    sprintf (tab->bufSnd, "IPS %5d %10d ", len, ++tab->nm_out);

    n = sendto (cs_sLoc, tab->bufSnd, len, 0, 
	(struct sockaddr *) &cs_adrFar, cs_lgFar);

    if (n < 0) perror ("ERROR sendto");

    return n;
}


/***************************************************************************************
 * Idem cs_sendto + calcul taille message automatique (valable en cas de string).
 *
 ***************************************************************************************/
int cs_sendto_str ()
{
    return cs_sendto (strlen (cs_ctx_tab[cs_ctx].mSnd) + 1);
}

/***************************************************************************************
 * Re'envoie le dernier message d'un ctx, sans modifier l'entete.
 *
 ***************************************************************************************/
int cs_sendto_again ()
{
    int n;

#   ifdef DEBUG
    Cs_ctx_tab *tab = &cs_ctx_tab[cs_ctx];
    printf ("AGAIN IPS > tel%d : %s\n", cs_ctx, tab->mSnd);
#   endif

    n = sendto (cs_sLoc, cs_ctx_tab[cs_ctx].bufSnd, 
	cs_ctx_tab[cs_ctx].sendto_len, 0, 
	(struct sockaddr *) &cs_adrFar, cs_lgFar);

    if (n < 0) perror ("ERROR sendto");

    return n;
}


/***************************************************************************************
 * Pour changer de destinataire : cs_change_dest (ctx)
 *
 * Modifie le contexte global : cs_ctx
 *
 * Il peut etre utile de sauvegarder un contexte, puis de le restituer ;
 * c'est le cas dans cs_requete().
 ***************************************************************************************/
void cs_change_dest (ctx)
int ctx;
{
    cs_ctx = ctx;
    if (cs_ctx >= 0 && cs_ctx < CS_CTX_TAB)
    memcpy (&cs_adrFar, &cs_ctx_tab[cs_ctx].adrFar, sizeof (cs_adrFar));
}


/***************************************************************************************
 * Renvoie un contexte libre, -1 si erreur
 * Le ctx libre renvoye' est le plus "vieux" possible par se'curite'.
 *
 * Attention, l'appelant doit marquer
 *	cs_ctx_tab[i].libre = FAUX;
 ***************************************************************************************/
int cs_renvoie_ctx_libre ()
{
    int i, 
        ctx = -1, 		/* erreur par defaut */
        v = cs_vieux_ctx;	/* > au plus vieux actuel */

    for (i = 0; i < CS_CTX_TAB; i++)
	if (cs_ctx_tab[i].libre) 
	    if (cs_ctx_tab[i].vieux < v) { ctx = i; v = cs_ctx_tab[i].vieux; }

    return ctx;
}


/***************************************************************************************
 * Init un contexte
 *
 ***************************************************************************************/
void cs_init_ctx (ctx)
int ctx;
{
    Cs_ctx_tab *tab = &(cs_ctx_tab[ctx]);

    tab->libre = VRAI;
    tab->pid = 0;
    tab->working = FAUX;
    tab->ico = (Ico *) NULL;
    tab->data1 = 
    tab->data2 = 
    tab->adr1  = (void *) NULL;
    tab->vieux = cs_vieux_ctx++; /* incremente' a chaque init et free */
    tab->ord_suiv = 22; /* WAIT par defaut */
    tab->nm_in =
    tab->nm_out = 0;
    tab->sendto_len = 0;
    tab->mSnd = tab->bufSnd + CS_ENTETE;
    memset (tab->bufSnd, 0, CS_BUFSIZE);
    memset (&(tab->adrFar), 0, cs_lgFar);

#   ifdef DEBUG
    printf ("CS_INIT_CTX %d, nm_in = %d\n", ctx, tab->nm_in);
#   endif
}

void cs_free_ctx (ctx)
int ctx;
{
    pcp_free (cs_ctx_tab[ctx].data1);
    pcp_free (cs_ctx_tab[ctx].data2);
    cs_init_ctx (ctx);
}

/***************************************************************************************
 * Fait un "kill -TERM pid" et libere directement le contexte
 *
 * Attention il n'y a pas de refresh de l'icone, car cs_kill_ctx est principalement 
 * appele' par telop_free.
 ***************************************************************************************/
void cs_kill_ctx (ctx)
int ctx;
{
    char ligne[256];
    Telop *op;

    if (cs_ctx_tab[ctx].libre) return;

    /* Le .ico peut etre NULL pendant cs_quit() */
    if (cs_ctx_tab[ctx].ico == (Ico *) NULL) { cs_free_ctx (ctx); return; }

    /* Detachement entre le ctx et l'ico */
    op = (Telop *) cs_ctx_tab[ctx].ico->obj;
    op->ctx = -1;
    cs_ctx_tab[ctx].ico = (Ico *) NULL;

    /* Le pid est-il > 0 */
    if (cs_ctx_tab[ctx].pid == 0) { cs_free_ctx (ctx); return; }

    /* fait un kill sur le pid */
    if (op->nom_host[0] == 0)
         sprintf (ligne, "kill -TERM %d &", cs_ctx_tab[ctx].pid);
    else sprintf (ligne, "rsh -n %s kill -TERM %d &", op->nom_host, cs_ctx_tab[ctx].pid);
#   ifdef DEBUG
    printf ("Execution de %s\n", ligne);
#   endif

    if (system (ligne) != 0)
	perror ("ERROR while system ()");
    cs_free_ctx (ctx);

    /* Remarque : 
     * comme on fait un kill -TERM, le process kille' va renvoyer un BYE
     * or le ctx ne sera plus attribue', ce qui fera renvoyer un QUIT par
     * cs_recvfrom.
    */
}


/***************************************************************************************
 * acces a data2
 *
 ***************************************************************************************/
void cs_set_data2 (ctx, data2)
int ctx;
void *data2;
{
    cs_ctx_tab[ctx].data2 = data2;
}

void *cs_get_data2 (ctx)
int ctx;
{
    if (ctx < 0 || ctx >= CS_CTX_TAB) return NULL;
    return cs_ctx_tab[ctx].data2;
}

/***************************************************************************************
 * Retourne si une requete est en cours
 *
 ***************************************************************************************/
int cs_is_working (ico)
Ico *ico;
{
    Telop *op = (Telop *) ico->obj;

    if (op->ctx == -1) return FAUX;
    return cs_ctx_tab[op->ctx].working;
}


/***************************************************************************************
 * Execute op->nomf et renvoie le ctx, < 0 si erreur
 *
 * Appel : ctx = cs_execute (ico);
 *
 * Il faut de plus que cs_ctx_tab[ctx].data1 = (char *) "ma_requete", 
 * car l'executable va appeler la callback cs_input avec "HELO".
 *
 * CALLED IN : cs_requete ()
 ***************************************************************************************/
int cs_execute (ico)
Ico *ico;
{
    Telop *op = (Telop *) ico->obj;
    char ligne[256], bla[256];
    int  ctx;

    if ((ctx = cs_renvoie_ctx_libre ()) < 0) return -1;

    if (op->nom_host[0] == 0)
         sprintf (ligne, "%s -ips %d %d &", op->nomf, cs_pLoc, ctx);
    else sprintf (ligne, "rsh -n %s %s -ips %d %d -host %s &", 
	op->nom_host, op->nomf, cs_pLoc, ctx, glo.host_name);

    sprintf (bla, "Running on %s ...", 
	(op->nom_host[0]) ? op->nom_host : glo.host_name);
    cs_footer (ctx, bla);

    if (system (ligne) != 0) {
	perror ("ERROR while system ()");
	return -2;
    }

    /* Tout s'est bien passe' : on marque le contexte occupe' */
    cs_ctx_tab[ctx].libre = FAUX;
    cs_ctx_tab[ctx].ico = ico;
    op->ctx = ctx;
    ico_refresh_now (ico);
}


/***************************************************************************************
 * Demande une requete au tel_oper, en l'executant ou en la stockant pour "HELO".
 * Remarque : au retour, le ctx est disponible dans op->ctx
 *
 * Attention : ne pas appeler cs_requete tant que une action est en cours pour cet ico !!
 * En particulier, ne pas appeler 2 fois dans la meme fonction.
 *
 * Renvoie 0 si ok, -1 pour erreur
 *
 * FORME DE LA REQUETE :
 *	DEF_ACT idf
 *	CHO_ACT idf nb num val num val ...
 ***************************************************************************************/
int cs_requete (ico, requete)
Ico *ico;
char *requete;
{
    Telop *op = (Telop *) ico->obj;
    char ligne[256];
    int dispo = FAUX;

    if (op->ctx != -1) {

	if (cs_ctx_tab[op->ctx].working) {
	    cs_footer (op->ctx, "Sorry, dialog in progress");
	    return -1;
	}

	if (cs_ctx_tab[op->ctx].pid != 0) {

	    /* fait un kill -0 sur le pid pour tester existence */
	    if (op->nom_host[0] == 0)
	         sprintf (ligne, "kill -0 %d", cs_ctx_tab[op->ctx].pid);
	    else sprintf (ligne, "rsh -n %s kill -0 %d", 
		op->nom_host, cs_ctx_tab[op->ctx].pid);
	    dispo = (system (ligne) == 0);

	    if (!dispo) { /* Le process est mort sans le signaler */
		cs_free_ctx (op->ctx);
		ico_refresh_now (ico);
	    }
	}
	else {
	    cs_footer (op->ctx, "ERROR pid = 0");
	    return -1;
	}
    }

#   ifdef DEBUG
    if (dispo)
	 printf ("cs_requete : le pid existe, j'envoie la requete\n");
    else printf ("cs_requete : je lance l'execution\n");
#   endif

    if (dispo) {
	/* On sauvegarde le contexte actuel */
	int old_ctx = cs_ctx;

	/* Change le contexte car le sendto n'est pas en reponse a un recvfrom */
	cs_change_dest (op->ctx);

	/* Envoie la requete */
	strcpy (cs_ctx_tab[op->ctx].mSnd, requete);
	cs_ctx_tab[op->ctx].working = VRAI;
	cs_sendto_str ();

	/* On restitue le contexte */
	cs_change_dest (old_ctx);
    }
    else {
	if (cs_execute (ico) >= 0) {
	    cs_ctx_tab[op->ctx].data1 = (char *) stg_Cpy (requete);
	    cs_ctx_tab[op->ctx].working = VRAI;
	} else return -1;
    }

    return (op->ctx < 0) ? -1 : 0;
}


/***************************************************************************************
 * Callback socket - gestion du protocole.
 *
 * Utilise le contexte global : cs_ctx
 ***************************************************************************************/
Notify_value cs_input (client, fd)
Notify_client client;
int fd;
{
    int len, motd, ordre;
    Cs_ctx_tab *tab;
    char *mSnd;

    /*
     * attente reponse --> cs_ctx et cs_mRcv
     * Si (cs_recvfrom() >= 0) alors le message n'a JAMAIS ete traite',
     * et le contexte est bien attribue'.
    */
    if (cs_recvfrom () < 0) return;
    if (sscanf (cs_mRcv, " %d%n", &ordre, &motd) != 1) return;
    tab = &cs_ctx_tab[cs_ctx];
    mSnd = tab->mSnd;

    /*
     * Traitement de l'ordre
    */
    switch (ordre) { /* switch #1 */


    /* ------------------- C O M M A N D E S - G E N E R A L E S ---------- HELO / REQTE
     *
     * "HELO pid num_protocol"
     * Le client s'identifie a IPS, lui donne son pid et son numero de protocole.
     * IPS passe ensuite a 17:REQTE, ou renvoie 14:SHORT.
     *
     * "REQTE"
     * Le client demande la requete a effectuer
     * En retour, IPS envoie la requete.
    */
    case 11 : {

	char bla[256];
	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;

	cs_footer (cs_ctx, "HELO");

	if (sscanf (cs_mRcv+motd, " %d %s", &(tab->pid), bla) < 1) tab->pid = 0;

	if (tab->pid == 0) {
	    /* Erreur de protocole : absence de pid */
	    tab->working = FAUX; /* Le dialogue est fini */
	    sprintf (mSnd, "34"); /* QUIT */
	    cs_sendto_str ();
	    break;
	}
	if (strcmp (bla, CS_NUM_PROTOCOL)) {

	    tab->working = FAUX; /* Le dialogue est fini */
	    sprintf (mSnd, "34"); /* QUIT */
	    cs_sendto_str (); /* on envoie le message tout de suite */

	    pcp_error ("Not the same IPS_SOCKET protocol number !",
		"", "IPS number is", CS_NUM_PROTOCOL,
		"while tel_oper number is", bla, 
		"", "You have to update libraries",
		"and compile your tel_oper again", NULL);
	    break;
	}

	/*
	 * Si le tel_oper est en remote host, on lui envoie un test 14:SHORT
	 * de repre'sentation locale de short ; la reponse provoque 17:REQTE
	*/
	if (op->nom_host[0] != 0) {

	    int az = 'A'*256 + 'z';
	    sprintf (mSnd, "14 %d %d", az, htons(az)); /* SHORT */
	    cs_sendto_str ();
	    break;
	}
	/* Sinon on passe a 17:REQTE */

    }
    case 17 : { /* REQTE : envoi de la requete */

	strcpy (mSnd, (char *) tab->data1);
	pcp_free (tab->data1);
	tab->data1 = NULL;
	cs_sendto_str ();
    } break;

    /* ---------------------------------------------------------------------- CIAO / BYE
     * "CIAO" ou "BYE"
     *
     * Le client alerte IPS qu'il se suicide.
     * CIAO : retour QUIT (avec F_ACQUIT : le client sera su^r que IPS est informe').
     * BYE  : pas de retour.
    */
    case 31 : /* CIAO */

	tab->working = FAUX; /* Le dialogue est fini */
	sprintf (mSnd, "34"); /* QUIT */
	cs_sendto_str ();
	/* pas de break, on continue */

    case 37 : /* BYE */
    {
	/* ATTENTION, ico peut etre NULL a ce stade. */
	if (tab->ico != (Ico *) NULL) {
	    Telop *op = tab->ico->obj;
	    telop_maj_inout (tab->ico); /* appele' avant de changer led ou ctx */
	    op->ctx = -1; /* detachement */
	    telop_change_led (tab->ico, LED_SLEEP);
	}
	cs_free_ctx (cs_ctx);
    } break;

    /* ----------------------------------------------------------------------------- REP
     * "REP nb_bloc taille_bloc taille_tot"
     *
     * Le client donne infos sur buffer
     * En retour : demande du premier bloc
    */
    case 49 : {

	sscanf (cs_mRcv+motd, " %d %d %d", &(tab->d_nb), &(tab->d_tb), &(tab->d_tt));

	/* creation data */
	tab->data1 = (char *) pcp_alloc (tab->d_tt + 16); /* securite */
	if (tab->data1 == NULL) break; /* malloc error */
	tab->adr1 = tab->data1; /* Le BLOC:55 travaille avec adr1 */

	sprintf (mSnd, "%d 0", ordre+3);
	cs_sendto_str ();
    } break;

    /* ---------------------------------------------------------------------------- SEND
     * "SEND num_bloc"
     *
     * Le client demande l'envoi du bloc num_bloc
     * Attention, il faut que tab->adr1 = qqchose, par ex. = tab->data1
     * En retour : BLOC1 num taille bloc
    */
    case 63 : {

	int nb, t;

	sscanf (cs_mRcv+motd, " %d", &nb);

	t = (CS_GRAIN*(nb+1) > tab->d_tt) ? tab->d_tt-CS_GRAIN*nb : CS_GRAIN;
	sprintf (mSnd, "%d %d %d ", ordre+3, nb, t);
	/* Attention il faut que strlen (mSnd) < CS_ORDBLOC */
	memcpy ((mSnd+CS_ORDBLOC), (char *)tab->adr1 + CS_GRAIN*nb, t);
	cs_sendto (CS_ORDBLOC+t);
    } break;

    /* ---------------------------------------------------------------------------- BLOC
     * "BLOC num taille bloc"
     *
     * Le client envoie un bloc
     * Attention, il faut que tab->adr1 = qqchose, par ex. = tab->data1
     * En retour : demande du bloc suivant (ordre-3) ou stop (ordre+3)
    */
    case 55 : {

	int n, t;

	sscanf (cs_mRcv+motd, " %d %d", &n, &t);
	memcpy ((char *)tab->adr1+tab->d_tb*n, cs_mRcv+CS_ORDBLOC, t);

	/* Demande bloc suivant */
	if (n+1 < tab->d_nb)
	     sprintf (mSnd, "%d %d", ordre-3, n+1);
	else sprintf (mSnd, "%d", ordre+3);
	cs_sendto_str ();
    } break;

    /* ---------------------------------------------------------------------------- DONE
     * "DONE"
     *
     * Le client dit qu'il a tout recu
     * En retour : ord_suiv
    */
    case 69 : {

	/* liberation data1 (mais pas de adr1) */
	pcp_free (tab->data1); /* si il vaut NULL ca ne fait rien */
	tab->data1 = (void *) NULL;

	/* On appelle ord_suiv */
	sprintf (mSnd, "%d", tab->ord_suiv);
	cs_sendto_str ();
    } break;


    /* --------------- C O M M A N D E S - D E - S E C T I O N ------------------ DEF_GO
     * "DEF_GO"
     *
     * Le client dit que tout est envoye', on peut construire la parbox.
     * En retour : mise en attente
    */
    case 105 : {

	Ico *ico = tab->ico;
	int x1, y1;

	cs_footer (cs_ctx, "DEF ... done");

	/* appel de telop_decode_def */
	telop_decode_def (ico, tab->data1, tab->d_tt);

	/* liberation data */
	pcp_free (tab->data1);
	tab->data1 = (void *) NULL;

	/* Mise a jour icone */
	x1 = ico->xm; y1 = ico->ym;
	telop_prepare_noms (ico);
	ico_homot_clic (ico, ico->xb, ico->yb, x1, y1, ico->xm, ico->ym);

	tab->working = FAUX; /* Le dialogue est fini */
	sprintf (mSnd, "22"); /* WAIT */
	cs_sendto_str ();
    } break;

    /* ---------------------------------------------------------------------------- ENCO
     * "ENCO"
     *
     * Le client demande d'encoder les parametres
     * En retour : REP1 nb_bloc taille_bloc taille_totale
    */
    case 303 : /* DIM_ENCO */
    {
	telop_change_led (tab->ico, LED_RUN);
    }
    case 203 : /* CHO_ENCO */
    case 403 : /* FIG_ENCO */
    {
	/* creation data */
	tab->data1 = (char *) pcp_alloc (CS_GROSBUF);
	if (tab->data1 == NULL) break; /* malloc error */

	/* encodage */
	telop_encode_cho (tab->ico, tab->data1, &(tab->d_tt), CS_GROSBUF);
	tab->adr1 = tab->data1; /* Le SEND:63 travaille avec adr1 */

	tab->ord_suiv = ordre+5;
	sprintf (mSnd, "%d %d %d %d", 60,
		(tab->d_tt-1)/CS_GRAIN+1, CS_GRAIN, tab->d_tt);
	cs_sendto_str ();
    } break;


    /* ------------------------------------------------------------------------- CHO_MAJ
     * "CHO_MAJ"
     *
     * Le client dit que tout est envoye', on peut mettre a jour la parbox.
     * En retour : mise en attente
    */
    case 213 : {

	Ico *ico = tab->ico;
	int x1, y1;

	cs_footer (cs_ctx, "CHOICE ... done");

	/* appel de telop_decode_cho */
	telop_decode_cho (ico, tab->data1, tab->d_tt);

	/* liberation data */
	pcp_free (tab->data1);
	tab->data1 = (void *) NULL;

	tab->working = FAUX; /* Le dialogue est fini */
	sprintf (mSnd, "22"); /* WAIT */
	cs_sendto_str ();
    } break;

    /* ----------------------------------------------------------------------- DIM_ENCO2
     * "DIM_ENCO2"
     *
     * Le client demande d'encoder les infos des images : taille, lut, noms
     * En retour : REP2 nb_bloc taille_bloc taille_totale
    */
    case 311 : /* DIM_ENCO2 */
    {
	/* creation data */
	tab->data1 = (char *) pcp_alloc (CS_GROSBUF);
	if (tab->data1 == NULL) break; /* malloc error */

	/* encodage */
	telop_encode_dim (tab->ico, tab->data1, &(tab->d_tt), CS_GROSBUF);
	tab->adr1 = tab->data1; /* Le SEND:63 travaille avec adr1 */

	tab->ord_suiv = ordre+5;
	sprintf (mSnd, "%d %d %d %d", 60,
		(tab->d_tt-1)/CS_GRAIN+1, CS_GRAIN, tab->d_tt);
	cs_sendto_str ();
    } break;

    /* ----------------------------------------------------------------------- DIM_DECO3
     * "DIM_DECO3"
     *
     * Le client dit que tout est envoye', on peut mettre a jour les images.
     * En retour : DODO, puis FOR ou WAIT
    */
    case 321 : {

	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;
	int x1, y1, test = VRAI;

	cs_footer (cs_ctx, "applying DIM changes");

	/* On fait patienter le client en attendant maj images */
	sprintf (mSnd, "24"); /* DODO */
	cs_sendto_str ();

	/* appel de telop_decode_dim */
	/* Attention ca peut prendre du temps ; c'est pourquoi on a envoye' DODO */
	test = telop_decode_dim (ico, tab->data1, tab->d_tt);

	/* liberation data */
	pcp_free (tab->data1);
	tab->data1 = (void *) NULL;

	if (test) {
	    sprintf (mSnd, "324"); /* FOR */
	}
	else { /* ca s'est mal passe' */
	    telop_maj_inout (ico); /* appele' avant de changer led ou ctx */
	    cs_footer (cs_ctx, "EXE failed");
	    telop_change_led (ico, LED_SLEEP);

	    tab->working = FAUX; /* Le dialogue est fini */
	    sprintf (mSnd, "22"); /* WAIT */
	}
	cs_sendto_str ();
    } break;

    /* ------------------------------------------------------------------------- EXE_ASK
     * "EXE_ASK num IPS_IMAGE"
     *
     * Le client demande la data num de nature IPS_IMAGE
     * En retour : REP nb_bloc taille_bloc taille_totale
    */
    case 327 : {

	Telop_data_cs *data = (Telop_data_cs *) cs_get_data2 (cs_ctx);
	int num, nat;
	char bla[256];

	sscanf (cs_mRcv+motd, " %d %d", &num, &nat);

	switch (nat) {

	    case IPS_IMAGE : {
		Image *ima = (Image *) data->v[num]->obj;
		tab->adr1 = ima->tab;
		tab->d_tt = ima->b.xytot * sizeof (Unt16);
		sprintf (mSnd, "%d %d %d %d", ordre+3,
		    (tab->d_tt-1)/CS_GRAIN+1, CS_GRAIN, tab->d_tt);
	    } break;
	    default : 
#		ifdef DEBUG
		printf ("ERREUR : type de data non prevu\n");
#		endif
		tab->working = FAUX; /* Le dialogue est fini */
		sprintf (mSnd, "22"); /* WAIT */
	}

	sprintf (bla, "Sending data %d (%dk)", num, tab->d_tt/1024);
	cs_footer (cs_ctx, bla);

	tab->ord_suiv = ordre+9;
	cs_sendto_str ();
    } break;

    /* ----------------------------------------------------------------------- EXE_READY
     * "EXE_READY"
     *
     * Mettre le footer a "Running"
     * En retour : GO
    */
    case 339 : {

	cs_footer (cs_ctx, "Executing op");

	sprintf (mSnd, "%d", ordre+3);
	cs_sendto_str ();
    } break;

    /* -------------------------------------------------------------- EXE_FOR / EXE_NEXT
     * "EXE_FOR"
     * Mettre d'abord le compteur a 0, puis faire "EXE_NEXT".
     *
     * "EXE_NEXT"
     * Demander l'image suivante, en passant taille_tot et en fixant tab->adr1,
     * ou passer a READY.
     * En retour "ASK num IPS_IMAGE taille_tot" ou "READY"
    */
    case 345 : /* EXE_FOR */
    {
	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;
        tab->d_nd = op->nb_image_in;

	/* On enleve le Run% */
	telop_change_led (ico, LED_RUN);
    }
    case 357 : {
	Telop *op = (Telop *) tab->ico->obj;
	Telop_data_cs *data = (Telop_data_cs *) cs_get_data2 (cs_ctx);
	int tot;
	char bla[256];

        if (++tab->d_nd <= op->nb_image_in + op->nb_image_out) {

	    switch (IPS_IMAGE) { /* Plus tard, mettre nat */
		case IPS_IMAGE : {
		    Image *ima = (Image *) data->v[tab->d_nd]->obj;
		    tot = ima->b.xytot * sizeof (Unt16);
		    tab->adr1 = ima->tab;
		} break;
		default : 
#		    ifdef DEBUG
		    printf ("ERREUR : type de data non prevu\n");
#		    endif
		    tot = 0;
		    tab->adr1 = NULL;
	    }
	    sprintf (bla, "Receiving data %d (%dk)", tab->d_nd, tot/1024);
	    cs_footer (cs_ctx, bla);

	    sprintf (mSnd, "348 %d %d %d", tab->d_nd, IPS_IMAGE, tot); /* ASK */
	}
        else sprintf (mSnd, "360"); /* READY */
        cs_sendto_str ();
    } break;

    /* ------------------------------------------------------------------------- EXE_REP
     * "EXE_REP nb_bloc taille_bloc taille_totale"
     *
     * t_o donne infos buffer
     * En retour demande du premier bloc
    */
    case 351 : {

	/* lecture parametres buffer */
	sscanf (cs_mRcv+motd, " %d %d %d", &(tab->d_nb), &(tab->d_tb), &(tab->d_tt));

	/* Reutilisation buffer ; BLOC:55 travaille avec tab->adr1 */
	/* deja fait dans EXE_NEXT:357 */

	sprintf (mSnd, "52 0"); /* SEND */
	cs_sendto_str ();
    } break;

    /* ------------------------------------------------------------------------- EXE_MAJ
     * "EXE_MAJ infos"
     *
     * Le client dit que tout est envoye', on peut mettre a jour la parbox.
     * En retour : STAT
    */
    case 365 : {

	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;

	/* appel de telop_decode_cho */
	telop_decode_cho (ico, tab->data1, tab->d_tt);

	/* liberation data */
	pcp_free (tab->data1);
	tab->data1 = (void *) NULL;

	sprintf (mSnd, "%d", ordre+3);
	cs_sendto_str ();
    } break;

    /* ----------------------------------------------------------------------- EXE_INFOS
     * "EXE_INFOS infos"
     *
     * Le client donne des infos a afficher dans le footer : le temps, etc.
     * On met a jour egalement les images et l'icone du telop.
     * En retour : mise en attente, ou section FIG 400.
    */
    case 371 : {

	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;
	char bla[256];

	/* maj resultats */
	telop_maj_inout (ico); /* appele' avant de changer led ou ctx */

	/* maj icone operateur */
	sprintf (bla, "EXE done%s", cs_mRcv+motd);
	cs_footer (cs_ctx, bla);
	telop_change_led (ico, LED_SLEEP);

	/* Si cet operateur a des figures, alors il peut avoir des evenements stockes
	 * (en particulier des REFRESH IN+OUT provoques par telop_maj_inout).
	 * On relance dans ce cas la requete 400.
	*/
	if (op->fig != NULL && op->fig->ev_top != 0) {
	    sprintf (mSnd, "400 %s", op->idf); /* section FIG */
	} else {
	    tab->working = FAUX; /* Le dialogue est fini */
	    sprintf (mSnd, "22"); /* WAIT */
	}
	cs_sendto_str ();
    } break;

    /* ----------------------------------------------------------------------- EXE_SPY
     * "EXE_SPY pourcent"
     *
     * Le client donne le pourcentage de l'execution re'alise' (r = 0..100).
     * Pas de retour.
    */
    case 399 : {

	int r;

	sscanf (cs_mRcv+motd, " %d", &r);
	r = LED_RUN_LO + (r * (LED_RUN_HI - LED_RUN_LO)) / 100;
	if (r < LED_RUN_LO) r = LED_RUN_LO;
	else if (r > LED_RUN_HI) r = LED_RUN_HI;

	telop_change_led (tab->ico, r); /* run% */
    } break;

    /* ----------------------------------------------------------------------- FIG_EVT
     * "FIG_EVT"
     *
     * Le client demande l'envoi des evenements.
     * En retour : liste d'evenements.
    */
    case 411 : {

	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;
	Fig_data *fig = op->fig;  

	/* on envoie les evenements (liste terminee par "0 0")
	 * puis on vide la liste
	*/
	sprintf (mSnd, "%d %s 0 0", ordre+3, fig->ev_list);
	fig_vide_ev_list (fig);
	cs_sendto_str ();
    } break;

    /* ----------------------------------------------------------------------- FIG_DRAW
     * "FIG_DRAW"
     *  Dessins recus, fait dessins, libere datas.
     *  Retour : va en 422 ou 424 si evenements stockes
    */
    case 419 : {

	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;
	Fig_data *fig = op->fig;  

	/* on fait les dessins */
	fig_decode_dessins (ico, tab->data1);

	/* liberation data */
	pcp_free (tab->data1);
	tab->data1 = (void *) NULL;

	/* Test si evenements stockes */
	if (fig->ev_top != 0 && fig->do_run == FAUX) {
	    /* Premiere chance pour envoyer nouveaux evenements ;
	     * - on envoie les evenements restants et on vide la liste
	     * - on ne passe pas par encode_cho/decode_cho.
	     */
	    sprintf (mSnd, "422 %s", fig->ev_list);
	    fig_vide_ev_list (fig); 
	} else {
	    sprintf (mSnd, "424"); /* READY */
	}
	cs_sendto_str ();
    } break;


    /* ----------------------------------------------------------------------- FIG_MAJ
     * "FIG_MAJ"
     *  Met a jour parbox, puis Working=0 si il n'y a plus d'evts stockes.
     *  Retour : DODO ou retourne en 400.
    */
    case 429 : {

	Ico *ico = tab->ico;
	Telop *op = (Telop *) ico->obj;
	Fig_data *fig = op->fig;
	int test;

	cs_footer (cs_ctx, "FIG ... done");

	/* appel de telop_decode_cho */
	telop_decode_cho (ico, tab->data1, tab->d_tt);

	/* liberation data */
	pcp_free (tab->data1);
	tab->data1 = (void *) NULL;

	/* Si do_run, on teste si un Run est possible et on prepare les donnees */
	if (test = fig->do_run) {
	    fig->do_run = FAUX;
	    test = (telop_check_run (ico, FAUX) == 0);
	} 

	if (test) {
	    /* Le Run est possible, et les donnees sont preparees */
	    sprintf (mSnd, "300 %s", op->idf); /* section DIM/EXE */
	} else if (fig->ev_top != 0) {
	    /* Seconde chance pour envoyer nouveaux evenements ;
	     * On a deja fait encode_cho/decode_cho, alors on relance la requete 400.
	    */
	    sprintf (mSnd, "400 %s", op->idf); /* section FIG */
	} else {
	    tab->working = FAUX; /* Le dialogue est fini */
	    sprintf (mSnd, "24"); /* DODO */
	}
	cs_sendto_str ();
    } break;


    /* ----------------------------------------------------------------------------- 
     * Erreur
    */
    default : 
#	ifdef DEBUG
	printf ("ERROR in cs_input : Ordre %d inconnu !\n", ordre);
#	endif
	tab->working = FAUX; /* Le dialogue est fini */
	sprintf (mSnd, "22"); /* WAIT */
	cs_sendto_str ();

    } /* switch #1 */
}

