/*
    coulx.c - Edouard Thiel - 26/02/1999

    Compilation :	cc -c -Wall coulx.c

    Module permettant de calculer et afficher des Ximage.
*/

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "coulx.h"

typedef union {
    unsigned long  l; 
    unsigned short s[2];
    unsigned char  c[4]; 
} Ent421;

static unsigned long c_LUTR[256], c_LUTG[256], c_LUTB[256];

static int c_decaR, c_decaG, c_decaB, 
           c_multR, c_multG, c_multB,
           c_bpp, c_byte, c_client_bo, c_server_bo,
           c_mode_lut, c_force_lut = 0, c_cas = 0, c_depth,
           c_n0, c_n1, c_n2, c_n3;
           
static Display *c_display;
static Visual *c_visual;
           
enum {
	C_ERROR, 
        C_BPP8_LUT, C_BPP16_LUT, C_BPP24_LUT, C_BPP32_LUT, 
        C_BPP24_RAW, C_BPP32_RAW
};

static char *c_cas_strings[] = {
	"C_ERROR", 
        "C_BPP8_LUT", "C_BPP16_LUT", "C_BPP24_LUT", "C_BPP32_LUT", 
        "C_BPP24_RAW", "C_BPP32_RAW"
};


void CoulxAnalyseMask (unsigned long mask, int *mult, int *deca)
{
    int i;
    
    for (i = 0; i < 32; i++)
	if (mask & 1L << i) break;
    
    *deca = i;
    *mult = mask >> i;
}

void CoulxEchange2 (int *a, int *b)
{
    int tmp;
    tmp = *a; *a = *b; *b = tmp;
}

void CoulxEchangeN (int *a, int *b, int *c, int *d, int n)
{
    switch (n) {
      case 2 : CoulxEchange2 (a, b); break;
      case 3 : CoulxEchange2 (a, c); break;
      case 4 : CoulxEchange2 (a, d); CoulxEchange2 (b, c); break;
    }
}

void CoulxPreInit (int force_lut)
{
    c_force_lut = force_lut;
}


void CoulxInit (Display *display, Visual *visual, int depth)
{
    XImage *xi;
    int i;
    Ent421 val;
    
    /* Mmorisation */
    c_display = display; c_visual = visual; c_depth = depth;
    
    /* On cr une XImage pour extraire des paramtres  (v.2 p.170).
    */   
    xi = XCreateImage (c_display, c_visual, c_depth, ZPixmap, 0, NULL,
	16, 16, 8, 0);
    c_bpp = xi->bits_per_pixel;
    c_byte = c_bpp / 8;
    
    CoulxAnalyseMask (xi->red_mask,   &c_multR, &c_decaR);
    CoulxAnalyseMask (xi->green_mask, &c_multG, &c_decaG);
    CoulxAnalyseMask (xi->blue_mask,  &c_multB, &c_decaB);
    
    /* Calcul des LUT 
     *
     * Dans le mode LUT, on utilise les prcalculs sur les masques ;
     * dans le mode RAW, on recopie directement les octets (+ efficace).
    */
    if (c_force_lut)
         c_mode_lut = 1;
    else c_mode_lut = (!(c_multR == 255 && c_multG == 255 && c_multB == 255));
    
    if (c_mode_lut)
    for (i = 0; i < 256; i++) {
	c_LUTR[i] =  (i * c_multR / 255) << c_decaR;
	c_LUTG[i] =  (i * c_multG / 255) << c_decaG;
	c_LUTB[i] =  (i * c_multB / 255) << c_decaB;
    } 
    
    /* Calcul du cas
    */
    switch (c_bpp) {
      case  8 : c_cas = C_BPP8_LUT;
                break;
      case 16 : c_cas = C_BPP16_LUT;
                break;
      case 24 : c_cas = (c_mode_lut) ? C_BPP24_LUT : C_BPP24_RAW;
                break;
      case 32 : c_cas = (c_mode_lut) ? C_BPP32_LUT : C_BPP32_RAW;
                break;
      default : c_cas = C_ERROR;
    }
    
    /* Test du byte order : 
     *    LSBFirst = Little-endian = PC i86      : 0 1 2 3 
     *    MSBFirst = Big-endian    = SGI/Sun/Mac : 3 2 1 0
     *
     * Non gr : le Middle ou PDP endian de DEC : 2 3 0 1
    */
    val.l = 1;
    c_client_bo = (val.c[0] == 1) ? LSBFirst : MSBFirst;
    c_server_bo = xi->byte_order;
    
    /* Calcul des constantes c_n0, .. c_n3
     *
     * Attention, elles ne jouent pas le mme rle selon le mode :
     * - Mode LUT : numros d'octets dans un long
     * - Mode RAW : offset dans data[]
    */
    if (c_mode_lut) {
    
	/* Ct client, on numrote les octets dans un long */
        if (c_client_bo == LSBFirst)
             { c_n0 = 0; c_n1 = 1; c_n2 = 2; c_n3 = 3; }
        else { c_n0 = 3; c_n1 = 2; c_n2 = 1; c_n3 = 0; }
            
	/* Ct server, on change c_byte octets  mettre dans data */
	if (c_server_bo == MSBFirst)
            CoulxEchangeN (&c_n0, &c_n1, &c_n2, &c_n3, c_byte);
            
    } else {
    
        /* On rcupre la position de R,G,B dans le masque */
	c_n0 = c_decaR/8; c_n1 = c_decaG/8; c_n2 = c_decaB/8;
        
        /* Position du 0 pour complter, la somme faisant toujours 6 */
        c_n3 = 6 - c_n0 - c_n1 - c_n2;
        
	/* Ct server, on soustrait les offsets  c_byte-1 */
	if (c_server_bo == MSBFirst) {
            c_n0 = c_byte-1 - c_n0; c_n1 = c_byte-1 - c_n1;
            c_n2 = c_byte-1 - c_n2; c_n3 = c_byte-1 - c_n3;
        }
    }
    
    XDestroyImage (xi);         
}


void CoulxAffiInfos ()
{
    printf ("client : %s  ", c_client_bo == LSBFirst ? "PC" : "SIL");
    printf ("server : %s\n", c_server_bo == LSBFirst ? "PC" : "SIL");
    printf ("depth = %d  bpp = %d  bytes = %d\n", c_depth, c_bpp, c_byte);
    printf ("RGB = %d << %d  %d << %d  %d << %d\n", 
	c_multR, c_decaR, c_multG, c_decaG, c_multB, c_decaB);
    printf ("cas = %s (%d %d %d %d)\n", 
        c_cas_strings[c_cas], c_n0, c_n1, c_n2, c_n3);
}


XImage *CoulxCalcXImage (unsigned char *tabR, unsigned char *tabG, 
	unsigned char *tabB, int width, int height)
{
    int r, t, size = width * height;
    XImage *xi;
    char *data;
    Ent421 val;

    /* Cration XImage  v.2 p.170
     * On met bitmap_pad = 8 systmatiquement
    */
    xi = XCreateImage (c_display, c_visual, c_depth, ZPixmap, 0, NULL,
	width, height, 8, 0);
    
    /* Allocation data
    */
    data = (char *) malloc (size * c_byte);
    if (data == NULL) {
        printf ("ERREUR malloc (%d octets)\n", size * c_byte);
        XDestroyImage (xi);
        return NULL;
    }
    xi->data = data;
    
    /* Calcul data 
    */
    switch (c_cas) {
    
      case C_BPP8_LUT :
	for (t = 0; t < size; t++) {
	    val.l = c_LUTR[tabR[t]] | c_LUTG[tabG[t]] | c_LUTB[tabB[t]];
	    data[t*c_byte  ] = val.c[c_n0]; 
	}
	break;
        
      case C_BPP16_LUT :
	for (t = 0; t < size; t++) {
	    val.l = c_LUTR[tabR[t]] | c_LUTG[tabG[t]] | c_LUTB[tabB[t]];
            r = t*c_byte;
	    data[r  ] = val.c[c_n0]; 
	    data[r+1] = val.c[c_n1]; 
	}
	break;
        
      case C_BPP24_LUT :
	for (t = 0; t < size; t++) {
	    val.l = c_LUTR[tabR[t]] | c_LUTG[tabG[t]] | c_LUTB[tabB[t]];
            r = t*c_byte;
	    data[r  ] = val.c[c_n0]; 
	    data[r+1] = val.c[c_n1]; 
	    data[r+2] = val.c[c_n2]; 
	}
	break;
        
      case C_BPP32_LUT :
	for (t = 0; t < size; t++) {
	    val.l = c_LUTR[tabR[t]] | c_LUTG[tabG[t]] | c_LUTB[tabB[t]];
            r = t*c_byte;
            data[r  ] = val.c[c_n0];
	    data[r+1] = val.c[c_n1];
	    data[r+2] = val.c[c_n2];
	    data[r+3] = val.c[c_n3];
	}
	break;
        
      case C_BPP24_RAW:
	for (t = 0; t < size; t++) {
            r = t*c_byte;
	    data[r+c_n0] = tabR[t];
	    data[r+c_n1] = tabG[t];
	    data[r+c_n2] = tabB[t];
	}
	break;
                
      case C_BPP32_RAW :
	for (t = 0; t < size; t++) {
            r = t*c_byte;
	    data[r+c_n0] = tabR[t];
	    data[r+c_n1] = tabG[t];
	    data[r+c_n2] = tabB[t];
	    data[r+c_n3] = 0; 
	}
	break;
        
      default : printf ("ERREUR cas oubli\n");
    }
 
    /* Fin */    
    return xi;
}


void CoulxAffiImage (Window win, GC gc, XImage *xi, int dest_x, int dest_y)
{
    /* Affichage image  v.2 p.478 */
    XPutImage (c_display, win, gc, xi, 0, 0, dest_x, dest_y,
	xi->width, xi->height);
}


void CoulxLibererImage (XImage *xi)
{
    /* Libre xi et xi->data  v.2 p.196 */
    XDestroyImage (xi);              
}

