/***************************************************************************************/
/* edi_image.c                                                I.P.S  v 3.3  18/3/1994  */
/*                                                                                     */
/*                                                       Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"


/***************************************************************************************
 * Appel de l'editeur pour une ou deux images                                          *
 *                                                                                     *
 ***************************************************************************************/

#define tools_list	"undo","clear","inverse", \
			"< a --> b","> a --> b", " a ---> b"," a <--> b"
#define acts_list	"move","draw","line","rectang","circle","fill", \
			"isolate","print","capture"

void edIMA_edit_ico (clic, regarde_selection)
Ico *clic;
int regarde_selection;
{
    Ico *v_ico[5];
    int nombre;

    if (regarde_selection) {
	if (!ico_sel_scan3 (clic, v_ico, 2, &nombre)) return;

	if ( (v_ico[0] == NULL || v_ico[0]->nat_obj != IPS_IMAGE) ||
	     (v_ico[1] != NULL && v_ico[1]->nat_obj != IPS_IMAGE) ) {
	    pcp_avertissement ("WARNING :","that selection cannot be edited !",
		"Select one or two images.");
	    return;
	}
    } else {
	v_ico[0] = clic;
	v_ico[1] = NULL;
    }

    edi_create_win (
	EDI_ICO_2IMAGES, v_ico[0], v_ico[1],
	EDI_TOOLS, "Tools", edIMA_tools_proc, tools_list, NULL,
	EDI_VAL_A, "a :", 0, 65535, 1,
	EDI_VAL_B, "b :", 0, 65535, 0,
	EDI_ACT_A, 0, acts_list, NULL,
	EDI_ACT_B, 0, acts_list, NULL,
	EDI_EVENT_PROC, edIMA_event_proc,
	NULL);
}


/****************************************************************************************
 * EDI_TOOLS_PROC : edIMA_tools_proc
 *
 ***************************************************************************************/

int edIMA_tools_proc (edi, choice)
Edi  *edi;
char *choice;
{
  int       compt_a = EDI_GET_VAL_A(edi);
  int       compt_b = EDI_GET_VAL_B(edi);
  int       value, x, y, t;
  int       xmax = edi->xsiz1 + edi->cmin1;
  int       ymax = edi->ysiz1 + edi->cmin1;
  Unt16     c;

  if      (!strcmp(choice, "Undo"     )) value = 1;
  else if (!strcmp(choice, "clear"    )) value = 2;
  else if (!strcmp(choice, "inverse"  )) value = 3;
  else if (!strcmp(choice, "< a --> b")) value = 4;
  else if (!strcmp(choice, "> a --> b")) value = 5;
  else if (!strcmp(choice, " a ---> b")) value = 7;
  else if (!strcmp(choice, " a <--> b")) value = 8;
  else value = 1;

  if (edi->tab1 == NULL) return; /* protection */

  if (value > 1) if (edi_undo_save (edi) < 0) return;
 
  switch (value)
  {
    case 1 : /* annuler */
	edi_undo_now (edi);
        break;
    case 2 :  /* vider */
        for (y = edi->cmin1; y < ymax; y++)
        for (x = edi->cmin1; x < xmax; x++)
        edi->tab1[y*edi->xtot1+x] = 0;
        break;
    case 3 :  /* inverser */
        for (y = edi->cmin1; y < ymax; y++)
        for (x = edi->cmin1; x < xmax; x++)
        { t = y*edi->xtot1+x;
          if (edi->tab1[t] == 0)
               edi->tab1[t] = 255;
          else edi->tab1[t] = 0;
        }
        break;
    case 4 :  /* < a --> b */
        for (y = edi->cmin1; y < ymax; y++)
        for (x = edi->cmin1; x < xmax; x++)
        { t = y*edi->xtot1+x;
          if (edi->tab1[t] < compt_a)
               edi->tab1[t] = compt_b;
        }
        break;        
    case 5 :  /* > a --> b */
        for (y = edi->cmin1; y < ymax; y++)
        for (x = edi->cmin1; x < xmax; x++)
        { t = y*edi->xtot1+x;
          if (edi->tab1[t] > compt_a)
               edi->tab1[t] = compt_b;
        }
        break;     
    case 7 :  /* a ---> b */
        for (y = edi->cmin1; y < ymax; y++)
        for (x = edi->cmin1; x < xmax; x++)
        { t = y*edi->xtot1+x;
          if (edi->tab1[t] == compt_a)
               edi->tab1[t] = compt_b;
        }
        break;
    case 8 :  /* a <--> b */
        for (y = edi->cmin1; y < ymax; y++)
        for (x = edi->cmin1; x < xmax; x++)
        { t = y*edi->xtot1+x;
          if (edi->tab1[t] == compt_a)
               edi->tab1[t] = compt_b;
          else if (edi->tab1[t] == compt_b)
               edi->tab1[t] = compt_a;
        }
        break;
  }

  return VRAI; /* on demande de redessiner */
}


/****************************************************************************************
 * dessin d'un segment de droite par bresenham avec edi->val_cur
 *
 ***************************************************************************************/

void edIMA_bresenham (edi, x1, y1, x2, y2)
Edi     *edi;
int      x1, y1, x2, y2;
{
  int  a, x, y, dx, dy, px, py, incr1, incr2, d;
  char test;
  int  tx = edi->bxmin + edi->cmin1;
  int  ty = edi->bymin + edi->cmin1;

  /* initialisations */
  dx = x2 - x1; dy = y2 - y1;
  test = (abs(dy) <= abs(dx));  /* pente <= 1 */
  if ((test && dx < 0) || (!test && dy < 0))
    {
      a = x1; x1 = x2; x2 = a;
      a = y1; y1 = y2; y2 = a;
      dx = -dx; dy = -dy;
    }

  x = x1; y = y1;

  if (0 <= x && x < edi->xsiz1 && 0 <= y && y < edi->ysiz1)
    {
      edi->tab1[(ty + y)*edi->xtot1 + tx + x] = edi->val_cur;
      edi_dessin_point (edi, x, y, edi->val_cur);
    }

  /* trace de la droite */
  if (test)
       { /* pente <= 1 */
         if (dy >= 0) py = 1; else { py = -1; dy = -dy; }
         incr1 = 2*dy; d = incr1 - dx; incr2 = d - dx;

         while (x < x2)
         {
           x += 1;
           if (d < 0) d += incr1; else { d += incr2; y += py; }

           if (0 <= x && x < edi->xsiz1 && 0 <= y && y < edi->ysiz1)
           {
             edi->tab1[(ty + y)*edi->xtot1 + tx + x] = edi->val_cur;
             edi_dessin_point (edi, x, y, edi->val_cur);
           }
         } /* while */
       } /* pente <= 1 */

  else { /* pente > 1  */
         if (dx >= 0) px = 1; else { px = -1; dx = -dx; }
         incr1 = 2*dx; d = incr1 - dy; incr2 = d - dy;

         while (y < y2)
         {
           y += 1;
           if (d < 0) d += incr1; else { d += incr2; x += px; }

           if (0 <= x && x < edi->xsiz1 && 0 <= y && y < edi->ysiz1)
           {
             edi->tab1[(ty + y)*edi->xtot1 + tx + x] = edi->val_cur;
             edi_dessin_point (edi, x, y, edi->val_cur);
           }
         } /* while */
       } /* pente > 1  */
}


/****************************************************************************************
 * remplissage d'un contour ferme par edi->val_cur (pas besoin de cadre externe)
 *
 * Le contour est 8-connexe donc on remplis en 4-connexite'.
 *
 ***************************************************************************************/

void edIMA_remplissage (edi, x1, y1)
Edi   *edi;
int    x1, y1;
{
  int  i, j, p, x, y, xa, ya, ta;
  int  pilx[10000], pily[10000];
  int  bmin  = edi->cmin1;
  int  bxmax = edi->cmin1 + edi->xsiz1;
  int  bymax = edi->cmin1 + edi->ysiz1;
  int  d2 = -2;
  int  d3 = 1+edi->xtot1;
  int  d4 = -2*edi->xtot1;

  /* initialisation */
  pilx[0] = x1+edi->cmin1; pily[0] = y1+edi->cmin1; p = 1; 
  edi->tab1[pilx[0]+pily[0]*edi->xtot1] = edi->val_cur;

  do
  {
    x = pilx[--p]; y = pily[p];

    xa = x + 1; ya = y; ta = xa+ya*edi->xtot1;
    if (xa < bxmax && edi->tab1[ta] != edi->val_cur) {
        pilx[p] = xa; pily[p++] = ya; edi->tab1[ta] = edi->val_cur;
    }
    xa = x - 1; ta += d2;
    if (xa >= bmin && edi->tab1[ta] != edi->val_cur) {
        pilx[p] = xa; pily[p++] = ya; edi->tab1[ta] = edi->val_cur;
    }
    xa = x; ya = y + 1; ta += d3;
    if (ya < bymax && edi->tab1[ta] != edi->val_cur) {
        pilx[p] = xa; pily[p++] = ya; edi->tab1[ta] = edi->val_cur;
    }
    ya = y - 1; ta += d4;
    if (ya >= bmin && edi->tab1[ta] != edi->val_cur) {
        pilx[p] = xa; pily[p++] = ya; edi->tab1[ta] = edi->val_cur;
    }

    if (p >= 9995)  /* on vide la pile */
    {
      j = 0;
      while (j < p)
      {
        xa = pilx[j]; ya = pily[j]; ta = xa+ya*edi->xtot1;
        if ( (xa+1 >= bxmax || edi->tab1[ta+1] == edi->val_cur)
          && (xa-1 <  bmin  || edi->tab1[ta-1] == edi->val_cur)
          && (ya+1 >= bymax || edi->tab1[ta+edi->xtot1] == edi->val_cur)
          && (ya-1 <  bmin  || edi->tab1[ta-edi->xtot1] == edi->val_cur) ) {
            pilx[j] = pilx[--p]; pily[j] = pily[p];
        }
        else j++;
      } /* while j */
    } /* if p */
  }
  while (0 < p && p < 9995);
}


/****************************************************************************************
 * isolation d'un objet 8-connexe situe' en x1, y1
 * resultat : x1,y1 et x2,y2 encadrent l'objet.
 *
 ***************************************************************************************/
int edIMA_isolate (edi, tab3, xytot)
Edi   *edi;
Unt16 *tab3;
int   xytot;
{
  int  i, j, p, x, y, xa, ya, ta, t;
  int  pilx[10000], pily[10000];
  int  bmin  = edi->cmin1;
  int  bxmax = edi->cmin1 + edi->xsiz1;
  int  bymax = edi->cmin1 + edi->ysiz1;

  /* initialisation */
  pilx[0] = edi->x1; pily[0] = edi->y1; p = 1;
  t = pilx[0]+pily[0]*edi->xtot1;
  if (edi->tab1[t] == 0) return -1;

  /* init tab3 */
  tab3[t] = edi->tab1[t];
  for (t = 0; t < xytot; t++) tab3[t] = 0;

  do
  {
    x = pilx[--p]; y = pily[p];

    if (x < edi->x1) edi->x1 = x;
    else if (x > edi->x2) edi->x2 = x;
    if (y < edi->y1) edi->y1 = y;
    else if (y > edi->y2) edi->y2 = y;

    xa = x-1; ya = y-1; ta = xa+ya*edi->xtot1;
    if (xa >= bmin && ya >= bmin && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }
    xa++; ta++;
    if (ya >= bmin && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }
    xa++; ta++;
    if (xa < bxmax && ya >= bmin && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }
    xa = x-1; ya++; ta = xa+ya*edi->xtot1;
    if (xa >= bmin && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }
    xa+= 2; ta += 2;
    if (xa < bxmax && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }
    xa = x-1; ya++; ta = xa+ya*edi->xtot1;
    if (xa >= bmin && ya < bymax && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }
    xa++; ta++;
    if (ya < bymax && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }
    xa++; ta++;
    if (xa < bxmax && ya < bymax && edi->tab1[ta] > 0 && tab3[ta] == 0) {
        pilx[p] = xa; pily[p++] = ya; tab3[ta] = edi->tab1[ta];
    }

    if (p >= 9995)  /* on vide la pile */
    {
      j = 0;
      while (j < p)
      {
        xa = pilx[j]; ya = pily[j]; ta = xa+ya*edi->xtot1;
        if ( (xa-1 <  bmin  || ya-1 < bmin   || tab3[ta-1-edi->xtot1] > 0)
          && (                 ya-1 < bmin   || tab3[ta  -edi->xtot1] > 0)
          && (xa+1 >= bxmax || ya-1 < bmin   || tab3[ta+1-edi->xtot1] > 0)
          && (xa-1 <  bmin                   || tab3[ta-1           ] > 0)
          && (xa+1 >= bxmax                  || tab3[ta+1           ] > 0)
          && (xa-1 <  bmin  || ya+1 >= bymax || tab3[ta-1+edi->xtot1] > 0)
          && (                 ya+1 >= bymax || tab3[ta  +edi->xtot1] > 0)
          && (xa+1 >= bxmax || ya+1 >= bymax || tab3[ta+1+edi->xtot1] > 0) ) {
            pilx[j] = pilx[--p]; pily[j] = pily[p];
        }
        else j++;
      } /* while j */
    } /* if p */
  }
  while (0 < p && p < 9995);

  if (edi->x1 > bmin) edi->x1 -= 1;
  if (edi->y1 > bmin) edi->y1 -= 1;
  if (edi->x2 < bxmax-1) edi->x2 += 1;
  if (edi->y2 < bymax-1) edi->y2 += 1;
  return 0;
}

/****************************************************************************************
 * Trace du dessin courant en xor
 *
 ***************************************************************************************/

void edIMA_dessin_xor (edi)
Edi *edi;
{
  int x, y, r;

  switch (edi->act_cur)
  {
    case 2 :  /* ligne */
        XDrawLine (edi->dpy, edi->xwin, edi->gc,
            edi->x1, edi->y1, edi->x2, edi->y2);
        break;
    case 3 :  /* rectangle */
    case 7 :  /* print area */
    case 8 :  /* capture */
      {
	int x1, y1, x2, y2;
	if (edi->x1 < edi->x2)
	     { x1 = edi->x1; x2 = edi->x2; }
	else { x1 = edi->x2; x2 = edi->x1; }
	if (edi->y1 < edi->y2)
	     { y1 = edi->y1; y2 = edi->y2; }
	else { y1 = edi->y2; y2 = edi->y1; }
	XDrawRectangle (edi->dpy, edi->xwin, edi->gc, x1, y1, x2-x1+1, y2-y1+1);
      } break;
    case 4 :  /* cercle */
        x = edi->x2-edi->x1+1;
        y = edi->y2-edi->y1+1;
        r = (int) (sqrt((double) x*x + y*y));
        XDrawArc (edi->dpy, edi->xwin, edi->gc,
            edi->x1-r, edi->y1-r, r*2, r*2, 0, 360*64);
        break;
  }
}


/****************************************************************************************
 * EDI_EVENT_PROC : edIMA_event_proc
 * 
 ***************************************************************************************/

int edIMA_event_proc (edi, acte)
Edi  *edi;
int   acte;
{
    int x, y, xa, xb, xc, ya, yb, yc, c, r, t, repeindre = FAUX;

    if (edi->tab1 == NULL) return FAUX;

  /*
   * action selon `acte' et `dessin'
  */

  switch(edi->act_cur*10 + acte)
  {
    /* Dessin NORMAL */
    case 11 : /* bouton 1,2 enfonce */
        edi->x1 = edi->sx / edi->zoom;
        edi->y1 = edi->sy / edi->zoom;
        edi->tab1[(edi->cmin1 + edi->bymin + edi->y1)*edi->xtot1
                  +edi->cmin1 + edi->bxmin + edi->x1] = edi->val_cur;
        edi_dessin_point (edi, edi->x1, edi->y1, edi->val_cur);
        break;
    case 13 : /* bouton 1,2 tire */ 
        edi->x2 = edi->x1;
        edi->y2 = edi->y1;
        edi->x1 = edi->sx / edi->zoom;
        edi->y1 = edi->sy / edi->zoom;
        edIMA_bresenham (edi, edi->x1, edi->y1, edi->x2, edi->y2);
        break;

    /* Dessin LIGNE */
    case 22 : /* bouton 1,2 relache */
        edIMA_dessin_xor (edi);
        XSetFunction (edi->dpy, edi->gc, GXcopy);
        edIMA_bresenham (edi, edi->x1/edi->zoom, edi->y1/edi->zoom, 
                        edi->x2/edi->zoom, edi->y2/edi->zoom);
        break;

    /* Dessin RECTANGLE */
    case 32 : /* bouton 1,2 relache */
        edIMA_dessin_xor (edi);
        XSetFunction (edi->dpy, edi->gc, GXcopy);
	if (edi_undo_save (edi) < 0) break;
        xa = edi->x1 / edi->zoom + edi->bxmin + edi->cmin1;
        xb = edi->x2 / edi->zoom + edi->bxmin + edi->cmin1;
        ya = edi->y1 / edi->zoom + edi->bymin + edi->cmin1;
        yb = edi->y2 / edi->zoom + edi->bymin + edi->cmin1;
        if (xa > xb) { c = xa; xa = xb; xb = c; }
        if (ya > yb) { c = ya; ya = yb; yb = c; }
        for (y = ya; y <= yb; y++)
        for (x = xa; x <= xb; x++)
        edi->tab1[y*edi->xtot1+x] = edi->val_cur;
        repeindre = VRAI;
       break;

    /* Dessin CERCLE */
    case 42 : /* bouton 1,2 relache */
        edIMA_dessin_xor (edi);
        XSetFunction (edi->dpy, edi->gc, GXcopy);
	if (edi_undo_save (edi) < 0) break;
        xc = edi->x1 / edi->zoom + edi->bxmin + edi->cmin1;
        yc = edi->y1 / edi->zoom + edi->bymin + edi->cmin1;
        xa = edi->x2 / edi->zoom + edi->bxmin + edi->cmin1;
        ya = edi->y2 / edi->zoom + edi->bymin + edi->cmin1;
        t  = (xc-xa)*(xc-xa) + (yc-ya)*(yc-ya);
        r  = (int) (sqrt((double) t));
        xa = xc - r; if (xa < edi->cmin1) xa = edi->cmin1;
        ya = yc - r; if (ya < edi->cmin1) ya = edi->cmin1;
        xb = xc + r; if (xb > edi->cmin1 + edi->xsiz1) xb = edi->cmin1 + edi->xsiz1 - 1;
        yb = yc + r; if (yb > edi->cmin1 + edi->ysiz1) yb = edi->cmin1 + edi->ysiz1 - 1;
        for (y = ya; y <= yb; y++)
        for (x = xa; x <= xb; x++)
        if ( (xc-x)*(xc-x) + (yc-y)*(yc-y) <= t )
        edi->tab1[y*edi->xtot1+x] = edi->val_cur;
        repeindre = VRAI;
        break;

    /* Dessin print zone */
    case 72 : /* bouton 1,2 relache */
        edIMA_dessin_xor (edi);
        XSetFunction (edi->dpy, edi->gc, GXcopy);
        xa = edi->x1 / edi->zoom + edi->bxmin + edi->cmin1;
        xb = edi->x2 / edi->zoom + edi->bxmin + edi->cmin1;
        ya = edi->y1 / edi->zoom + edi->bymin + edi->cmin1;
        yb = edi->y2 / edi->zoom + edi->bymin + edi->cmin1;
        if (xa > xb) { c = xa; xa = xb; xb = c; }
        if (ya > yb) { c = ya; ya = yb; yb = c; }
        printf("\n\nIMAGE :\n\n");
        for (y = ya; y <= yb; y++) {
          for (x = xa; x <= xb; x++)
            printf("%4d",edi->tab1[y*edi->xtot1+x]);
          printf("\n");          
        }
        printf("\n\n");
        break;

    /* capture : creation icone */
    case 82 : { /* bouton 1,2 relache */
        Ico *ico;
        Image *ima;
        int xsiz, ysiz;

        edIMA_dessin_xor (edi);
        XSetFunction (edi->dpy, edi->gc, GXcopy);
        xa = edi->x1 / edi->zoom + edi->bxmin + edi->cmin1;
        xb = edi->x2 / edi->zoom + edi->bxmin + edi->cmin1;
        ya = edi->y1 / edi->zoom + edi->bymin + edi->cmin1;
        yb = edi->y2 / edi->zoom + edi->bymin + edi->cmin1;
        if (xa > xb) { c = xa; xa = xb; xb = c; }
        if (ya > yb) { c = ya; ya = yb; yb = c; }
        xsiz = xb - xa + 1;
        ysiz = yb - ya + 1;
        ico = ico_creat (glo.sx, glo.sy, IPS_IMAGE);
        ico_tete_pz (ico);
        ima = (Image *) ico->obj;
        image_creat (ima, 0, xsiz, ysiz, NULL);
        t = 0;
        for (y = ya; y <= yb; y++)
          for (x = xa; x <= xb; x++)
            ima->tab[t++] = edi->tab1[y*edi->xtot1+x];
        sprintf (ima->ch1, "capture");
        image_calc_hublot (ico);
        ima->coul_tab = edi->coul_tab1;
        ico_douane (ico);
        ico_refresh_now (ico);
        glo.sx += 10; glo.sy += 16;
      } break;

    /* Dessin ligne, rectangle, cercle, print area */
    case 21 :
    case 31 : /* bouton 1,2 enfonce */
    case 41 :
    case 71 :
    case 81 :
        edi->x1 = edi->x2 = edi->sx;
        edi->y1 = edi->y2 = edi->sy;
        XSetFunction (edi->dpy, edi->gc, GXxor);
        XSetForeground (edi->dpy, edi->gc, coul_16c[14]);
        edIMA_dessin_xor (edi);
        break;
    case 23 :
    case 33 : /* bouton 1,2 tire */ 
    case 43 :
    case 73 :
    case 83 :
        edIMA_dessin_xor (edi);
        edi->x2 = edi->sx;
        edi->y2 = edi->sy;
        edIMA_dessin_xor (edi);
        break;
    case 24 :
    case 34 : /* bouton 1,2 annule */ 
    case 44 :
    case 74 :
    case 84 :
        edIMA_dessin_xor (edi); 
        XSetFunction (edi->dpy, edi->gc, GXcopy);
        break;

    /* Dessin REMPLIR */
    case 51 : /* bouton 1,2 enfonce */
	if (edi_undo_save (edi) < 0) break;
        edi->x1 = edi->sx / edi->zoom + edi->bxmin;
        edi->y1 = edi->sy / edi->zoom + edi->bymin;
        edIMA_remplissage (edi, edi->x1, edi->y1);
        repeindre = VRAI;
        break;


    /* Dessin ISOLATE */
    case 61 : { /* bouton 1,2 enfonce */
        Ico *ico;
        Image *ima;
        int xsiz, ysiz, xytot;
	Unt16 *tab3;

	xytot = edi->xtot1 * (edi->ysiz1 + 2*edi->cmin1);
	tab3 = (Unt16 *) pcp_alloc (xytot * sizeof (Unt16));
	if (tab3 == (Unt16 *) NULL) break;
	
        edi->x1 = edi->x2 = edi->sx / edi->zoom + edi->bxmin + edi->cmin1;
        edi->y1 = edi->y2 = edi->sy / edi->zoom + edi->bymin + edi->cmin1;

        if (edIMA_isolate (edi, tab3, xytot) < 0) {
	    pcp_free (tab3);
	    break;
	}
	xsiz = edi->x2 - edi->x1 +1;
	ysiz = edi->y2 - edi->y1 +1;

        ico = ico_creat (glo.sx, glo.sy, IPS_IMAGE);
        ico_tete_pz (ico);
        ima = (Image *) ico->obj;
        image_creat (ima, 0, xsiz, ysiz, NULL);

        t = 0;
        for (y = edi->y1; y <= edi->y2; y++)
          for (x = edi->x1; x <= edi->x2; x++)
            ima->tab[t++] = tab3[y*edi->xtot1+x];
	pcp_free (tab3);

        sprintf (ima->ch1, "isolated");
        image_calc_hublot (ico);
        ima->coul_tab = edi->coul_tab1;
        ico_douane (ico);
        ico_refresh_now (ico);
        glo.sx += 10; glo.sy += 16;
      } break;

    /* Dessin EXEMPLE */
    case 101 : /* bouton 1,2 enfonce */
        break;
    case 102 : /* bouton 1,2 relache */
        break;
    case 103 : /* bouton 1,2 tire */ 
        break;
    case 104 : /* bouton 1,2 annule */ 
        break;
  }

    return repeindre;
}

