/***************************************************************************************/
/* fichiers.c                                                 I.P.S  v 3.1  18/11/1993 */
/*                                                                                     */
/* module des fichiers                                   Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"

#define cat_NB		3
#define cat_NOMS	"Image", "Voronoi", "Macro"
#define cat_IMAGE	0
#define cat_VOR2D	1
#define cat_MACRO	2

#define for_NOM_L	"Raw bitmap", "PGM"
#define for_NOM_S	"Raw bitmap", "PGM", "EPS"
#define for_RAW		0
#define for_PGM		1
#define for_EPS		2

typedef struct { unsigned char r, g, b; } Uchar3;

static Frame		frame_L, frame_S;

static Panel_item	pL_pwd, pL_name, pL_categ,
			pS_pwd, pS_name, pS_categ, pS_auto_ext;

static int		pL_categ_val = 0,
			pS_categ_val = 0,
			pL_width[cat_NB], pL_height[cat_NB],
			pS_width[cat_NB], pS_height[cat_NB],
			pL_width_com, pL_height_com,
			pS_width_com, pS_height_com;

static Panel		pL_panel[cat_NB], pL_panel_com,
			pS_panel[cat_NB], pS_panel_com;

static Panel_item	pLimage_offset,
			pLimage_num,
			pLimage_ysiz,
			pLimage_xsiz,
			pLimage_format,
			pLimage_bits,
			pLimage_palette,
			pLimage_buttBrowse,
			pLimage_buttWhat,
			pLimage_buttLoad,
			pLimage_buttNext,
			pLimage_buttAll,
			pLimage_buttView,
			pLvor2d_buttLoad;

static Panel_item	pSimage_format,
			pSimage_bits,
			pSimage_compress,
			pSvor2d_format;


void		fichier_load_creer_frame	();
void		fichier_save_creer_frame	();
void		fichier_ana_fin_nom		();
int		fichier_L_What			();
void		fichier_drop			();
int		fichier_L			();
void		fichier_L_image			();
void		fichier_L_vor2d			();
int		fichier_S_test_open		();
int		fichier_S_image			();
int		fichier_S_vor2d			();
void		fichier_new_L_categ		();
int		fichier_L_categ			();
int		fichier_S_categ			();
void		fichier_L_image_grise		();
int		fichier_L_image_grise_proc	();
void		fichier_S_new_extension		();
Panel_item	fichier_FramePcpButtCreat	();
void		fichier_FramePcpMenuProc	();


/***************************************************************************************
 * Creer la boite de dialogue du menu fichier/load
 *
 ***************************************************************************************/
void
fichier_load_creer_frame ()
{
  int   i, categ;
  Panel panel;
  char *thedir;

  /* Cherche le pwd ou a ete lance' ips */
  thedir = (char *) getenv ("PWD");

  frame_L = (Frame) xv_create (glo.frame_principal, FRAME_CMD,
      FRAME_LABEL,              "Load File",
      NULL);

  pL_panel_com = (Panel) xv_get (frame_L, FRAME_CMD_PANEL);
  xv_set (pL_panel_com, PANEL_LAYOUT, PANEL_VERTICAL, NULL);

  pL_pwd = (Panel_item) xv_create (pL_panel_com, PANEL_TEXT,
      PANEL_LABEL_STRING,         "Directory ",
      PANEL_VALUE,                (thedir == NULL) ? "" : thedir,
      PANEL_VALUE_DISPLAY_LENGTH, 39,
      PANEL_VALUE_STORED_LENGTH,  120,
      NULL);

  pL_name = (Panel_item) xv_create (pL_panel_com, PANEL_TEXT,
      PANEL_LABEL_STRING,         "File name",
      PANEL_VALUE,                "",
      PANEL_VALUE_DISPLAY_LENGTH, 39,
      PANEL_VALUE_STORED_LENGTH,  120,
      NULL);

  pL_categ = (Panel_item) xv_create (pL_panel_com, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "Category",
      PANEL_CHOICE_STRINGS,  cat_NOMS, NULL,
      PANEL_VALUE,           0,
      PANEL_NOTIFY_PROC,     fichier_L_categ,
      NULL);

  xv_set (pL_panel_com, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

/* cf ~/SRC/xview/xview-3.2p1.4/lib/libxview/file_chooser/hist_menu.c
  avec MENU_GEN_PROC ; utiliser dans kernel/browse_dir.c*/
  pLimage_buttBrowse = (Panel_item) xv_create (pL_panel_com, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "Browse",
      PANEL_INACTIVE,        TRUE, 
      /* PANEL_INACTIVE,        FALSE, */
      /*PANEL_NOTIFY_PROC,     fichier_L_Browse,*/
      /* PANEL_ITEM_MENU,       NULL, */
      NULL);

  pLimage_buttWhat = (Panel_item) xv_create (pL_panel_com, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "What",
      PANEL_NOTIFY_PROC,     fichier_L_What,
      NULL);


  window_fit (pL_panel_com);
  pL_width_com  = xv_get (pL_panel_com, XV_WIDTH);
  pL_height_com = xv_get (pL_panel_com, XV_HEIGHT);

  for (i = 0; i < cat_NB; i++)
  pL_panel[i] = (Panel) xv_create (frame_L, PANEL, 
      PANEL_LAYOUT,          PANEL_VERTICAL,
      XV_X,                  0,
      XV_Y,                  pL_height_com,
      XV_SHOW,               (i == 0),
      WIN_BORDER,	     TRUE,
      NULL);

  /*
   * panel IPS_IMAGE
  */
  categ = cat_IMAGE;
  panel = pL_panel[categ];

  pLimage_format = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "Format",
      PANEL_CHOICE_STRINGS,  for_NOM_L, NULL,
      PANEL_VALUE,           0,
      PANEL_NOTIFY_PROC,     fichier_L_image_grise_proc,
      NULL);

  xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

  pLimage_bits = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_CHOICE_STRINGS,  "8 bit", "16 bit", NULL,
      PANEL_VALUE,           0,
      NULL);

  pLimage_palette = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK, 
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "Colors",
      PANEL_CHOICE_STRINGS,  IPS_COUL_LIST, NULL,
      PANEL_VALUE,           0,
      NULL);

  xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, NULL);

  pLimage_offset = (Panel_item) xv_create (panel, PANEL_NUMERIC_TEXT,   
      PANEL_LABEL_STRING,         "Offset",
      PANEL_LABEL_X,              4,
      PANEL_VALUE_X,              80,
      PANEL_MIN_VALUE,            0,
      PANEL_MAX_VALUE,            127*256*256*256,
      PANEL_VALUE_DISPLAY_LENGTH, 10,
      PANEL_VALUE,                0,
      NULL);

  pLimage_num = (Panel_item) xv_create (panel, PANEL_NUMERIC_TEXT,   
      PANEL_LABEL_STRING,         "Number",
      PANEL_LABEL_X,              4,
      PANEL_VALUE_X,              80,
      PANEL_MIN_VALUE,            1,
      PANEL_MAX_VALUE,            256,
      PANEL_VALUE_DISPLAY_LENGTH, 10,
      PANEL_VALUE,                1,
      NULL);

  pLimage_ysiz = (Panel_item) xv_create (panel, PANEL_NUMERIC_TEXT,   
      PANEL_LABEL_STRING,         "Height",
      PANEL_LABEL_X,              4,
      PANEL_VALUE_X,              80,
      PANEL_MIN_VALUE,            8,
      PANEL_MAX_VALUE,            16384,
      PANEL_VALUE_DISPLAY_LENGTH, 10,
      PANEL_VALUE,                256,
      NULL);

  pLimage_xsiz = (Panel_item) xv_create (panel, PANEL_NUMERIC_TEXT,   
      PANEL_LABEL_STRING,         "Width",
      PANEL_LABEL_X,              4,
      PANEL_VALUE_X,              80,
      PANEL_MIN_VALUE,            8,
      PANEL_MAX_VALUE,            16384,
      PANEL_VALUE_DISPLAY_LENGTH, 10,
      PANEL_VALUE,                256,
      NULL);

    pLimage_buttLoad = (Panel_item) xv_create (panel, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "Load Image",
      XV_X,                  300,
      XV_Y,                  (int) xv_get (pLimage_offset, XV_Y),
      PANEL_NOTIFY_PROC,     fichier_L,
      PANEL_CLIENT_DATA,     fichier_L_image,
      NULL);

    pLimage_buttNext = (Panel_item) xv_create (panel, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "Next",
      XV_X,                  300,
      XV_Y,                  (int) xv_get (pLimage_num, XV_Y),
      PANEL_NOTIFY_PROC,     fichier_L,
      PANEL_CLIENT_DATA,     fichier_L_image,
      NULL);

    pLimage_buttAll = (Panel_item) xv_create (panel, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "All",
      XV_X,                  300 + 6 + (int) xv_get (pLimage_buttNext, XV_WIDTH),
      XV_Y,                  (int) xv_get (pLimage_num, XV_Y),
      PANEL_NOTIFY_PROC,     fichier_L,
      PANEL_CLIENT_DATA,     fichier_L_image,
      NULL);

    pLimage_buttView = (Panel_item) xv_create (panel, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "Quick View",
      XV_X,                  300,
      XV_Y,                  (int) xv_get (pLimage_xsiz, XV_Y),
      PANEL_NOTIFY_PROC,     fichier_L,
      PANEL_CLIENT_DATA,     fichier_L_image,
      PANEL_INACTIVE, TRUE,
      NULL);

  /*
   * panel IPS_VOR2D
  */
  categ = cat_VOR2D;
  panel = pL_panel[categ];

  pLvor2d_buttLoad = (Panel_item) xv_create (panel, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "Load",
      XV_X,                  300,
      PANEL_NOTIFY_PROC,     fichier_L,
      PANEL_CLIENT_DATA,     fichier_L_vor2d,
      NULL);


  /*
   * Finitions
  */
  for (i = 0; i < cat_NB; i++) {
    window_fit (pL_panel[i]);
    if (xv_get (pL_panel[i], XV_WIDTH) < pL_width_com)
      xv_set (pL_panel[i], XV_WIDTH, pL_width_com, NULL);
    pL_width[i]  = xv_get (pL_panel[i], XV_WIDTH);
    pL_height[i] = xv_get (pL_panel[i], XV_HEIGHT);
  }
  window_fit (frame_L);
  xv_set (pL_panel_com,
	XV_WIDTH,	WIN_EXTEND_TO_EDGE,
	XV_HEIGHT,	WIN_EXTEND_TO_EDGE,
	NULL);
}

/***************************************************************************************
 * Creer la boite de dialogue du menu fichier/save
 *
 ***************************************************************************************/
void
fichier_save_creer_frame ()
{
  int   i, categ;
  Panel panel;
  char *thedir;

  /* Cherche le pwd ou a ete lance' ips */
  thedir = (char *) getenv ("PWD");

  frame_S = (Frame) xv_create (glo.frame_principal, FRAME_CMD,
      FRAME_LABEL,              "Save File",
      NULL);

  pS_panel_com = (Panel) xv_get (frame_S, FRAME_CMD_PANEL);
  xv_set (pS_panel_com, PANEL_LAYOUT, PANEL_VERTICAL, NULL);

  pS_pwd = (Panel_item) xv_create (pS_panel_com, PANEL_TEXT,
      PANEL_LABEL_STRING,         "Directory ",
      PANEL_VALUE,                (thedir == NULL) ? "" : thedir,
      PANEL_VALUE_DISPLAY_LENGTH, 40,
      PANEL_VALUE_STORED_LENGTH,  120,
      NULL);

  pS_name = (Panel_item) xv_create (pS_panel_com, PANEL_TEXT,
      PANEL_LABEL_STRING,         "File name",
      PANEL_VALUE,                "noname",
      PANEL_VALUE_DISPLAY_LENGTH, 40,
      PANEL_VALUE_STORED_LENGTH,  120,
      NULL);

  pS_categ = (Panel_item) xv_create (pS_panel_com, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "Category",
      PANEL_CHOICE_STRINGS,  cat_NOMS, NULL,
      PANEL_VALUE,           0,
      PANEL_NOTIFY_PROC,     fichier_S_categ,
      NULL);

  xv_set (pS_panel_com, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

  pS_auto_ext = (Panel_item) xv_create (pS_panel_com, PANEL_CHECK_BOX,
      PANEL_LABEL_STRING,    "Automatic Extension",
      XV_X,                  220,
      PANEL_VALUE,           1,
      NULL);

  window_fit (pS_panel_com);
  pS_width_com  = xv_get (pS_panel_com, XV_WIDTH);
  pS_height_com = xv_get (pS_panel_com, XV_HEIGHT);

  for (i = 0; i < cat_NB; i++)
  pS_panel[i] = (Panel) xv_create (frame_S, PANEL, 
      PANEL_LAYOUT,          PANEL_VERTICAL,
      XV_X,                  0,
      XV_Y,                  pS_height_com,
      XV_SHOW,               (i == 0),
      WIN_BORDER,	     TRUE,
      NULL);

  /*
   * panel IPS_IMAGE
  */
  categ = cat_IMAGE;
  panel = pS_panel[categ];

  pSimage_format = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "Format",
      PANEL_CHOICE_STRINGS,  for_NOM_S, NULL,
      PANEL_VALUE,           0,
      NULL);

  xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

  pSimage_bits = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_CHOICE_STRINGS,  "8 bit", "16 bit", "vga colors", NULL,
      PANEL_VALUE,           0,
      NULL);

  xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, NULL);

  pSimage_compress = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "Compression",
      PANEL_CHOICE_STRINGS,  "none", "compress .Z", "gzip .gz (fast)", 
		"gzip -9 .gz (best)", NULL,
      PANEL_VALUE,           3,
      NULL);

  xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

  (void) xv_create (panel, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "Save selection",
      XV_X,                  270,
      PANEL_NOTIFY_PROC,     fichier_S_image,
      NULL);

  /*
   * panel IPS_VOR2D
  */
  categ = cat_VOR2D;
  panel = pS_panel[categ];

  pSvor2d_format = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "Format",
      PANEL_CHOICE_STRINGS,  "Diagram", "Points", NULL,
      PANEL_VALUE,           0,
      NULL);

  xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

  (void) xv_create (panel, PANEL_BUTTON,
      PANEL_LABEL_STRING,    "Save selection",
      XV_X,                  270,
      PANEL_NOTIFY_PROC,     fichier_S_vor2d,
      NULL);


  /*
   * Finitions
  */
  for (i = 0; i < cat_NB; i++) {
    window_fit (pS_panel[i]);
    if (xv_get (pS_panel[i], XV_WIDTH) < pS_width_com)
      xv_set (pS_panel[i], XV_WIDTH, pS_width_com, NULL);
    pS_width[i]  = xv_get (pS_panel[i], XV_WIDTH);
    pS_height[i] = xv_get (pS_panel[i], XV_HEIGHT);
  }
  window_fit (frame_S);
  xv_set (pS_panel_com,
	XV_WIDTH,	WIN_EXTEND_TO_EDGE,
	XV_HEIGHT,	WIN_EXTEND_TO_EDGE,
	NULL);
}

/***************************************************************************************
 * Recoit un nom de fichier n1, et cherche dans l'extension la taille de l'image
 * et la chaine "bw" ou "cg"
 ***************************************************************************************/
void
fichier_ana_fin_nom (n1, x, y, bw)
char *n1;
int   *x, *y, *bw;
{
  int i, j, k;

  *x = 256;
  *y = 256;
  i  = 0;

  while (n1[i] != 0 && n1[i] != '.') i++;
  if (strstr (n1+i, "bw")) *bw = 1;
  else if (strstr (n1+i, "cg")) *bw = 2;
  else *bw = 0;

  while (n1[i] != 0 && !(n1[i] >= '0' && n1[i] <= '9')) i++;
  if (n1[i] != 0) 
  {
    j = i; k = n1[j]-'0'; i++;
    while (n1[i] >= '0' && n1[i] <= '9') { k = k*10 + n1[i]-'0'; i++; }
    if (8 <= k && k <= 16384) *x = *y = k; 
  }
  while (n1[i] != 0 && !(n1[i] >= '0' && n1[i] <= '9')) i++;
  if (n1[i] != 0) 
  {
    j = i; k = n1[j]-'0'; i++;
    while (n1[i] >= '0' && n1[i] <= '9') { k = k*10 + n1[i]-'0'; i++; }
    if (8 <= k && k <= 16384) *x = k; 
  }
#ifdef DEBUG
  printf ("nom = `%s'  x = %d  y = %d bw = %d\n", n1, *x, *y, *bw);
#endif
}

/***************************************************************************************
 * Prepare la boite de dialogue d'apres le nom du fichier
 ***************************************************************************************/
int fichier_L_What (item,event)
Panel_item item;
Event *event;
{
    char	*s_pwd  = (char *) xv_get (pL_pwd , PANEL_VALUE),
		*s_name = (char *) xv_get (pL_name, PANEL_VALUE),
		s_funame[256];

    stg_RegroupChemNomFi (s_pwd, s_name, s_funame);

    fichier_drop (s_funame);

    return XV_OK;
}

/***************************************************************************************
 * Recoit un nom de fichier en DROP
 * prepare la boite de dialogue
 ***************************************************************************************/
void
fichier_drop (buf1)
char *buf1;
{
    char buf2[128], buf3[128], *fin;
    int  x, y, bw, i;

    stg_ScindeChemNomFi (buf1, buf2, buf3);
    xv_set (pL_pwd , PANEL_VALUE, buf2, NULL);
    xv_set (pL_name, PANEL_VALUE, buf3, NULL);

    i = 0;
    while (buf3[i] != 0 && buf3[i] != '.') i++;
    fin = buf3 + i;

    if (strstr (fin, ".vor")) {

	fichier_new_L_categ (cat_VOR2D);
	if (xv_get (frame_L, XV_SHOW) != TRUE)
	(void) fichier_L (pLvor2d_buttLoad, NULL);
   }

    else {

	fichier_new_L_categ (cat_IMAGE);

	if (strstr (fin, ".pgm")) {

	    fichier_L_image_grise (for_PGM);
	}
	else {

	    fichier_L_image_grise (for_RAW);
	    fichier_ana_fin_nom (buf3, &x, &y, &bw);
	    xv_set (pLimage_xsiz, PANEL_VALUE, x, NULL);
	    xv_set (pLimage_ysiz, PANEL_VALUE, y, NULL);
	    xv_set (pLimage_palette, PANEL_VALUE, bw, NULL);
	}
	if (xv_get (frame_L, XV_SHOW) != TRUE)
	(void) fichier_L (pLimage_buttLoad, NULL);
    }

}


/***************************************************************************************
 * Ouvre et decompresse le fichier decrit dans la boite de dialogue
 * puis appelle la fonction de chargement correspondante
 *
 ***************************************************************************************/
int fichier_L (item,event)
Panel_item item;
Event *event;
{
    char	*s_pwd  = (char *) xv_get (pL_pwd , PANEL_VALUE),
		*s_name = (char *) xv_get (pL_name, PANEL_VALUE),
		s_funame[256], s_decomp[256];
    int		fd, a, decomp;
    void	(*fonction_L)(); 

    stg_RegroupChemNomFi (s_pwd, s_name, s_funame);

    /*
     * Decompression eventuelle
    */
    a = strlen (s_funame); decomp = FAUX;
    if (a > 2 && !strcmp (s_funame+a-2, ".Z")) {

        char chaine_exec[280];

        pcp_footer ("Uncompress ", s_funame ,NULL);
        sprintf (s_decomp, "%s/.ips_tmp_load", glo.home_dir);
        sprintf (chaine_exec, "zcat %s > %s", s_funame, s_decomp);
        s_funame[a-2] = 0; decomp = VRAI;
        system (chaine_exec);
        pcp_footer_n ();
    }
    else if (a > 3 && !strcmp (s_funame+a-3, ".gz")) {

        char chaine_exec[280];

        pcp_footer ("Gunzip ", s_funame ,NULL);
        sprintf (s_decomp, "%s/.ips_tmp_load", glo.home_dir);
        sprintf (chaine_exec, "gunzip -c %s > %s", s_funame, s_decomp);
        s_funame[a-3] = 0; decomp = VRAI;
        system (chaine_exec);
        pcp_footer_n ();
    }

    /*
     * Tentative d'ouverture du fichier
    */
    pcp_footer ("Loading ", s_funame, NULL); 
    if (decomp)
         fd = open(s_decomp,0);
    else fd = open(s_funame,0);

    if (fd == -1) {
	pcp_avertissement ("Can't open the file", "", s_funame);
	pcp_footer_n (); return XV_OK;
    }

    /*
     * Appel fonction de chargement
    */
    fonction_L = (void *) xv_get (item, PANEL_CLIENT_DATA);
    if (fonction_L != NULL) (*fonction_L)(item, fd, s_funame, s_name);

    /*
     * Nettoyage
    */
    close(fd);
    if (decomp) {
	char chaine_exec[280];
	sprintf (chaine_exec, "rm %s", s_decomp);
	system (chaine_exec);
    }
    pcp_footer_n ();

    return XV_OK;
}

/***************************************************************************************
 * Charge le fichier IPS_IMAGE
 *
 ***************************************************************************************/
void fichier_L_image (item, fd, s_funame, s_name)
Panel_item item;
int fd;
char *s_funame, *s_name;
{
    long	d_lseek;
    void	*line_buf;
    int		pt_siz, tota_siz, line_siz,
		format, fbits, xsiz, ysiz, num, offset, coul,
		lecture_ok = VRAI,
		nb_lu = 0;

    format = (int) xv_get (pLimage_format, PANEL_VALUE);

    /*
     * lecture en-tete fichier ou params taille
    */
    switch (format) {

	case for_PGM : {
	    char ligne[1024];

	    num    = 1;
	    coul   = IPS_COUL_64G;
	    offset = 3;

	    /* ouvre le header */
	    if (read(fd, ligne, 1024) < 3) {
		pcp_avertissement ("ERROR on file", s_funame, "Can't read header");
		return;
	    } else ligne[1024-1] = 0; /* securite */

	    /* teste header = P5 */
	    if (strncmp (ligne, "P5", 2)) {
		pcp_avertissement ("Bad magic number", "", "should be P5");
		return;
	    }

	    /* saute commentaires */
	    while (ligne[offset] == '#')
		if (stg_NextLine(ligne, &offset) == 0) return;

	    /* lit taille x, y */
	    sscanf (ligne+offset, "%d %d", &xsiz, &ysiz);
	    if (stg_NextLine(ligne, &offset) == 0) return;

	    /* lit 8-16 bits */
	    sscanf (ligne+offset, "%d", &fbits);
	    fbits = (fbits == 65535) ? 1 : 0;
	    if (stg_NextLine(ligne, &offset) == 0) return;

	    /* test coherence du header */
	    if (xsiz < 8 || ysiz < 8 || xsiz > 16384 || ysiz > 16384) {
		pcp_avertissement ("ERROR in the header of", "", s_funame);
		return;
	    }
	} break;

	case for_RAW :
	default :
	    fbits  = (int) xv_get (pLimage_bits, PANEL_VALUE);
	    xsiz   = (int) xv_get (pLimage_xsiz, PANEL_VALUE);
	    ysiz   = (int) xv_get (pLimage_ysiz, PANEL_VALUE);
	    num    = (int) xv_get (pLimage_num, PANEL_VALUE);
	    offset = (int) xv_get (pLimage_offset, PANEL_VALUE);
	    coul   = (int) xv_get (pLimage_palette, PANEL_VALUE);

	    /* Gestion des boutons */
	    if (item == pLimage_buttNext)
		xv_set (pLimage_num, PANEL_VALUE, ++num, NULL);
	    break;

    } /* switch format */


    /*
     * Calcul des nb de bits
    */
    switch (fbits) {
	case 0 : pt_siz = sizeof(Unt8 ); break;  /*  8 bits */
	case 1 : pt_siz = sizeof(Unt16); break;  /* 16 bits */
    }
    tota_siz  = xsiz * ysiz * pt_siz;
    line_siz  = xsiz * pt_siz;
    d_lseek   = offset + (long) tota_siz * (num-1);
    lseek (fd, d_lseek, 0);
    line_buf  = (void *) pcp_alloc (line_siz);
    if (line_buf == NULL) lecture_ok = FAUX;
    
    /*
     * #2 Lecture(s)
    */
    while (lecture_ok) {

	Unt16 *tab = (Unt16 *) pcp_alloc (xsiz * ysiz * sizeof(Unt16));

	/*
	 * #3 la memoire est allouee
	*/
	if (tab != NULL) {

	    int k;
	    int j = 0;
	    int d = tota_siz;

	    switch (pt_siz) {

	    case 1 : /* 8 bits */
		while (d > 0) 
		    if (read(fd, line_buf, line_siz) == line_siz) {
			for (k = 0; k < xsiz; k++) 
			    tab[j+k] = ((Unt8*)line_buf)[k];
			j += xsiz; d -= line_siz;
		    }
		    else { lecture_ok = FAUX; break; }
		break;

	    case 2 : /* 16 bits */
		while (d > 0) 
		    if (read(fd, line_buf, line_siz) == line_siz) {
			for (k = 0; k < xsiz; k++) 
			    tab[j+k] = ((Unt16*)line_buf)[k];
			j += xsiz; d -= line_siz;
		    }
		    else { lecture_ok = FAUX; break; }
		break;
	    }

	    /*
	     * Creation icone
	    */
	    if (lecture_ok) {

		    Ico   *ico = ico_creat (glo.sx, glo.sy, IPS_IMAGE);
		    Image *ima = (Image *) ico->obj;
		    char  buf3[IMAGE_CH1LEN+1];

		    ico_tete_pz (ico);
		    image_creat (ima, 0, xsiz, ysiz, tab);
		    stg_DebNomFi (s_name, buf3);
		    sprintf (ima->ch1, "%s %d", buf3, num+nb_lu);
		    image_calc_hublot (ico);
		    ima->coul_tab = glo_get_ctab_lut (coul);
		    /* ico->xb -= ico->xm/2; ico->yb -= ico->ym/2; */
		    ico_douane (ico);
		    ico_refresh_now (ico);
		    glo.sx += 10; glo.sy += 16;
		    nb_lu++;
	    }

	    /*
	     * Probleme de lecture, abandon
	    */
	    else { 

		    pcp_free (tab);
		    if (nb_lu == 0) {
			xv_set (frame_L, XV_SHOW, TRUE, NULL);
			pcp_avertissement ("Reading error", "or", "End of file");
		    }
	    }

	} /* #3 */
	else lecture_ok = FAUX;

	/*
	 * Gestion des boutons
	*/
	if (item != pLimage_buttAll) lecture_ok = FAUX;

    } /* #2 */

    pcp_free(line_buf);
}

/***************************************************************************************
 * Charge le fichier IPS_VOR2D
 *
 ***************************************************************************************/

void fichier_L_vor2d (item, fd, s_funame, s_name)
Panel_item item;
int fd;
char *s_funame, *s_name;
{
    enum { a_DEFAUT, a_DELAUNAY, a_VORONOI };
    FILE   *f;
    char   ligne[256], mot[128], *suite;
    int    lecture_pb  = 0,
	   lecture_nl  = 0,
	   a_point_num = 0,
           action      = a_DEFAUT,
           motp, n;
    Ico   *ico;
    Vor2d *ob;

    /*
     * Transforme int fd --> FILE *f
     * ATTENTION : plus aucun return avant fclose !!
    */
    if ((f = fdopen (fd, "r")) == NULL) {
	pcp_error ("fdopen failed in", "fichier_L_vor2d", NULL);
	return;
    }

    /*
     * Creation Icone
    */
    ico = ico_creat (glo.sx, glo.sy, IPS_VOR2D);
    ob  = (Vor2d *) ico->obj;
    vor2d_creat (ob, 100, 100);

    /*
     * Lecture
    */
    while (lecture_pb == 0 && fgets (ligne, 256, f) != NULL) {

	lecture_nl++;

	if ( sscanf (ligne, "%s%n", mot, &motp) == EOF || mot[0] == ';' || mot[0] == '#' )
	    continue; /* on saute la ligne */
	suite = ligne + motp;

	if (!strcmp (mot, "DELAUNAY")) {

	    action = a_DELAUNAY;
	    sscanf (suite, "%d", &n);
	    vor2d_adapt_tab_point (ob, n + 16);
	    ob->nb_point = n;
	}
	else if (!strcmp (mot, "VORONOI")) {

	    action = a_VORONOI;
	    sscanf (suite, "%d", &n);
	    vor2d_adapt_tab_sommet (ob, n + 16);
	    ob->nb_sommet = n;
	}
	else switch (action) {

	    case a_DELAUNAY : {
		double x, y;
		int    num_s;
		if ( sscanf (ligne, "%d %lf %lf %d", &n, &x, &y, &num_s) == 4
			&& -3 <= n && n < ob->nb_point ) {
		    ob->tab_point[n].point[0] = x;
		    ob->tab_point[n].point[1] = y;
		    ob->tab_point[n].num_s = num_s;
		}
		else lecture_pb = lecture_nl;
	    } break;

	    case a_VORONOI : {
		double x, y;
		int    gc0, gc1, gc2, sv0, sv1, sv2;
		if ( sscanf (ligne, "%d %lf %lf %d %d %d %d %d %d",
			    &n, &x, &y, &gc0, &gc1, &gc2, &sv0, &sv1, &sv2) == 9
			&& -4 <= n && n < ob->nb_sommet ) {
		    ob->tab_sommet[n].point[0] = x;
		    ob->tab_sommet[n].point[1] = y;
		    ob->tab_sommet[n].num_gc[0] = gc0;
		    ob->tab_sommet[n].num_gc[1] = gc1;
		    ob->tab_sommet[n].num_gc[2] = gc2;
		    ob->tab_sommet[n].num_sv[0] = sv0;
		    ob->tab_sommet[n].num_sv[1] = sv1;
		    ob->tab_sommet[n].num_sv[2] = sv2;
		}
		else lecture_pb = lecture_nl;
	    } break;

	    case a_DEFAUT :
	    default : {
		double x, y;
		if ( sscanf (ligne, "%lf %lf", &x, &y ) == 2 ) {
		    if (a_point_num >= ob->nb_max_point)
			vor2d_adapt_tab_point (ob, a_point_num + 100);
		    ob->tab_point[a_point_num].point[0] = x;
		    ob->tab_point[a_point_num].point[1] = y;
		    ob->nb_point = ++a_point_num;
		}
		else lecture_pb = lecture_nl;
	    } break;
	}
    }


    /*
     * Termine creation icone
    */
    if (lecture_pb == 0) {

	char  buf3[VOR2D_CH1LEN+1];

	ico_tete_pz (ico);
	stg_DebNomFi (s_name, buf3);
	vor2d_prepare_noms (ico, buf3);
	ico_douane (ico);
	ico_refresh_now (ico);
	glo.sx += 10; glo.sy += 16;
    }
    else {
	ico_free (ico);
	sprintf (ligne, "line %d", lecture_pb);
	pcp_avertissement ("Syntax error", "in", ligne);
    }

    /*
     * Fermeture
    */
    fclose (f);
}

/***************************************************************************************
 * Teste l'ouverture du fichier et renvoie erreur
 * - si fichier inexistant, le cree
 * - si fichier existe, demande Abort / Overwrite, + Append si append = VRAI
 *
 ***************************************************************************************/
int fichier_S_test_open (s_funame, fd, append)
char *s_funame;
int *fd, append;
{
    pcp_footer ("Saving ", s_funame, NULL); 

    *fd = open (s_funame, 0);

    if (*fd == -1)
	*fd = creat (s_funame,0644);
    else {
	Rect *rect = (Rect*) xv_get (xv_get(glo.panel1, CANVAS_NTH_PAINT_WINDOW,0), WIN_MOUSE_XY);
	int result;

	if (append) result = notice_prompt(glo.panel1, NULL,
		NOTICE_FOCUS_XY,         rect->r_left, rect->r_top,
		NOTICE_MESSAGE_STRINGS,  "WARNING : file already exists", s_funame, NULL,
		NOTICE_BUTTON_YES,       "Abort",
		NOTICE_BUTTON,           "Overwrite", 101,
		NOTICE_BUTTON,           "Append",    102,
		NOTICE_NO_BEEPING,       TRUE,
		NULL);
	else result = notice_prompt(glo.panel1, NULL,
		NOTICE_FOCUS_XY,         rect->r_left, rect->r_top,
		NOTICE_MESSAGE_STRINGS,  "WARNING : file already exists", s_funame, NULL,
		NOTICE_BUTTON_YES,       "Abort",
		NOTICE_BUTTON,           "Overwrite", 101,
		NOTICE_NO_BEEPING,       TRUE,
		NULL);

	close(*fd);
	switch (result) {
	    case NOTICE_YES : pcp_footer_n (); return FAUX;
	    case 101 : *fd = open(s_funame, O_WRONLY | O_TRUNC); break;
	    case 102 : *fd = open(s_funame, O_WRONLY | O_APPEND); break;
	}
    }

    if (*fd == -1) {
	pcp_avertissement ("Can't open the file", "", s_funame);
	pcp_footer_n (); return FAUX;
    }
    return VRAI;
}

/***************************************************************************************
 * Sauve la selection dans le fichier image
 *
 ***************************************************************************************/
int fichier_S_image (item,event)
Panel_item item;
Event *event;
{
    int		format = (int) xv_get (pSimage_format, PANEL_VALUE),
		fbits  = (int) xv_get (pSimage_bits, PANEL_VALUE),
		fd, n, prem, xs, ys, nlut, i, x, y, bloc, compr;
    char	s_funame[256], *s_pwd, *s_name, chaine_exec[280];
    Ico		*p[100];
    void	*buf_tab;

    pcp_footer ("Saving ...", NULL); 

    /*
     * tri de la selection, que des images, meme taille
    */
    if (!ico_sel_scan1 (p, 100, IPS_IMAGE, "images", &n) || n == 0)
	{ pcp_footer_n (); return XV_OK; }

    xs = ((Image *) p[0]->obj)->b.xsiz;
    ys = ((Image *) p[0]->obj)->b.ysiz;
    nlut = glo_get_attr_lut (((Image *) p[0]->obj)->coul_tab);

    if (format == for_RAW) {
	for (i = 1; i < n; i++)
	if (xs != ((Image *) p[i]->obj)->b.xsiz ||
	    ys != ((Image *) p[i]->obj)->b.ysiz ) {
		pcp_avertissement ("WARNING !","Only images with the same size",
		    "must be saved in the file");
		pcp_footer_n (); return XV_OK;
	    }
    }
    else if (n > 1) {
	pcp_avertissement ("WARNING ! In that format,","only 1 image",
		    "must be saved at a time");
	pcp_footer_n (); return XV_OK;
    }

    /*
     * creation liste des images et buffer fichier
    */
    switch (fbits) {
        case 0 : bloc = xs * ys; break;
        case 1 : bloc = xs * ys * 2; break;
        case 2 : bloc = xs * ys * 3; break;
    }
    switch (fbits) {
	case 0 : buf_tab = (Unt8  *) pcp_alloc (bloc); break;
	case 1 : buf_tab = (Unt16 *) pcp_alloc (bloc); break;
	case 2 : buf_tab = (Uchar3 *) pcp_alloc (bloc); break;
    }
    if (buf_tab == NULL) { pcp_footer_n (); return XV_OK; }

    /*
     * extension automatique
    */
    if (xv_get (pS_auto_ext, PANEL_VALUE)) switch (format) {

	case for_PGM :
            if (fbits == 0 || fbits == 1)
                fichier_S_new_extension ("pgm");
            else if (fbits == 2)
                fichier_S_new_extension ("ppm");
	    break;
	case for_EPS :
	    fichier_S_new_extension ("eps");
	    break;

	case for_RAW :
	default : {	
	    char bla[200];
	    if (xs == ys)
	         sprintf (bla, "%d", xs);
	    else sprintf (bla, "%d.%d", ys, xs);
	    if (nlut == 1) sprintf (bla, "%sbw", bla);
	    else if (nlut == 2) sprintf (bla, "%scg", bla);
	    fichier_S_new_extension (bla);
	} break;
    }

    s_pwd = (char *) xv_get (pS_pwd , PANEL_VALUE);
    s_name = (char *) xv_get (pS_name, PANEL_VALUE);
    stg_RegroupChemNomFi (s_pwd, s_name, s_funame);

    /*
     * test ouverture fichier
    */
    if (!fichier_S_test_open (s_funame, &fd, format == for_RAW))
	return XV_OK;

    /*
     * Ecriture en-tete
    */
	switch (format) {

	case for_PGM :
	case for_EPS : {
		char entete[1024];
                
                if (fbits == 0 || fbits == 1)
		    sprintf (entete, "P5\n# Portable Gray Map file from IPS v 3.2\n%d %d\n%d\n",
			xs, ys, (fbits == 0) ? 255 : 65535);
                else if (fbits == 2)
		    sprintf (entete, "P6\n# Portable Pix Map file from IPS v 3.2\n%d %d\n%d\n",
			xs, ys, 255);
		write (fd, entete, strlen (entete));
	    } break;

	case for_RAW :
	default : break;
    }


    /*
     * sauver
    */
    for (i = 0; i < n; i++) {

	Image *im = (Image *) p[i]->obj;
        int t1, t2, t3;

	switch (fbits) {

	    case 0 :
		for (y = 0; y < im->b.ysiz; y++)
		for (x = 0; x < im->b.xsiz; x++)
		    ((Unt8*)buf_tab)[y*xs+x] =
			im->tab[(y+im->b.cmin)*im->b.xtot + x+im->b.cmin];
		break;
	    case 1 :
		for (y = 0; y < im->b.ysiz; y++)
		for (x = 0; x < im->b.xsiz; x++)
		    ((Unt16*)buf_tab)[y*xs+x] =
			im->tab[(y+im->b.cmin)*im->b.xtot + x+im->b.cmin];
		break;
	    case 2 :
		for (y = 0; y < im->b.ysiz; y++)
		for (x = 0; x < im->b.xsiz; x++)
                {
                    t1 = y*xs+x;
                    t2 = (y+im->b.cmin)*im->b.xtot + x+im->b.cmin;
                    t3 = 1+(im->tab[t2]-1)%255;
		    ((Uchar3*)buf_tab)[t1].r = colR_16c[t3];
		    ((Uchar3*)buf_tab)[t1].g = colG_16c[t3];
		    ((Uchar3*)buf_tab)[t1].b = colB_16c[t3];
		}
                break;
	}
	write (fd, buf_tab, bloc);
    }

    /*
     * termine ecriture
    */
    pcp_free(buf_tab);
    close(fd);

    /*
     * Transformation eventuelle en EPS
    */
    if (format == for_EPS) {
	sprintf (chaine_exec, "imconv -pgm %s -eps %s", s_funame, s_funame);
	system (chaine_exec);
    }

    /*
     * Compression eventuelle
    */
    compr = (int) xv_get (pSimage_compress, PANEL_VALUE);
    switch (compr) {
	case 1 : sprintf (chaine_exec, "compress -f %s", s_funame); break;
	case 2 : sprintf (chaine_exec, "gzip -f %s", s_funame); break;
	case 3 : sprintf (chaine_exec, "gzip -f -9 %s", s_funame); break;
    }
    if (compr > 0) {
	pcp_footer (chaine_exec, " ..." ,NULL);
	system (chaine_exec);
    }


    pcp_footer_n (); 
    return XV_OK;
}

/***************************************************************************************
 * Sauve la selection dans le fichier vor2d
 *
 ***************************************************************************************/
int 
fichier_S_vor2d (item,event)
Panel_item item;
Event *event;
{
    char	s_funame[256], *s_pwd, *s_name, chaine_exec[280];
    int		fd,
		format = (int) xv_get (pSvor2d_format, PANEL_VALUE);
    FILE	*f;
    Ico		*ico;
    Vor2d	*ob;

    /*
     * tri de la selection, un seul Voronoi
    */
    if (!ico_sel_scan2 (&ico, IPS_VOR2D, "Voronoi"))
	{ pcp_footer_n (); return XV_OK; }
    ob = (Vor2d *) ico->obj;

    /*
     * extension automatique
    */
    if (xv_get (pS_auto_ext, PANEL_VALUE))
	fichier_S_new_extension ("vor");

    s_pwd = (char *) xv_get (pS_pwd , PANEL_VALUE);
    s_name = (char *) xv_get (pS_name, PANEL_VALUE);
    stg_RegroupChemNomFi (s_pwd, s_name, s_funame);

    /*
     * test ouverture fichier
    */
    if (!fichier_S_test_open (s_funame, &fd, FALSE))
	return XV_OK;
    if ((f = fdopen (fd, "w")) == NULL) {
	pcp_error ("fdopen failed in", "fichier_S_vor2d", NULL);
	return XV_OK;
    }

    /*
     * ECRITURE
    */
    if (format == 0) { /* Diagram */

	int i;

	fprintf (f, "# STRUCTURE VORONOI/DELAUNAY 2D\n\n");

	fprintf (f, "# Description des points de Delaunay\n");
	fprintf (f, "# i x y num_s\n");
	fprintf (f, "DELAUNAY %d\n", ob->nb_point);
	for (i = -3; i < ob->nb_point; i++) {
	    fprintf (f, "%d %f %f %d\n", 
		i, ob->tab_point[i].point[0], ob->tab_point[i].point[1],
		ob->tab_point[i].num_s);
	}
	fprintf (f, "\n# Description des sommets de Voronoi\n");
	fprintf (f, "# i x y gc0 gc1 gc2 sv0 sv1 sv2\n");
	fprintf (f, "VORONOI %d\n", ob->nb_sommet);
	for (i = -4; i < ob->nb_sommet; i++) {
	    fprintf (f, "%d %f %f %d %d %d %d %d %d\n", 
		i, ob->tab_sommet[i].point[0], ob->tab_sommet[i].point[1],
		ob->tab_sommet[i].num_gc[0], ob->tab_sommet[i].num_gc[1],
		ob->tab_sommet[i].num_gc[2], ob->tab_sommet[i].num_sv[0],
		ob->tab_sommet[i].num_sv[1], ob->tab_sommet[i].num_sv[2] );
	}
    }
    else if (format == 1) { /* Points */

	int i;

	for (i = 0; i < ob->nb_point; i++) {
	    fprintf (f, "%f %f\n", 
		ob->tab_point[i].point[0], ob->tab_point[i].point[1]);
	}
    }

    /*
     * termine ecriture
    */
    fclose (f);
    close (fd);

    pcp_footer_n (); 
    return XV_OK;
}

/***************************************************************************************
 * Affiche le panel correspondant a la categorie demandee
 *
 ***************************************************************************************/

void  fichier_new_L_categ (value)
int   value;
{
    xv_set (pL_categ, PANEL_VALUE, value, NULL);
    fichier_L_categ (pL_categ, value, (Event *) NULL);
}

int         fichier_L_categ (item,value,event)    
Panel_item  item;
int         value;
Event      *event;
{
  int width, height;

  if (value == pL_categ_val) return XV_OK;

  width  = pL_width[value];
  height = pL_height[value];
  xv_set (frame_L,
	XV_WIDTH, (width > pL_width_com) ? width : pL_width_com,
	XV_HEIGHT, height + pL_height_com,
	NULL);
  xv_set (pL_panel[pL_categ_val], XV_SHOW, FALSE, NULL);
  pL_categ_val = value;
  xv_set (pL_panel[pL_categ_val], XV_SHOW, TRUE, NULL);
  return XV_OK;
}

int         fichier_S_categ (item,value,event)    
Panel_item  item;
int         value;
Event      *event;
{
  int width, height;

  if (value == pS_categ_val) return XV_OK;

  width  = pS_width[value];
  height = pS_height[value];
  xv_set (frame_S,
	XV_WIDTH, (width > pS_width_com) ? width : pS_width_com,
	XV_HEIGHT, height + pS_height_com,
	NULL);
  xv_set (pS_panel[pS_categ_val], XV_SHOW, FALSE, NULL);
  pS_categ_val = value;
  xv_set (pS_panel[pS_categ_val], XV_SHOW, TRUE, NULL);
  return XV_OK;
}

/***************************************************************************************
 * Dans pL_categ[cat_IMAGE] : Grise bits / colors selon Format
 *
 ***************************************************************************************/

void  fichier_L_image_grise (value)
int   value;
{
    xv_set (pLimage_format, PANEL_VALUE, value, NULL);
    fichier_L_image_grise_proc (pLimage_format, value, (Event *) NULL);
}

int         fichier_L_image_grise_proc (item,value,event)    
Panel_item  item;
int         value;
Event      *event;
{
  xv_set (pLimage_offset,  PANEL_INACTIVE, value != for_RAW, NULL);
  xv_set (pLimage_num,     PANEL_INACTIVE, value != for_RAW, NULL);
  xv_set (pLimage_ysiz,    PANEL_INACTIVE, value != for_RAW, NULL);
  xv_set (pLimage_xsiz,    PANEL_INACTIVE, value != for_RAW, NULL);
  xv_set (pLimage_bits,    PANEL_INACTIVE, value != for_RAW, NULL);
  xv_set (pLimage_palette, PANEL_INACTIVE, value != for_RAW, NULL);
  xv_set (pLimage_buttNext,PANEL_INACTIVE, value != for_RAW, NULL);
  xv_set (pLimage_buttAll, PANEL_INACTIVE, value != for_RAW, NULL);

  return XV_OK;
}

/***************************************************************************************
 * Ajoute la nouvelle extension
 *
 ***************************************************************************************/
void  fichier_S_new_extension (new_ext)    
char *new_ext;
{
    char bla[200];
    int i;

    sprintf (bla, "%s", (char *) xv_get (pS_name, PANEL_VALUE));

    i = 0;
    while (bla[i] != 0 && bla[i] != '.') i++;
    if (i == 0) { sprintf (bla, "noname"); i = strlen (bla); }
    if (bla[i] != '.') bla[i] = '.';

    sprintf (bla + i + 1, new_ext);

    xv_set (pS_name, PANEL_VALUE, bla, NULL);
}

/***************************************************************************************
 * Creation du bouton et du menu "Fichier" du frame_principal
 *
 ***************************************************************************************/
Panel_item fichier_FramePcpButtCreat (panel)
Panel panel;
{
    Menu menu  = (Menu) xv_create (NULL, MENU,
	MENU_STRINGS,		"Load", "Save", "About", "Quit", NULL,
	MENU_NOTIFY_PROC,	fichier_FramePcpMenuProc,
	NULL);

    return (Panel_item) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,	"File",
	PANEL_ITEM_MENU,	menu,
	NULL);
}

/***************************************************************************************
 * Activation du choix dans le menu "Fichier" du frame_principal
 *
 ***************************************************************************************/
void fichier_FramePcpMenuProc (menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    char *menu_str = (char *) xv_get (menu_item, MENU_STRING);

    if (!strcmp(menu_str, "Load")) {

	xv_set (frame_L, XV_SHOW, TRUE, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
    }
    else if (!strcmp(menu_str, "Save")) {

	/* xv_set (pS_pwd, PANEL_VALUE, 
	    (char *) xv_get (pL_pwd, PANEL_VALUE), NULL); */
	xv_set (frame_S, XV_SHOW, TRUE, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
    }
    else if (!strcmp(menu_str, "About")) {

	pcp_about ();
    }
    else if (!strcmp(menu_str, "Quit")) {

	pcp_quit ();
    }
}
