/***************************************************************************************/
/* figw.c                                                     I.P.S  v 3.7  14/3/1996  */
/*                                                                                     */
/*                                                       Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"


/***************************************************************************************
 * Cree un nouveau Fig_data* et initialise.
 * Renvoie le pointeur, ou NULL si echec.
 *
 * Appele' par fig_decode_def.
 ***************************************************************************************/
Fig_data *fig_new ()
{
    Fig_data *wdata = (Fig_data *) pcp_alloc (sizeof (Fig_data));
    if (wdata == NULL) return NULL;
    memset (wdata, 0, sizeof (Fig_data));
    wdata->draw_mode = FIG_CPY;
    wdata->draw_color = 255;
    wdata->do_run = FAUX;
    return wdata;
}


/***************************************************************************************
 * Libere le Fig_data*
 *
 * Appele' par telop_free.
 ***************************************************************************************/
void fig_free (wdata)
Fig_data *wdata;
{
    int i;
    if (wdata == NULL) return;
    for (i = 0; i < OPER_VECM; i++) pcp_free (wdata->legende[i]);
    /* Rq: pas de free pour les Edi*, car ils recoivent un MESSA_KILL */
    pcp_free (wdata);
}


/***************************************************************************************
 * Duplique le Fig_data*
 *
 * Appele' par telop_duplic.
 ***************************************************************************************/
Fig_data *fig_duplic (o_data)
Fig_data *o_data;
{
    Fig_data *n_data;
    int i;

    if (o_data == NULL) return NULL;
    n_data = fig_new ();
    if (n_data == NULL) return NULL;

    for (i = 0; i < OPER_VECM; i++) {
	n_data->num_ima[i] = o_data->num_ima[i];
	n_data->legende[i] = stg_Cpy (o_data->legende[i]);
    }

    return n_data;
}


/***************************************************************************************
 * Applique definition d'un IPS_FIG_IMAGE
 *
 * Appele' par telop_decode_def.
 ***************************************************************************************/
void fig_decode_def (op, num_fig, num_ima, legende)
Telop *op;
int num_fig, num_ima;
char *legende;
{
    /* Teste pre-existence, sinon creation */
    if (op->fig == NULL) op->fig = fig_new ();
    if (op->fig == NULL) return;

    /* Teste integrite des handles */
    if (num_fig < 1 || num_fig >= TELOP_VECM) {
	pcp_error ("Range error for window handle", "in section DEF", NULL);
	return;
    }

    /* nouvel indice pour stockage : a partir de maintenant on decale tout de 1 */
    num_fig--; 

    if (op->fig->num_ima[num_fig] != 0) {
	pcp_error ("window handle already used", "in section DEF", NULL);
	return;
    }
    if (num_ima < 1 || num_ima >= TELOP_VECM) {
	pcp_error ("Range error for image handle", "in section DEF", NULL);
	return;
    }

    /* stockage */
    op->fig->num_ima[num_fig] = num_ima;
    op->fig->legende[num_fig] = stg_Cpy (legende);
}


/***************************************************************************************
 * Vide la liste d'evenements
 *
 ***************************************************************************************/
void fig_vide_ev_list (fig)
Fig_data *fig;
{
    fig->ev_top = 0;
    fig->ev_list[0] = 0;
}


/***************************************************************************************
 *
 ***************************************************************************************/
void fig_show_windows (ico)
Ico *ico;
{
    Telop *op = (Telop *) ico->obj;
    Fig_data *fig = op->fig;  
    Ico *v[TELOP_VECM];
    int n, i;

    /*
     * Retrouve la selection, tests et tri
    */
    if (telop_retrouve_selec (ico, v, &n) < 0) return;

    /*
     * Parcourt liste de fig et affichage/creation fenetres
    */
    for (i = 0; i < OPER_VECM; i++) 
    if (fig->num_ima[i] > n) {
	char bla[200];
	sprintf (bla, "Trying to open Figure Window on image %d", fig->num_ima[i]);
	pcp_avertissement ("WARNING", bla, "which is not selected");

    } else if (fig->num_ima[i] != 0) {

	Ico *ico_image = v[fig->num_ima[i]];
	Edi *edi = fig->edi[i];

	if (edi == NULL) { 

	    /* Creation de la fenetre */
	    fig->edi[i] = fig_create_win (ico, ico_image, i, fig->legende[i]);

	} else if (edi->ico1 == ico_image) {

	    /* L'icone image n'a pas change', on met en avant-plan la fenetre. */
	    xv_set (edi->frame, XV_SHOW, TRUE, 0);
	    edi_repaint_direct (edi);

	} else {
	    /* La selection a change' : cf edi_choix_vision() */
	    Image *im;

	    /* Change les icones du client */
	    messa_ChangeIcoClient (edi->messa_index, edi->ico1->index_creat, 0);
	    if (edi->ico1 != edi->ico2)
	    messa_ChangeIcoClient (edi->messa_index, edi->ico2->index_creat, 0);
	    messa_ChangeIcoClient (edi->messa_index, 0, ico_image->index_creat);
	    edi->ico1 = edi->ico2 = ico_image;

	    /* MAJ de cette fenetre d'edition */
	    im = (Image *) edi->ico1->obj;
	    edi_maj_par1 (edi, im->tab, im->b.cmin, im->b.xsiz, im->b.ysiz, im->coul_tab);
	    edi_maj_nom (edi, fig->legende[i], edi->ico1->index_creat, edi->ico2->index_creat);
	    edi_maj_par2 (edi, im->tab, im->b.cmin, im->b.xsiz, im->b.ysiz, im->coul_tab);
	    edi_repaint_direct (edi);
	    rep_repaint_direct (edi);
	}
    }
}

/***************************************************************************************
 * Creation d'une fenetre fig.
 *
 * Retourne Edi*
 *
 * On conserve ico_telop dans le CLIENT_DATA 0.
 * DANGER : ico_telop doit exister quand on l'utilisera dans les callbacks !
 * SOLUTION : on rajoute ico_telop dans EDI_ICO_CLIENTS, de la sorte un
 * Clear sur l'ico provoquera un MESSA_KILL sur la fenetre !
 * DONC, si la fenetre existe, alors l'icone existe aussi, CQFD.
 *
 * Il ne faut pas que un clear sur le telOper provoque des FIG_DONE, on teste donc
 * si reason == EDI_DONE_BY_MOUSE dans la done_proc.
 *
 * Il ne faut pas que la fermeture d'une figure provoque un refresh general ; on met
 * donc EDI_DEAD_REFRESH a FAUX.
 *
 ***************************************************************************************/
Edi *fig_create_win (ico_telop, ico_image, num_fig, legende)
Ico *ico_telop, *ico_image;
int num_fig;
char *legende;
{
    return edi_create_win (
	EDI_ICO_IMAGE, 		ico_image,
	EDI_NAME, 		legende,
	EDI_EVENT_PROC,		fig_event_proc,
	EDI_REPAINT_PROC,	fig_repaint_proc,
	EDI_DONE_PROC,		fig_done_proc,
	EDI_ICO_CLIENTS,	ico_telop, NULL,
	EDI_CLIENT_DATA,	0, ico_telop,
	EDI_CLIENT_DATA,	1, num_fig,
	EDI_DEAD_REFRESH,	FAUX,
	NULL);
}


/***************************************************************************************
 * Callbacks de EDI_EVENT_PROC, EDI_REPAINT_PROC, EDI_DONE_PROC.
 *
 * On transmet l'evenement a fig_record_event
 *
 ***************************************************************************************/
void fig_repaint_proc (edi)
Edi *edi;
{
    fig_record_event (edi, FIG_REFRESH);
}

void fig_done_proc (edi, reason)
Edi *edi;
int reason;
{ 
    Ico *ico = (Ico *) edi->client_data[0];
    Telop *op = (Telop *) ico->obj;
    int num_fig = (int) edi->client_data[1];

    if (reason == EDI_DONE_BY_MOUSE)
	fig_record_event (edi, FIG_DONE);

    /* On dereference le Edi*, qui sera detruit dans edi_tuer_edi */
    op->fig->edi[num_fig] = NULL;
}

int fig_event_proc (edi, action)
Edi *edi;
int action;
{
    int event;

    switch (action) {
	case EDI_ACT_ENFONCE : event = FIG_PUSH;    break;
	case EDI_ACT_RELACHE : event = FIG_RELEASE; break;
	case EDI_ACT_TIRE    : event = FIG_DRAG;    break;
	case EDI_ACT_ANNULE  : event = FIG_CANCEL;  break;
	default : return FAUX;
    }

    fig_record_event (edi, event);

    return FAUX; /* ne pas redessiner */
}


/***************************************************************************************
 * Enregistre l'evenement, et lance eventuellement une requete
 *
 ***************************************************************************************/
void fig_record_event (edi, event)
Edi *edi;
int event;
{
    Ico *ico = (Ico *) edi->client_data[0];
    Telop *op = (Telop *) ico->obj;
    Fig_data *fig = op->fig;  
    int num_fig = (int) edi->client_data[1] + 1; /* cause stockage */
    char ordre[256];
    int len;

    /* 
     * Ecrit l'ordre a partir de event
    */
    switch (event) {

	case FIG_REFRESH :
	case FIG_DONE :
	    sprintf (ordre, "%d %d", event, num_fig);
	    break;

	case FIG_PUSH :
	case FIG_RELEASE :
	case FIG_DRAG :
	case FIG_CANCEL : {
	    double x1 = (double) edi->sx / (double) edi->zoom + edi->bxmin;
	    double y1 = (double) edi->sy / (double) edi->zoom + edi->bymin;
	    int    bt = (edi->sb == EDI_MOUSE_A) ? 1 : 2;

	    sprintf (ordre, "%d %d %d %d %.2lf %.2lf", 
		event, num_fig, bt, edi->zoom, x1, y1);
	} break;
    }

    /*
     * Stocke l'ordre en queue de ev_list
    */
    len = strlen (ordre);

    if (fig->ev_top + 1 + len + 1 > EV_LIST_MAX) 
	pcp_footer ("Sorry, event list is full", NULL);
    else {
	sprintf (fig->ev_list + fig->ev_top, " %s", ordre);
	fig->ev_top += len+1;
    }

    /*
     * Si rien n'est en cours, on lance une requete disant qu'il y a des event
     * dans ev_list.
    */
    if (!cs_is_working (ico)) {
	char req[256];

	sprintf (req, "400 %s", op->idf);
	cs_requete (ico, req);
    }
}


/***************************************************************************************
 * Decode la liste de dessins et dessine.
 *
 ***************************************************************************************/
void fig_decode_dessins (ico, buf)
Ico *ico;
char *buf;
{
    Telop *op = (Telop *) ico->obj;
    Fig_data *fig = op->fig;
    Edi *edi;
    int ordre, nfig, i, annuler_xor;
    char *p = buf; /* init while */

    /*
     * Lecture arguments
    */
    while ((ordre = atoi (stg_buf_rd (&p))) != 0) {

	/* Lecture de nfig */
	nfig = atoi (stg_buf_rd (&p));

	/* teste handle nfig et retrouve edi */
	if (nfig < 1 || nfig >= TELOP_VECM) {
	    printf ("Range error for window handle in section FIG\n");
	    nfig = 1;
	} 
	edi = fig->edi[nfig-1]; /* qui peut etre NULL */

	/* Mode de dessin et couleur */
	annuler_xor = FAUX;
	if (edi != NULL)
	    if (fig->draw_mode == FIG_XOR) {
		annuler_xor = VRAI;
		XSetFunction (edi->dpy, edi->gc, GXxor);
		XSetForeground (edi->dpy, edi->gc, 255);
	    } else {
		XSetForeground (edi->dpy, edi->gc, coul_bwc[fig->draw_color]);
	    }

	switch (ordre) {

	    case FIG_COLOR :
		fig->draw_color = atoi (stg_buf_rd (&p));
		break;
	    case FIG_MODE :
		fig->draw_mode = atoi (stg_buf_rd (&p));
		break;

	    case FIG_DRAW_POINT : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - edi->bymin) * edi->zoom;
		    XDrawPoint (edi->dpy, edi->xwin, edi->gc, ex1, ey1);
		}
	    } break;

	    case FIG_DRAW_LINE : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - edi->bymin) * edi->zoom,
		        ex2 = (double) (x2 - edi->bxmin) * edi->zoom,
		        ey2 = (double) (y2 - edi->bymin) * edi->zoom;
		    XDrawLine (edi->dpy, edi->xwin, edi->gc, ex1, ey1, ex2, ey2);
		}
	    } break;

	    case FIG_DRAW_RECT : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - edi->bymin) * edi->zoom,
		        ex2 = (double) (x2 - edi->bxmin) * edi->zoom,
		        ey2 = (double) (y2 - edi->bymin) * edi->zoom;
		    XDrawRectangle (edi->dpy, edi->xwin, edi->gc, 
			min(ex1,ex2), min(ey1,ey2), abs(ex2-ex1), abs(ey2-ey1));
		}
	    } break;

	    case FIG_FILL_RECT : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - edi->bymin) * edi->zoom,
		        ex2 = (double) (x2 - edi->bxmin) * edi->zoom,
		        ey2 = (double) (y2 - edi->bymin) * edi->zoom;
		    XFillRectangle (edi->dpy, edi->xwin, edi->gc, 
			min(ex1,ex2), min(ey1,ey2), abs(ex2-ex1)+1, abs(ey2-ey1)+1);
		}
	    } break;

	    case FIG_DRAW_ELLIP : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - edi->bymin) * edi->zoom,
		        ex2 = (double) (x2 - edi->bxmin) * edi->zoom,
		        ey2 = (double) (y2 - edi->bymin) * edi->zoom;
		    XDrawArc (edi->dpy, edi->xwin, edi->gc, min(ex1,ex2), min(ey1,ey2), 
			abs(ex2-ex1)+1, abs(ey2-ey1)+1, 0, 360*64);
		}
	    } break;

	    case FIG_DRAW_CIRC : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       r  = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - r - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - r - edi->bymin) * edi->zoom,
		        e2r = (double) r * 2 * edi->zoom;
		    XDrawArc (edi->dpy, edi->xwin, edi->gc, 
			ex1, ey1, e2r, e2r, 0, 360*64);
		}
	    } break;

	    case FIG_DRAW_ARC : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p));
		int    a1 = atoi (stg_buf_rd (&p)),
		       a2 = atoi (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - edi->bymin) * edi->zoom,
		        ex2 = (double) (x2 - edi->bxmin) * edi->zoom,
		        ey2 = (double) (y2 - edi->bymin) * edi->zoom;
		    XDrawArc (edi->dpy, edi->xwin, edi->gc, min(ex1,ex2), min(ey1,ey2), 
			abs(ex2-ex1)+1, abs(ey2-ey1)+1, a1, a2);
		}
	    } break;

	    case FIG_FILL_ARC : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p));
		int    a1 = atoi (stg_buf_rd (&p)),
		       a2 = atoi (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex1 = (double) (x1 - edi->bxmin) * edi->zoom,
		        ey1 = (double) (y1 - edi->bymin) * edi->zoom,
		        ex2 = (double) (x2 - edi->bxmin) * edi->zoom,
		        ey2 = (double) (y2 - edi->bymin) * edi->zoom;
		    XFillArc (edi->dpy, edi->xwin, edi->gc, min(ex1,ex2), min(ey1,ey2), 
			abs(ex2-ex1)+1, abs(ey2-ey1)+1, a1, a2);
		}
	    } break;

	    case FIG_DRAW_TRIANG : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p)),
		       x3 = atof (stg_buf_rd (&p)),
		       y3 = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    short xpts[8];
	  xpts[0] = xpts[6] = (double) (x1 - edi->bxmin) * edi->zoom;
	  xpts[1] = xpts[7] = (double) (y1 - edi->bymin) * edi->zoom;
		    xpts[2] = (double) (x2 - edi->bxmin) * edi->zoom;
		    xpts[3] = (double) (y2 - edi->bymin) * edi->zoom;
		    xpts[4] = (double) (x3 - edi->bxmin) * edi->zoom;
		    xpts[5] = (double) (y3 - edi->bymin) * edi->zoom;
		    XDrawLines (edi->dpy, edi->xwin, edi->gc, (XPoint *) xpts, 
			4, CoordModeOrigin);
		}
	    } break;

	    case FIG_FILL_TRIANG : {
		double x1 = atof (stg_buf_rd (&p)),
		       y1 = atof (stg_buf_rd (&p)),
		       x2 = atof (stg_buf_rd (&p)),
		       y2 = atof (stg_buf_rd (&p)),
		       x3 = atof (stg_buf_rd (&p)),
		       y3 = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    short xpts[6];
		    xpts[0] = (double) (x1 - edi->bxmin) * edi->zoom;
		    xpts[1] = (double) (y1 - edi->bymin) * edi->zoom;
		    xpts[2] = (double) (x2 - edi->bxmin) * edi->zoom;
		    xpts[3] = (double) (y2 - edi->bymin) * edi->zoom;
		    xpts[4] = (double) (x3 - edi->bxmin) * edi->zoom;
		    xpts[5] = (double) (y3 - edi->bymin) * edi->zoom ;
		    XFillPolygon (edi->dpy, edi->xwin, edi->gc, (XPoint *) xpts, 
			3, Convex, CoordModeOrigin);
		}
	    } break;

	    case FIG_DRAW_CROSS : {
		double x = atof (stg_buf_rd (&p)),
		       y = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex = (double) (x - edi->bxmin) * edi->zoom,
		        ey = (double) (y - edi->bymin) * edi->zoom;
		    XDrawLine (edi->dpy, edi->xwin, edi->gc, ex-3, ey, ex+3, ey);
		    XDrawLine (edi->dpy, edi->xwin, edi->gc, ex, ey-3, ex, ey+3);
		}
	    } break;

	    case FIG_DRAW_CURSOR : {
		double x = atof (stg_buf_rd (&p)),
		       y = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex = (double) (x - edi->bxmin) * edi->zoom,
		        ey = (double) (y - edi->bymin) * edi->zoom;
		    XDrawLine (edi->dpy, edi->xwin, edi->gc, ex-9, ey, ex-3, ey);
		    XDrawLine (edi->dpy, edi->xwin, edi->gc, ex+3, ey, ex+9, ey);
		    XDrawLine (edi->dpy, edi->xwin, edi->gc, ex, ey-9, ex, ey-3);
		    XDrawLine (edi->dpy, edi->xwin, edi->gc, ex, ey+3, ex, ey+9);
		}
	    } break;

	    case FIG_DRAW_MARK : {
		double x = atof (stg_buf_rd (&p)),
		       y = atof (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex = (double) (x - edi->bxmin) * edi->zoom,
		        ey = (double) (y - edi->bymin) * edi->zoom;
		    if (annuler_xor) {
			XSetFunction (edi->dpy, edi->gc, GXcopy);
			annuler_xor = FAUX;
		    }
		    XSetForeground (edi->dpy, edi->gc, coul_bwc[14]);
		    XDrawRectangle (edi->dpy, edi->xwin, edi->gc, ex-3, ey-3, 7, 7);
		    XSetForeground (edi->dpy, edi->gc, coul_bwc[0]);
		    XDrawRectangle (edi->dpy, edi->xwin, edi->gc, ex-4, ey-4, 9, 9);
		}
	    } break;

	    case FIG_DRAW_LABEL : {
		double x = atof (stg_buf_rd (&p)),
		       y = atof (stg_buf_rd (&p));
		int label = atoi (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex = (double) (x - edi->bxmin) * edi->zoom,
		        ey = (double) (y - edi->bymin) * edi->zoom;
		    if (annuler_xor) {
			XSetFunction (edi->dpy, edi->gc, GXcopy);
			annuler_xor = FAUX;
		    }
		    XSetForeground (edi->dpy, edi->gc, coul_bwc[14]);
		    XDrawRectangle (edi->dpy, edi->xwin, edi->gc, ex-3, ey-3, 7, 7);
		    XSetForeground (edi->dpy, edi->gc, coul_bwc[0]);
		    XDrawRectangle (edi->dpy, edi->xwin, edi->gc, ex-4, ey-4, 9, 9);
		    edi_loupe_label (edi, ex+6, ey+7, label);
		}
	    } break;

	    case FIG_DRAW_VALUE : {
		double x = atof (stg_buf_rd (&p)),
		       y = atof (stg_buf_rd (&p));
		int label = atoi (stg_buf_rd (&p));
		if (edi != NULL) {
		    int ex = (double) (x - edi->bxmin) * edi->zoom,
		        ey = (double) (y - edi->bymin) * edi->zoom;
		    if (annuler_xor) {
			XSetFunction (edi->dpy, edi->gc, GXcopy);
			annuler_xor = FAUX;
		    }
		    edi_loupe_label (edi, ex+2, ey+3, label);
		}
	    } break;

	    case FIG_DO_RUN :
		fig->do_run = VRAI;
		break;

	    case FIG_CENTER : {
		int sx = (int) atof (stg_buf_rd (&p)),
		    sy = (int) atof (stg_buf_rd (&p)),
		    dx = sx - edi->bxmin - (edi->cxdif/2) / edi->zoom,
		    dy = sy - edi->bymin - (edi->cydif/2) / edi->zoom;

	        /* ajustement des bornes */
	        edi->bxmin += dx; edi->bxmax += dx;
	        edi->bymin += dy; edi->bymax += dy;
	        edi_reborne (edi);

		if (annuler_xor) {
		    XSetFunction (edi->dpy, edi->gc, GXcopy);
		    annuler_xor = FAUX;
		}
		edi_repaint_direct (edi);
		rep_repaint_direct (edi);
		/* Provoque un appel de la section FIG avec event = FIG_REFRESH */	
	      } break;

	    case FIG_CLEAN :
		if (annuler_xor) {
		    XSetFunction (edi->dpy, edi->gc, GXcopy);
		    annuler_xor = FAUX;
		}
		edi_repaint_direct (edi);
		/* Provoque un appel de la section FIG avec event = FIG_REFRESH */
		break;

	    default : printf ("Error in fig_decode_dessins()\n");
	}

	/* Annule mode de dessin XOR */
	if (annuler_xor)
	    XSetFunction (edi->dpy, edi->gc, GXcopy);
    }
}




