/***************************************************************************************/
/* host.c                                                     I.P.S  v 3.6  02/12/1995 */
/*                                                                                     */
/* Gestion de la fenetre "oper host"                     Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"


static Frame ho_frame;
static Panel_item ho_host_item;


void ho_creer_frame ();
void ho_affi_frame ();
void ho_butt_proc ();
char *ho_get_host ();
void ho_sel_change_host ();
void ho_aide ();

/***************************************************************************************
 * Cre'e' la fenetre
 *
 ***************************************************************************************/
void ho_creer_frame ()
{
    Panel panel;

    ho_frame = (Frame) xv_create (glo.frame_principal, FRAME_CMD,
	FRAME_LABEL, "Remote host",
	NULL);

    panel = (Panel) xv_get (ho_frame, FRAME_CMD_PANEL);

    ho_host_item = (Panel_item) xv_create (panel, PANEL_TEXT,
	PANEL_LABEL_STRING, "Host :",
	PANEL_VALUE, "",
	PANEL_VALUE_DISPLAY_LENGTH, 25,
	PANEL_VALUE_STORED_LENGTH, 128,
	NULL);

    xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, NULL);

    (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING, "Help",
	PANEL_NOTIFY_PROC, ho_butt_proc,
	NULL);

    xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

    (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING, "Change",
	PANEL_NOTIFY_PROC, ho_butt_proc,
	NULL);

    window_fit (panel);
    window_fit (ho_frame);
}

/***************************************************************************************
 * Affiche la fenetre
 *
 ***************************************************************************************/
void ho_affi_frame ()
{
    xv_set (ho_frame, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, NULL);
}

/***************************************************************************************
 * Callback des boutons de la fenetre
 *
 ***************************************************************************************/
void ho_butt_proc (item, event)
Panel_item item;
Event *event;
{
    char *bouton = (char*) xv_get (item, PANEL_LABEL_STRING);

    if (strcmp (bouton, "Help") == 0) {

	ho_aide ();

    } else if (strcmp (bouton, "Change") == 0) {

	ho_sel_change_host ();
    }
}

/***************************************************************************************
 * Renvoie le nom du host,
 * en fonction du localhost, de hlist et du nom dans la fenetre.
 * Un retour vide signifie localhost.
 *
 ***************************************************************************************/
char *ho_get_host (hlist)
char *hlist;
{
    char *nom = (char *) xv_get (ho_host_item, PANEL_VALUE);
    static char res[64];

    /* Par defaut reponse = "" pour localhost */
    res[0] = 0;

    /* On lit la ligne dans la boite de dialogue */
    if (nom == (char *) NULL || nom[0] == 0) {

	/* Le nom est vide, on regarde la hostlist */
	if (hlist[0] == 0) return res;

	/* On cherche localhost dans hostlist */
	nom = strstr (hlist, glo.host_name);
	if (nom != (char *) NULL) return res;

	/* On renvoie le premier nom de la hostlist */
	sscanf (hlist, "%s", res);
	return res;
    }

    /* Verifie que le host existe */
    if ((struct hostent *) gethostbyname (nom) == NULL) {
	perror ("ERROR in gethostbyname() ");
	return res;
    }
    return nom;
}

/***************************************************************************************
 * Change le host sur une selection
 *
 ***************************************************************************************/
void ho_sel_change_host ()
{
    Ico *cur;
    char *nom_host = ho_get_host ("");

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
    if (cur->sel) switch (cur->nat_obj) {

	case IPS_TELOP : {
	    Telop *op = (Telop *) cur->obj;

	    if (op->ctx == -1) {
		pcp_free (op->nom_host);
		op->nom_host = stg_Cpy (nom_host);
		telop_parbox_change_titre (op);
	    }
	} break;
    }

}

/***************************************************************************************
 * Message d'aide
 *
 ***************************************************************************************/
void ho_aide ()
{
    aide_affi_message (
"			Help on \"Remote host\" window \
                        ---------------------------- \
\
\
- Each time you create a telOper icon, the value of the \"Host :\" line is checked.\
  If the line is empty, then the telOper is run on local host (default),\
  else it is run on the remote host you have choosen.\
\
- To change the host on several icons : select some icons, then press \"Change\".\
  Each icon in the selection, being a telOper, and currently not alive, will\
  use this new host.  \
\
The host is recalled in the title of the \"Param\" windows.\
\
"); 
}

