/***************************************************************************************/
/* obj_image.c                                                 I.P.S  v 3.3  18/3/1994 */
/*                                                                                     */
/* gestion de l'objet image (ico->nat_obj = IPS_IMAGE)   Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/


#include "inclus.h"

static Frame		frame_image_props;
static Panel_item	panel_image_props_cmin,
			panel_image_props_xsiz,
			panel_image_props_ysiz,
			panel_image_props_lut,
			panel_image_props_name;
static Ico		*ico_image_props;
static int		index_messa_image_props = 0;


/***************************************************************************************
 * Calcul des bornes de l'image                                                        *
 *                                                                                     *
 ***************************************************************************************/
void
image_calc_bornes (image, cmin, xsiz, ysiz)
Image *image;
int cmin, xsiz, ysiz;
{ 
  image->b.cmin = cmin;
  image->b.xsiz = xsiz;
  image->b.xmax = xsiz + cmin;
  image->b.xtot = xsiz + cmin*2;
  image->b.ysiz = ysiz;
  image->b.ymax = ysiz + cmin;
  image->b.ytot = ysiz + cmin*2;
  image->b.xytot = image->b.xtot * image->b.ytot;
}

/***************************************************************************************
 * Creation et initialisation d'une image                                              *
 * si tab == NULL alors creation bitmap sinon image->tab = tab;                        *
 ***************************************************************************************/
void
image_creat (image, cmin, xsiz, ysiz, tab)
Image *image;
int cmin, xsiz, ysiz;
Unt16 *tab;
{ 
  int t;

  image_calc_bornes (image, cmin, xsiz, ysiz);

  if (tab == NULL) {
      if (( image->tab = (Unt16 *) pcp_alloc ( image->b.xytot * sizeof (Unt16)) ) != NULL)
          for (t = 0; t < image->b.xytot; t++) image->tab[t] = 0;
  }
  else image->tab = tab;

  image->coul_tab = coul_pic;
  image->ch1[0] = image->ch2[0] = image->ch3[0] = 0;

  image->hub_xima = (XImage *) NULL;
}
 
/***************************************************************************************
 * Suppression d'une image                                                             *
 *                                                                                     *
 ***************************************************************************************/
void
image_free (image)
Image *image;
{ 
  pcp_free (image->tab);
  if (image->hub_xima != (XImage *) NULL) XDestroyImage (image->hub_xima);
}
 
/***************************************************************************************
 * Duplication d'une image                                                             *
 *                                                                                     *
 ***************************************************************************************/
void
image_duplic (ico_n, ico_o)
Ico *ico_n, *ico_o;
{ 
  Image *image_o = (Image *) ico_o->obj;
  Image *image_n = (Image *) ico_n->obj;

  memcpy (image_n, image_o, sizeof(Image));
  image_prepare_noms (ico_n);
  if ((image_n->tab = (Unt16 *) pcp_alloc ( image_o->b.xytot * sizeof (Unt16))) != NULL)
      memcpy (image_n->tab, image_o->tab, image_o->b.xytot * sizeof(Unt16));

  image_n->hub_xima = (XImage *) NULL;
}
 
/***************************************************************************************
 * provoque futur reaffichage complet du hublot                                        *
 *                                                                                     *
 ***************************************************************************************/
void
image_obsolete (image)
Image *image;
{ 
  if (image->hub_xima != (XImage *) NULL) {
      XDestroyImage (image->hub_xima);
      image->hub_xima = (XImage *) NULL;
  }
}
 
/***************************************************************************************
 * Changement de la taille du cadre d'une image                                        *
 *                                                                                     *
 ***************************************************************************************/
int
image_chg_cad (ico, n_cad, n_xsiz, n_ysiz)
Ico *ico;
int    n_cad, n_xsiz, n_ysiz;
{ 
  Image *image  = (Image *) ico->obj;
  Unt16	*tmp;
  int    x, y, t, o_tot, n_tot, o_t, n_t, o_cad, o_xsiz, o_ysiz, xsiz, ysiz;

  if (ico->lock & LOCK_MODIF) return FAUX;

  o_cad  = image->b.cmin;
  o_tot  = image->b.xtot;
  o_xsiz = image->b.xsiz;
  o_ysiz = image->b.ysiz;

  image_calc_bornes (image, n_cad, n_xsiz, n_ysiz);

  n_tot = image->b.xtot;

  if ((tmp = (Unt16 *) pcp_alloc ( image->b.xytot * sizeof (Unt16))) != NULL) {
      for (t = 0; t < image->b.xytot; t++) tmp[t] = 0;

      o_t = o_cad * (o_tot + 1);
      n_t = n_cad * (n_tot + 1);
      xsiz = (o_xsiz < n_xsiz) ? o_xsiz : n_xsiz;
      ysiz = (o_ysiz < n_ysiz) ? o_ysiz : n_ysiz;

      for (y = 0; y < ysiz; y++) 
      {
        for (x = 0; x < xsiz; x++)
        tmp[n_t+x] = image->tab[o_t+x];
        o_t += o_tot;
        n_t += n_tot;
      }
      pcp_free (image->tab);
      image->tab = tmp;
      return VRAI;
  }
  else {
      image_calc_bornes (image, o_cad, o_xsiz, o_ysiz);
      return FAUX;
  }
}
 
/***************************************************************************************
 * Creation de la boite de dialogue des proprietes d'une image                         *
 *                                                                                     *
 ***************************************************************************************/
void
image_props_creer_frame ()
{ 
  Panel   panel;

  frame_image_props = (Frame) xv_create (glo.frame_principal, FRAME_CMD,
      FRAME_LABEL,              "Image Properties",
      XV_WIDTH,                 285,
      XV_HEIGHT,                180,
      NULL);

  panel = (Panel) xv_get (frame_image_props, FRAME_CMD_PANEL);
  xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, NULL);

  panel_image_props_cmin = (Panel_item) xv_create (panel, PANEL_NUMERIC_TEXT,
      PANEL_LABEL_STRING,         "Width of the frame =",
      PANEL_LABEL_X,              10,
      PANEL_VALUE_X,              200,
      PANEL_MIN_VALUE,            0,
      PANEL_MAX_VALUE,            255,
      PANEL_VALUE_DISPLAY_LENGTH, 5,
      PANEL_VALUE,                0,
      XV_Y,                       10,
      NULL);

  panel_image_props_ysiz = (Panel_item) xv_create (panel, PANEL_NUMERIC_TEXT,
      PANEL_LABEL_STRING,         "Number of rows (y size) =",
      PANEL_LABEL_X,              10,
      PANEL_VALUE_X,              200,
      PANEL_MIN_VALUE,            8,
      PANEL_MAX_VALUE,            16384,
      PANEL_VALUE_DISPLAY_LENGTH, 5,
      PANEL_VALUE,                0,
      NULL);

  panel_image_props_xsiz = (Panel_item) xv_create (panel, PANEL_NUMERIC_TEXT,
      PANEL_LABEL_STRING,         "Number of cols (x size) =",
      PANEL_LABEL_X,              10,
      PANEL_VALUE_X,              200,
      PANEL_MIN_VALUE,            8,
      PANEL_MAX_VALUE,            16384,
      PANEL_VALUE_DISPLAY_LENGTH, 5,
      PANEL_VALUE,                0,
      NULL);

  panel_image_props_lut = (Panel_item) xv_create (panel, PANEL_CHOICE_STACK,
      PANEL_LAYOUT,          PANEL_HORIZONTAL,   
      PANEL_LABEL_STRING,    "L.U.T.",
      PANEL_CHOICE_STRINGS,  IPS_COUL_LIST, NULL,
      PANEL_VALUE,           0,
      XV_X,                  10,
      NULL);

  panel_image_props_name = (Panel_item) xv_create (panel, PANEL_TEXT,
      PANEL_LABEL_STRING,         "Name :",
      PANEL_VALUE,                "",
      PANEL_VALUE_DISPLAY_LENGTH, 25,
      PANEL_VALUE_STORED_LENGTH,  IMAGE_CH1LEN,
      XV_X,                  10,
      NULL);

  (void) xv_create (panel, PANEL_BUTTON,   
      PANEL_LABEL_STRING,    "Apply",
      PANEL_NOTIFY_PROC,     image_props_proc,
      XV_X,                  30,
      NULL);

  xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

  (void) xv_create (panel, PANEL_BUTTON,   
      PANEL_LABEL_STRING,    "Apply & Edit",
      PANEL_NOTIFY_PROC,     image_props_proc,
      NULL);

  (void) xv_create (panel, PANEL_BUTTON,   
      PANEL_LABEL_STRING,    "Cancel",
      PANEL_NOTIFY_PROC,     image_props_proc,
      NULL);

  window_fit (panel);
  window_fit (frame_image_props);
}
 
/***************************************************************************************
 * Mise en place des param dans la boite de dialogue des proprietes d'une image        *
 *                                                                                     *
 ***************************************************************************************/
void
image_props_affi_frame (ico)
Ico *ico;
{ 
  Image *im = (Image *) ico->obj;
  int    mx = (int) xv_get (glo.frame_principal, XV_X) + ico->xb;
  int    my = (int) xv_get (glo.frame_principal, XV_Y) + ico->yb;

  messa_DeleteClient (index_messa_image_props);
  image_props_rescan (im);
  xv_set (frame_image_props, XV_X, mx, XV_Y, my, XV_SHOW, TRUE,
          FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
  ico_image_props = ico;
  index_messa_image_props = messa_NewClient (
	image_props_messaProc, im, &ico->index_creat, 1);
}

void image_props_rescan (im)
Image *im;
{
  xv_set (panel_image_props_cmin, PANEL_VALUE, im->b.cmin, NULL);
  xv_set (panel_image_props_xsiz, PANEL_VALUE, im->b.xsiz, NULL);
  xv_set (panel_image_props_ysiz, PANEL_VALUE, im->b.ysiz, NULL);
  xv_set (panel_image_props_lut,
	PANEL_VALUE, glo_get_attr_lut (im->coul_tab), 
	NULL);
  xv_set (panel_image_props_name, PANEL_VALUE, im->ch1, NULL);
}

/***************************************************************************************
 * _messaProc de la boite de dialogue des proprietes d'une image                       *
 * (elle est appelee uniquement si ico == ico_image_props)                             *
 ***************************************************************************************/
void image_props_messaProc (client_data, message)
void     *client_data;
Messa_ev *message;
{
    Image *im = (Image *) client_data;

#   ifdef DEBUG
    printf ("image_props_messaProc\n");
#   endif

    if (messa_rescan(message) || messa_lut(message) || messa_name(message))
	image_props_rescan (im);

    if (messa_kill(message)) {
	xv_set (frame_image_props, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, NULL);
	ico_image_props = NULL;
        messa_DeleteClient (index_messa_image_props);
    }
}

/***************************************************************************************
 * Notify proc de la boite de dialogue des proprietes d'une image                      *
 *                                                                                     *
 ***************************************************************************************/
void
image_props_proc (item, event)
Panel_item item;
Event *event;
{ 
  char  *bouton = (char*) xv_get (item, PANEL_LABEL_STRING);
  int    cmin   = (int)   xv_get (panel_image_props_cmin, PANEL_VALUE);
  int    xsiz   = (int)   xv_get (panel_image_props_xsiz, PANEL_VALUE);
  int    ysiz   = (int)   xv_get (panel_image_props_ysiz, PANEL_VALUE);
  int    lutn   = (int)   xv_get (panel_image_props_lut,  PANEL_VALUE);
  char  *name   = (char*) xv_get (panel_image_props_name, PANEL_VALUE);
  Ico   *ico    = ico_image_props;
  int    cas;

#ifdef DEBUG
  printf("image_props_proc %s\n", bouton);
#endif

  xv_set (frame_image_props, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, NULL);

  if      (!strcmp (bouton, "Apply"       )) cas = 1;
  else if (!strcmp (bouton, "Apply & Edit")) cas = 2;
  else if (!strcmp (bouton, "Cancel"      )) cas = 3;
  else cas = 3;

  image_props_direct (ico, cas, cmin, xsiz, ysiz, lutn, name);
}

/***************************************************************************************
 * Gestion des proprietes d'une image et envoi message MAJ                             *
 *                                                                                     *
 * Retour: test si le realloc eventuel s'est bien passe'                               *
 ***************************************************************************************/
int
image_props_direct (ico, cas, cmin, xsiz, ysiz, lutn, name)
Ico *ico;
int  cas, cmin, xsiz, ysiz, lutn;
char *name;
{
  Image *image  = (Image *) ico->obj;
  int    o_xm   = ico->xm;
  int    o_ym   = ico->ym;
  int    raf_ico = FAUX;
  int    raf_edi = FAUX;
  int	 test = VRAI;

  if (ico->lock & LOCK_MODIF) return FAUX;

  if (cas == 1 || cas == 2)
  {
    /* changement de palette */
    if (lutn != glo_get_attr_lut (image->coul_tab))
    {
      image->coul_tab = glo_get_ctab_lut (lutn);
      raf_ico = raf_edi = VRAI;
    }

    /* changement de taille */
    if (ysiz != image->b.ysiz || xsiz != image->b.xsiz) {
        test = image_chg_cad (ico, cmin, xsiz, ysiz);
        image_calc_hublot (ico);
        raf_ico = raf_edi = VRAI;
    }
    else if (cmin != image->b.cmin)
        test = image_chg_cad (ico, cmin, xsiz, ysiz);

    /* changement de nom */
    if (strcmp (image->ch1, name))
    {
      strncpy (image->ch1, name, IMAGE_CH1LEN);
      image_prepare_noms (ico);
      raf_ico = VRAI;
    }

    /* MAJ affi de l'ico */
    if (raf_ico) {
        method_obsolete (ico);
        ico_homot_clic (ico, ico->xb, ico->yb, o_xm, o_ym, ico->xm, ico->ym);
    }

    /* Envoi message pour MAJ generale */
    messa_Send (
	MESSA_FROM_ICO,	ico,
	MESSA_TO_ICO,	ico,
	MESSA_RESCAN,	VRAI,
	MESSA_REFRESH,	raf_edi,
	NULL);
  }

  if (cas == 2) edIMA_edit_ico (ico, FAUX);

  return test;
}
 
/***************************************************************************************
 * Calcul des bornes du hublot dans l'icone                                            *
 *                                                                                     *
 ***************************************************************************************/
void image_calc_hublot (ico)
Ico *ico;
{
  Image *im   = (Image *) ico->obj;
  int    size = (im->b.xsiz > im->b.ysiz) ? im->b.xsiz : im->b.ysiz;
  int    b2   = 80;

  /* Ces params sont fixes ici uniquement */
  im->hub_max = 64;
  im->hub_ext = 10;

  /* calcul du zoom et du step en x */
  im->hub_zoom = 1;
  im->hub_step = 1;
  if (size < im->hub_max) {
	im->hub_zoom = (im->hub_max - 1)/size +1;
	if (size * im->hub_zoom > b2) im->hub_zoom = b2 / size;
  }
  else if (size > im->hub_max) {
	im->hub_step = size / im->hub_max;
	if (size / im->hub_step > b2) im->hub_step = (size - 1)/b2 + 1;
  }

  /* Calcul de la taille du hublot */
  im->hub_xm = im->b.xsiz * im->hub_zoom / im->hub_step;
  im->hub_ym = im->b.ysiz * im->hub_zoom / im->hub_step;
  if (im->hub_xm > im->hub_max) im->hub_xm = im->hub_max;
  if (im->hub_ym > im->hub_max) im->hub_ym = im->hub_max;

  /* Calcul des titres */
  image_prepare_noms (ico);

  /* calcul des coord en x */
  ico->xm    = im->hub_max + 2*im->hub_ext;
  im->hub_xb = (ico->xm - im->hub_xm) / 2;

  /* calcul des coord en y */
  im->y_ch1  = 2 + glo.font_h;
  im->y_ch2  = im->y_ch1 + glo.font_h + 1;
  im->hub_yb = im->y_ch2 + glo.font_d + 2;
  im->y_ch3  = im->hub_yb + im->hub_ym + glo.font_h +1;
  ico->ym    = im->y_ch3 + glo.font_d + 2;
}

/***************************************************************************************
 * Affichage de l'objet Image dans l'icone                                             *
 *                                                                                     *
 ***************************************************************************************/
void image_affi_ico (ico)
Ico *ico;
{
  int x, y;
  Image *im = (Image *) ico->obj;

  x = ico->xb + im->hub_xb;
  y = ico->yb + im->hub_yb;

  XSetForeground (glo.dpy, glo.gc, glo.c[6]);
  XDrawRectangle (glo.dpy, glo.xwin, glo.gc, x-1, y-1, im->hub_xm+1, im->hub_ym+1 );

  image_affi_hublot (im, x, y);

  XSetForeground (glo.dpy, glo.gc, glo.c[6]);
  XDrawString (glo.dpy, glo.xwin, glo.gc, 
               ico->xb+im->x_ch1, ico->yb+im->y_ch1, im->ch1, im->s_ch1);
  XDrawString (glo.dpy, glo.xwin, glo.gc, 
               ico->xb+im->x_ch2, ico->yb+im->y_ch2, im->ch2, im->s_ch2);
  XDrawString (glo.dpy, glo.xwin, glo.gc, 
               ico->xb+im->x_ch3, ico->yb+im->y_ch3, im->ch3, im->s_ch3);
}

/***************************************************************************************
 * Affichage du pixmap dans le hublot de l'icone                                       *
 *                                                                                     *
 ***************************************************************************************/
void image_affi_hublot (image, x, y)
Image *image;
int    x, y;
{
  if (image->hub_xima == (XImage *) NULL) { 
      int      nb_x   = (image->hub_xm-1) / image->hub_zoom +1;
      int      nb_y   = (image->hub_ym-1) / image->hub_zoom +1;
      int      taille = nb_x * nb_y * image->hub_zoom * image->hub_zoom;
      
      if ( image->tab == NULL ) return;
      
      if (glo.depth <= 8) {
          char     *data;

          if ( (data = (char*) malloc (taille * sizeof(char))) == NULL )
              return;
          edi_vecteur_ecran ( data, image->hub_zoom, image->hub_step, image->b.cmin,
              image->b.cmin, nb_x, nb_y, image->b.xtot, image->tab, image->coul_tab);

          image->hub_xima = XCreateImage (glo.dpy, glo.visual, glo.depth, ZPixmap, 0, data,
              nb_x * image->hub_zoom, nb_y * image->hub_zoom, 8, 0);
              
      } else { /* mode RGB */
      
          char *dataR = NULL, *dataG = NULL, *dataB = NULL;
          if ( (dataR = (char*) calloc (taille, sizeof(char))) == NULL ||
               (dataG = (char*) calloc (taille, sizeof(char))) == NULL ||
               (dataB = (char*) calloc (taille, sizeof(char))) == NULL )
          { free (dataR); free (dataG); free (dataB); return; }

          edi_vecteur_ecran_RGB ( dataR, dataG, dataB,
                      image->hub_zoom, image->hub_step, image->b.cmin,
		      image->b.cmin, nb_x, nb_y, image->b.xtot, image->tab,
                      glo_get_colR(image->coul_tab),
                      glo_get_colG(image->coul_tab),
                      glo_get_colB(image->coul_tab)
              );
           image->hub_xima = CoulxCalcXImage (dataR, dataG, dataB,
              nb_x * image->hub_zoom, nb_y * image->hub_zoom);
              
           free (dataR); free (dataG); free (dataB);
       }
  }
  
  XPutImage (glo.dpy, glo.xwin, glo.gc, image->hub_xima,
      0, 0, x, y, image->hub_xm, image->hub_ym);
}

/***************************************************************************************
 * Calcule les chaines ch1, ch2 et ch3 + leurs coord / ico a partir du titre           *
 *                                                                                     *
 ***************************************************************************************/
void image_prepare_noms (ico)
Ico *ico;
{
  Image *im = (Image *) ico->obj;
  int    i, c;
  char   bla[64];
  Font_string_dims dims;

  /* attribution de ch1 */
  c = VRAI; i = 0;
  while (c)
  {
    if (im->ch1[i] == 0)
         c = FAUX;
    else {
           bla[i] = im->ch1[i]; bla[i+1] = 0;
           (void) xv_get (glo.petite_fonte, FONT_STRING_DIMS, bla, &dims);
           if (dims.width > im->hub_max+10) { c = FAUX; bla[i] = 0; }
         }
    i++;
  }

  /* attribution de ch2, ch3 et s_ch */
  sprintf (im->ch2, "n.%d", ico->index_creat); 
  sprintf (im->ch3, "%d.%d", im->b.ysiz, im->b.xsiz);
  im->s_ch1 = strlen(bla);
  im->s_ch2 = strlen(im->ch2);
  im->s_ch3 = strlen(im->ch3);


  /* attribution des coord en x des ch */
  im->x_ch1 = im->hub_ext-5;
  (void) xv_get (glo.petite_fonte, FONT_STRING_DIMS, im->ch2, &dims);
  im->x_ch2 = im->hub_ext + im->hub_max - dims.width;
  (void) xv_get (glo.petite_fonte, FONT_STRING_DIMS, im->ch3, &dims);
  im->x_ch3 = im->hub_ext + im->hub_max - dims.width;
}

/****************************************************************************************
 * FUNCTION  : image_CloseAllWin (Ico *ico);
 *
 * CALLED IN : method_free
 *
 ****************************************************************************************/
void image_CloseAllWin (ico)
Ico *ico;
{
    messa_Send (
	MESSA_FROM_ICO, ico,
	MESSA_TO_ICO, ico,
	MESSA_KILL, VRAI,
	NULL);
}

