/***************************************************************************************/
/* obj_oper.c                                                 I.P.S  v 3.1  18/11/1993 */
/*                                                                                     */
/* gestion de l'objet oper (ico->nat_obj = IPS_OPER)     Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/


#include "inclus.h"

static Menu menu_oper;

/****************************************************************************************
 * FUNCTION  : oper_init ( Oper *op, char *idf );
 *
 * PURPOSE   : Initialisation de l'oper op
 *
 * CALLED IN : oper_FramePcpMenuProc
 *
 ***************************************************************************************/
void   oper_init (op, menu_idf)
Oper  *op;
char  *menu_idf;
{ 
  op->idf		= stg_Cpy (menu_idf);	/* idf de la commande lue dans menu	*/
  op->led_busy		= 0;			/* 0=sleep, 1=wait, 2=running		*/
  op->nb_image_in	= 0;			/* nb d'images en entree		*/
  op->nb_image_out	= 0;			/* nb d'images en sortie		*/
  op->same_size		= VRAI;			/* meme taille des images existantes	*/
  op->val_cadre		= IPS_BORDER_NOONE;	/* cadre autour images : IPS_BORDER_	*/
  op->mot_x		= 10;			/* largeur max des sous titres		*/
  op->param_nb		= 0;			/* nb de params				*/
  op->param_type	= NULL;			/* vecteur des types des parametres	*/
  op->param_adr		= NULL;			/* vecteur des adresses des parametres	*/
  op->titre		= NULL;			/* titre de l'oper			*/
  op->lgd_images	= NULL;			/* Legendes pr les images en in		*/
  op->frame_parbox	= NULL;			/* Frame de la boite de dialogue 	*/
  op->attach_n		= 0;			/* aucun oper attache			*/
  op->attach_v		= NULL;			/* vect des num icos attaches a op	*/
}
 
/****************************************************************************************
 * FUNCTION  : oper_free ( Oper *op );
 *
 * PURPOSE   : Suppression de l'oper op
 *
 * CALLED IN : method_free
 *
 ***************************************************************************************/
void   oper_free (op)
Oper *op;
{
  int i, t, n, k;

  /* idf */
  if (op->idf != NULL) pcp_free (op->idf);

  /* titre de l'oper */
  if (op->titre != NULL) { op->titre[0] = 0; pcp_free(op->titre); }

  /* Legendes pr les images en in,out */
  n = op->nb_image_in + op->nb_image_out;
  for (i = 0; i < n; i++) pcp_free (op->lgd_images[i]);
  if (n > 0) pcp_free (op->lgd_images);

  /* attachement */
  if (op->attach_n > 0) pcp_free (op->attach_v);
 
  if (op->param_nb > 0)
  {
  /* vecteur des adresses des parametres */
    for (i = 0; i < op->param_nb; i++)
    switch (t = op->param_type[i])
    {
      case IPS_PAR_REAL : {
          Oper_par_real *para = (Oper_par_real *) op->param_adr[i];
          pcp_free (para->legende);
        } break;

      case IPS_PAR_CHK_BOX :
      case IPS_PAR_SLIDER :
      case IPS_PAR_INT : {
          Oper_par_int *para = (Oper_par_int *) op->param_adr[i];
          pcp_free (para->legende);
        } break;

      case IPS_PAR_TEXT : {
          Oper_par_text *para = (Oper_par_text *) op->param_adr[i];
          pcp_free (para->legende);
          pcp_free (para->defaut);
          pcp_free (para->valeur);
        } break;

      case IPS_PAR_CH_ABR :
      case IPS_PAR_CHOICE : {
          Oper_par_choice *para = (Oper_par_choice *) op->param_adr[i];
          for (k = 0; k < para->nchoice; k++) pcp_free (para->choices[k]);
          pcp_free (para->choices);
          pcp_free (para->legende);
        } break;

      default : pcp_error ("oper_free #01", "type not implemented", NULL);
    }

    for (i = 0; i < op->param_nb; i++) pcp_free (op->param_adr[i]);
    pcp_free (op->param_adr);

    /* vecteur des types des parametres */
    pcp_free (op->param_type);
  }
}
 
/****************************************************************************************
 * FUNCTION  : oper_duplic ( Ico *ico_new, *ico_old );
 *
 * PURPOSE   : Duplication de l'icone IPS_OPER : ico_n := ico_o
 *
 * CALLED IN : method_duplic
 *
 ***************************************************************************************/
void  oper_duplic (ico_n, ico_o)
Ico  *ico_n, *ico_o;
{ 
  Oper *op_o = (Oper *) ico_o->obj;
  Oper *op_n = (Oper *) ico_n->obj;
  int i, t, n, k;

  memcpy (op_n, op_o, sizeof(Oper));

  /* idf */
  op_n->idf = stg_Cpy (op_o->idf);
 
  /* titre de l'oper */
  op_n->titre = stg_Cpy (op_o->titre);

  /* Legendes pr les images en in,out */
  n = op_o->nb_image_in + op_o->nb_image_out;
  if (n > 0) 
  { op_n->lgd_images = (char **) pcp_alloc (n * sizeof(char *));
    for (i = 0; i < n; i++)
      op_n->lgd_images[i] = stg_Cpy (op_o->lgd_images[i]);
  }
 
  /* attachement */
  op_n->led_busy = 0;
  op_n->attach_n = 0;
  op_n->attach_v = NULL;

  if (op_o->param_nb > 0)
  {
    /* vecteur des types des parametres */
    op_n->param_type = (int *) pcp_alloc (op_o->param_nb * sizeof(int));
    for (i = 0; i < op_o->param_nb; i++) 
    op_n->param_type[i] = op_o->param_type[i];

    /* vecteur des adresses des parametres */
    op_n->param_adr = (void **) pcp_alloc (op_o->param_nb * sizeof(void *));
    for (i = 0; i < op_o->param_nb; i++)
    switch (t = op_o->param_type[i])
    {
      case IPS_PAR_REAL : {
          Oper_par_real *par_o = (Oper_par_real *) op_o->param_adr[i];
          Oper_par_real *par_n = (Oper_par_real *) pcp_alloc (sizeof(Oper_par_real));

          (Oper_par_real *) op_n->param_adr[i] = par_n;

          memcpy (par_n, par_o, sizeof(Oper_par_real));

          par_n->legende = stg_Cpy (par_o->legende); 
        } break;

      case IPS_PAR_CHK_BOX :
      case IPS_PAR_SLIDER :
      case IPS_PAR_INT  : {
          Oper_par_int *par_o = (Oper_par_int *) op_o->param_adr[i];
          Oper_par_int *par_n = (Oper_par_int *) pcp_alloc (sizeof(Oper_par_int));

          (Oper_par_int *) op_n->param_adr[i] = par_n;

          memcpy (par_n, par_o, sizeof(Oper_par_int));

          par_n->legende = stg_Cpy (par_o->legende); 
        } break;

      case IPS_PAR_TEXT : {
          Oper_par_text *par_o = (Oper_par_text *) op_o->param_adr[i];
          Oper_par_text *par_n = (Oper_par_text *) pcp_alloc (sizeof(Oper_par_text));

          (Oper_par_text *) op_n->param_adr[i] = par_n;

          par_n->taille_max = par_o->taille_max;

          par_n->legende = stg_Cpy (par_o->legende);
          par_n->defaut  = stg_Cpy (par_o->defaut);
          par_n->valeur  = (char *) pcp_alloc (par_o->taille_max + 1);
          strcpy (par_n->valeur,  par_o->valeur); 
        } break;

      case IPS_PAR_CH_ABR :
      case IPS_PAR_CHOICE : {
          Oper_par_choice *par_o = (Oper_par_choice *) op_o->param_adr[i];
          Oper_par_choice *par_n = (Oper_par_choice *) pcp_alloc (sizeof(Oper_par_choice));

          (Oper_par_choice *) op_n->param_adr[i] = par_n;

          memcpy (par_n, par_o, sizeof(Oper_par_choice));

          par_n->legende = stg_Cpy (par_o->legende);
          par_n->choices = (char **) pcp_alloc (par_o->nchoice * sizeof (char *));
          for (k = 0; k < par_o->nchoice; k++)
            par_n->choices[k] = stg_Cpy (par_o->choices[k]);
        } break;

      default : pcp_error ("oper_duplic #01", "type not implemented", NULL);
    }
  }
  else { op_n->param_adr = NULL; op_n->param_type = NULL; }

  /* divers */
  op_n->frame_parbox = NULL;
}
 
/****************************************************************************************
 * FUNCTION  : oper_affi_ico ( Ico *ico );
 *
 * PURPOSE   : Afficher l'icone IPS_OPER
 *
 * CALLED IN : method_affi_ico
 *
 ***************************************************************************************/
void  oper_affi_ico (ico)
Ico  *ico;
{
  Oper *op = (Oper *) ico->obj;
  int    a1, a2, a3, n, x, y, c, l, i;

  stg_PrintCenter (ico->xb+ico->xm/2, ico->yb+2, op->mot_x, op->titre);

  if (ico->sel)
       { a1 = glo.c[4]; a2 = glo.c[1]; }
  else { a1 = glo.c[1]; a2 = glo.c[4]; }
  y = ico->yb+ico->ym-20;
  ico_affi_rect_3d (ico->xb+10, y, 12, 12, a1, a2);

  a3 = (op->led_busy == 1) ? 6 :
       (op->led_busy == 2) ? 12 :
       (op->attach_n) ? 9 : 10;

  XSetForeground (glo.dpy, glo.gc, coul_pic[a3]);
  XFillRectangle (glo.dpy, glo.xwin, glo.gc, ico->xb+11, y+1, 10, 10);
}

/****************************************************************************************
 * FUNCTION  : oper_prepare_noms ( Ico *ico );
 *
 * PURPOSE   : Calcule les coords des sous titres de l'IPS_OPER
 *
 * CALLED IN : oper_FramePcpMenuProc
 *
 ****************************************************************************************/
void  oper_prepare_noms (ico)
Ico  *ico;
{
    Oper *ob = (Oper *) ico->obj;
    char bla[500];
    int nb;

    if (ob->titre == NULL) ob->titre = stg_Cpy ("WARNING: Oper without name");

    /* Normalise espaces */
    stg_NormSpace (ob->titre, bla);
    pcp_free (ob->titre);
    ob->titre = stg_Cpy (bla);

    /* Repartit les mots */
    stg_RepartiMots (bla, &(ob->mot_x), &nb);

    /* calcul de ico->xm, ico->ym */
    ico->ym = 3 + nb*(glo.font_h+2) + 16+12;
    ico->xm = max (ob->mot_x * glo.font_l + 14, 16+36);
}


/****************************************************************************************
 * FUNCTION  : oper_def ( Oper *op, int mode, arg1, arg2, ... , NULL );
 *
 * PURPOSE   : Fixe les caracteristiques de l'oper op
 *
 * CALLED IN : call_oper
 *
 ****************************************************************************************/
void   oper_def (Oper *op, int mode, ...)
{
  va_list  ap;
  int      ordre, i, n, num, param_type[OPER_VECM], k;
  void    *param_adr[OPER_VECM];
  char    *lgd_images[OPER_VECM], *legende;

  /* init lecture */
  va_start (ap, mode);

  if (!(mode == IPS_MODE_CREAT || mode == IPS_MODE_USE))
    pcp_error ("oper_def #01", "mode not defined", NULL);

  /* init params et lgd_images */
  op->param_nb = op->nb_image_in = op->nb_image_out = 0;
  for (i = 1; i < OPER_VECM; i++)
  { param_type[i] = NULL; lgd_images[i] = NULL; }

  /* lecture arguments */
  while ((ordre = va_arg (ap, int)) != NULL)
  switch (ordre) /* #1 */
  {  
    case IPS_NAME : {
        char *pligne = va_arg (ap, char *);
        if (mode == IPS_MODE_CREAT)
          op->titre = stg_Cpy (pligne); 
      } break;

    case IPS_NAME2 : {
        char *pli1 = va_arg (ap, char *);
        char *pli2 = va_arg (ap, char *);
        char pligne[200];

        sprintf (pligne, "%s%s", pli1, pli2);
        if (mode == IPS_MODE_CREAT)
          op->titre = stg_Cpy (pligne); 
      } break;

    case IPS_IMAGE_IN  : 
    case IPS_IMAGE_OUT :
        num     = va_arg (ap, int);
        legende = va_arg (ap, char *);

        if (num < 1 || num > OPER_VECM-1)
        { pcp_error ("oper_def #02","image number is not correct", NULL); break; }
        if (lgd_images[num] != NULL)
        { pcp_error ("oper_def #03","image number is already used", NULL); break; }

        switch (ordre) /* #3 */
        {
          case IPS_IMAGE_IN : 
              if (op->nb_image_out > 0)
              { pcp_error ("oper_def #04","IMAGE_IN defined after IMAGE_OUT", NULL); num = 0; }
              if (num > op->nb_image_in) op->nb_image_in = num;
              break;
          case IPS_IMAGE_OUT : 
              if (num > op->nb_image_out) op->nb_image_out = num;
              break;
        } /* #3 */

        if (num == 0) break;
        lgd_images[num] = stg_Cpy (legende);
        break;

    case IPS_BORDER_ANY :
    case IPS_BORDER_POSIT :
    case IPS_BORDER_NOONE :
        op->val_cadre = ordre;
        break;
    case IPS_BORDER :
        op->val_cadre = va_arg (ap, int);
        break;
    case IPS_SAME_SIZE :
        op->same_size = va_arg (ap, int);
        break;

    case IPS_PAR_REAL    : 
    case IPS_PAR_CHK_BOX :
    case IPS_PAR_SLIDER  :
    case IPS_PAR_INT     : 
    case IPS_PAR_TEXT    : 
    case IPS_PAR_CH_ABR  :
    case IPS_PAR_CHOICE  : 
        num = va_arg (ap, int);

        if (num < 1 || num > OPER_VECM-1)
        { pcp_error ("oper_def #05","param number is not correct", NULL); num = 0; }
        if (param_type[num] != NULL)
        { pcp_error ("oper_def #06","param number is already used", NULL); num = 0; }
        param_type[num] = ordre;
        if (num > op->param_nb) op->param_nb = num;

        switch (ordre) /* #2 */
        {
          case IPS_PAR_REAL : {
              char		*legende	= va_arg (ap, char *);
              double		borne_inf	= atof (va_arg (ap, char *));
              double		borne_sup	= atof (va_arg (ap, char *));
	      double		val_defaut	= atof (va_arg (ap, char *));
              Oper_par_real	*para;

              if (num == 0) break;

              para = (Oper_par_real *) pcp_alloc (sizeof(Oper_par_real));
              (Oper_par_real *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->inf = borne_inf;
              para->sup = borne_sup;
              para->defaut = para->valeur = val_defaut;
#             ifdef DEBUG
                printf ("IPS_PAR_REAL num.%2d \"%s\" %f %f %f\n", 
                        num, legende, borne_inf, borne_sup, val_defaut);
#             endif
            } break;

          case IPS_PAR_SLIDER :
          case IPS_PAR_INT : {
              char		*legende	= va_arg (ap, char *);
              int		borne_inf	= va_arg (ap, int);
              int		borne_sup	= va_arg (ap, int);
              int		val_defaut	= va_arg (ap, int);
              Oper_par_int	*para;

              if (num == 0) break;

              para = (Oper_par_int *) pcp_alloc (sizeof(Oper_par_int));
              (Oper_par_int *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->inf = borne_inf;
              para->sup = borne_sup;
              para->defaut = para->valeur = val_defaut;
#             ifdef DEBUG
                printf ("IPS_PAR_INT  num.%2d \"%s\" %d %d %d\n", 
                        num, legende, borne_inf, borne_sup, val_defaut);
#             endif
            } break;
 
          case IPS_PAR_CHK_BOX : {
              char		*legende	= va_arg (ap, char *);
              int		val_defaut	= va_arg (ap, int);
              Oper_par_int	*para;

              if (num == 0) break;

              para = (Oper_par_int *) pcp_alloc (sizeof(Oper_par_int));
              (Oper_par_int *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->inf = 0;
              para->sup = 1;
              para->defaut = para->valeur = val_defaut;
#             ifdef DEBUG
                printf ("IPS_PAR_CHK_BOX  num.%2d \"%s\" %d\n", 
                        num, legende, val_defaut);
#             endif
            } break;

          case IPS_PAR_TEXT : {
              char		*legende	= va_arg (ap, char *);
              int		taille_max	= va_arg (ap, int);
              char		*val_defaut	= va_arg (ap, char *);
              Oper_par_text	*para;

              if (num == 0) break;

              para = (Oper_par_text *) pcp_alloc (sizeof(Oper_par_text));
              (Oper_par_text *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->defaut  = stg_Cpy (val_defaut);
              para->taille_max = taille_max;
              para->valeur = (char *) pcp_alloc (taille_max + 1);
              strcpy (para->valeur, val_defaut);
#             ifdef DEBUG
                printf ("IPS_PAR_TEXT num.%2d \"%s\" %d \"%s\"\n", 
                        num, legende, taille_max, val_defaut);
#             endif
            } break;
 
          case IPS_PAR_CH_ABR :
          case IPS_PAR_CHOICE : {
              char		*legende	= va_arg (ap, char *);
              int		val_defaut	= va_arg (ap, int);
              Oper_par_choice	*para;
              char              *nom, *choices[OPER_VECM];

              if (num == 0) break;

              para = (Oper_par_choice *) pcp_alloc (sizeof(Oper_par_choice));
              (Oper_par_choice *) param_adr[num] = para;

              para->legende = stg_Cpy (legende);
              para->defaut = para->valeur = val_defaut;

              para->nchoice = 0;
              while ((nom = va_arg (ap, char *)) != NULL)
                choices[para->nchoice++] = stg_Cpy (nom);
              para->choices = (char **) pcp_alloc (para->nchoice * sizeof (char *));
              for (k = 0; k < para->nchoice; k++) para->choices[k] = choices[k];
#             ifdef DEBUG
                printf ("IPS_PAR_CHOICE num.%2d \"%s\" %d %d\n", 
                        num, legende, val_defaut, para->nchoice);
#             endif
            } break;
 
        } /* switch (ordre) #2 */
        break;

    case IPS_IDF : {
        char *pligne = va_arg (ap, char *);
        if (mode == IPS_MODE_CREAT)
          op->idf = stg_Cpy (pligne); 
      } break;

    default : pcp_error ("oper_def #07", op->idf, "error in definition", NULL);
  } /* switch (ordre) #1 */

  /* fin lecture args */
  va_end (ap);

  /* enregistre params - attention, decalage dans numeros */
  if (op->param_nb > 0)
  {
    op->param_adr  = (void **) pcp_alloc (op->param_nb * sizeof(void *));
    op->param_type = (int *)   pcp_alloc (op->param_nb * sizeof(int));
    for (i = 0; i < op->param_nb; i++) 
    { op->param_adr[i] = param_adr[i+1];
      op->param_type[i]= param_type[i+1];
    }
  }

  /* enregistre lgd_images */
  if (op->nb_image_out > 0) op->nb_image_out -= op->nb_image_in;
  n = op->nb_image_in + op->nb_image_out;
  if (n > 0) op->lgd_images = (char **) pcp_alloc (n * sizeof(char *));
  for (i = 0; i < n; i++) op->lgd_images[i] = lgd_images[i+1];
}

/****************************************************************************************
 * FUNCTION  : void oper_set ( Oper *op, int ordre, ... );
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ****************************************************************************************/
void   oper_set (Oper *op, int ordre, ...)
{
  va_list  ap;
  Oper_parbox *p_b = NULL;

  va_start (ap, ordre);

  if (op->frame_parbox != NULL)
      p_b = (Oper_parbox *) xv_get (op->frame_parbox, WIN_CLIENT_DATA);

  switch (ordre)
  {  
    case IPS_PAR_REAL : {
        int   num = va_arg (ap, int);
	char *bla = va_arg (ap, char *);
        Oper_par_real *para = (Oper_par_real *) op->param_adr[num-1];
        para->valeur = atof (bla);
        if (p_b != NULL) xv_set (p_b->adr[num-1], PANEL_VALUE, bla, NULL); 
      } break;
    case IPS_PAR_CHK_BOX :
    case IPS_PAR_SLIDER :
    case IPS_PAR_INT : {
        int num = va_arg (ap, int);
        Oper_par_int *para = (Oper_par_int *) op->param_adr[num-1];
        para->valeur = va_arg (ap, int);
        if (p_b != NULL) xv_set (p_b->adr[num-1], PANEL_VALUE, para->valeur, NULL); 
      } break;
    case IPS_PAR_CH_ABR :
    case IPS_PAR_CHOICE : {
        int num = va_arg (ap, int);
        Oper_par_choice *para = (Oper_par_choice *) op->param_adr[num-1];
        para->valeur = va_arg (ap, int);
        if (p_b != NULL) xv_set (p_b->adr[num-1], PANEL_VALUE, para->valeur, NULL); 
      } break;
    case IPS_PAR_ACTIVE : {
        int num   = va_arg (ap, int);
        int actif = va_arg (ap, int);
        if (p_b != NULL) xv_set (p_b->adr[num-1], PANEL_INACTIVE, !actif, NULL); 
      } break;
    default : pcp_error ("oper_set #01","unknown argument", NULL);
  }

  va_end (ap);
}


/****************************************************************************************
 * FUNCTION  : void *oper_get ( Oper *op, int ordre, ... );
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ****************************************************************************************/
void   *oper_get (Oper *op, int ordre, ...)
{
  va_list  ap;
  void    *renvoi;

  va_start (ap, ordre);

  renvoi = NULL;

  switch (ordre)
  {  
    case IPS_NAME :
        renvoi = op->titre;
        break;
    case IPS_IMAGE_IN :
        (int) renvoi = op->nb_image_in;
        break;
    case IPS_IMAGE_OUT :
        (int) renvoi = op->nb_image_out;
        break;
    case IPS_BORDER :
        (int) renvoi = op->val_cadre;
        break;
    case IPS_SAME_SIZE :
        (int) renvoi = op->same_size;
        break;
    case IPS_PAR_REAL : {
        int num = va_arg (ap, int);
        Oper_par_real *para = (Oper_par_real *) op->param_adr[num-1];
        (double *) renvoi = &(para->valeur);
      } break;
    case IPS_PAR_CHK_BOX :
    case IPS_PAR_SLIDER :
    case IPS_PAR_INT : {
        int num = va_arg (ap, int);
        Oper_par_int *para = (Oper_par_int *) op->param_adr[num-1];
        (int) renvoi = para->valeur;
      } break;
    case IPS_PAR_TEXT : {
        int num = va_arg (ap, int);
        Oper_par_text *para = (Oper_par_text *) op->param_adr[num-1];
        (char *) renvoi = para->valeur;
      } break;
    case IPS_PAR_CH_ABR :
    case IPS_PAR_CHOICE : {
        int num = va_arg (ap, int);
        Oper_par_choice *para = (Oper_par_choice *) op->param_adr[num-1];
        (int) renvoi = para->valeur;
      } break;
/*  case IPS_PAR_MIN :
        break;
    case IPS_PAR_MAX :
        break;
    case IPS_PAR_DEFAULT :
        break;
*/  case IPS_PAR_NB :
        (int) renvoi = op->param_nb;
        break;
    case IPS_PAR_TYPE : {
        int num = va_arg (ap, int);
        (int) renvoi = op->param_type[num-1];
      } break;
    case IPS_IDF :
        renvoi = op->idf;
        break;
    default : pcp_error ("oper_get #01","unknown argument", NULL);
  }

  va_end (ap);
  return renvoi;
}

/****************************************************************************************
 * FUNCTION  : 
 *
 * PURPOSE   : retrouve icones et si possible appelle oper_exec
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void oper_check_run (ico)
Ico *ico;
{
    Ico		*v[OPER_VECM], *cur;
    Oper	*op  = (Oper *) ico->obj;
    int		n2   = op->nb_image_in + op->nb_image_out;
    int		n, i;
    char	bla[200];

    /*
     * init
    */
    for (i = 0; i < OPER_VECM; i++) v[i] = NULL;
    n = 0;

    /*
     * retrouver la selection, tri et tests
    */
    if (op->attach_n == 0) {
	cur = glo.l_ico_a;
	while (cur != NULL) {
	    if (cur->sel) { 
		if (IPS_IS_DATA(cur->nat_obj)) {
		    n++;
		    if (n > n2) {
			sprintf (bla, "%d in and %d out\n",
			    op->nb_image_in, op->nb_image_out);
			pcp_avertis_ico ("WARNING !",
			    "Number of datas needed by this oper :",bla, ico);
			return;
		    }
		    else ico_tri_selec (cur, v, 1, n);  /* tri par insertion */
		}

		else if (cur->index_creat != ico->index_creat) {
		    pcp_avertis_ico ("WARNING !","Only datas",
			"must be selected", cur);
		    return;
		}
	    }
	    cur = cur->pa;
	}
    }

    /*
     * retrouver les icones attaches
    */
    else {
	for (i = 0; i < op->attach_n; i++) {

	    cur = ico_get_from_index_creat (op->attach_v[i]);
	    if (cur != (Ico *) NULL)
		v[++n] = cur;
	    else {
		pcp_avertis_ico ("WARNING !","An attached icon",
		    "is lost", ico);
		return;
	    }
	}
    }

    /*
     * test si les icones sont proteges par LOCK_MODIF
    */
    for (i = 1; i <= n; i++) if (v[i]->lock & LOCK_MODIF) {
	pcp_avertis_ico ("WARNING !","That icon", "is locked", v[i]);
	return;
    }

    v[0] = ico;
    oper_exec (v, n);
}

/****************************************************************************************
 * FUNCTION  : 
 *
 * PURPOSE   : execution de l'oper
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
int oper_exec (v, n)
Ico **v;
int n;
{
    Ico		*ico = v[0];
    Oper	*op = (Oper *) ico->obj;
    Ico		*cur;
    int		n1, n2, i, j, c;
    char	bla[200];
    Call_oper_p	c_r;
    int		exec_poss = VRAI;
    int		test = VRAI;

    /*
     * Nombre d'images en entree et en sortie
    */
    n1 = op->nb_image_in;
    n2 = op->nb_image_in + op->nb_image_out;

    /***************************
     * TESTE le nombre d'images
    */
    if (n < n1) {
	sprintf (bla, "%d in and %d out\n", op->nb_image_in, op->nb_image_out);
	pcp_avertis_ico ("WARNING !","Number of datas needed by this oper :",bla, ico);
	return FAUX;
    }

    /*
     * Init Call_oper_p	c_r;
    */
    c_r.ico   = v;
    c_r.oper  = op;
    c_r.ima_n = n2;
    for (i = 0; i < OPER_VECM; i++) {
	c_r.image[i] = (Image *) NULL; 
	c_r.pix[i] = (Unt16 *) NULL;
	c_r.ima_name[i][0] = 0;
	c_r.ima_lut[i] = c_r.ima_x[i] = c_r.ima_y[i] = 0;
    }

    /*
     * Adresses des images
    */
    for (i = 1; i <= n; i++) c_r.image[i] = (Image *) v[i]->obj;

    /*
     * calcul de la dimension des images
    */
    if (n > 0) {
	c_r.ima_x[1] = c_r.image[1]->b.xsiz; 
	c_r.ima_y[1] = c_r.image[1]->b.ysiz;
    }
    else c_r.ima_x[1] = c_r.ima_y[1] = 256;

    for (i = 2; i <= c_r.ima_n; i++) {
	c_r.ima_x[i] = c_r.ima_x[1];
	c_r.ima_y[i] = c_r.ima_y[1];
    }

    if (!op->same_size) for (i = 2; i <= n; i++) {
	c_r.ima_x[i] = c_r.image[i]->b.xsiz;
	c_r.ima_y[i] = c_r.image[i]->b.ysiz;
    }

    /*
     * nom et lut des images out
    */
    for (i = n1+1; i <= n2; i++) {

	if (strcmp (op->lgd_images[i-1], ""))
	     strncpy (c_r.ima_name[i], op->lgd_images[i-1], IMAGE_CH1LEN);
	else sprintf (c_r.ima_name[i], "Out %d", i);

	c_r.ima_lut[i] = (n > 0)
	    ? glo_get_attr_lut (c_r.image[1]->coul_tab)
	    : IPS_COUL_16C;
    }

    /*
     * Appel de ACT_DIM
    */
    call_oper (IPS_ACT_DIM, op->idf, &c_r);

    /************************************************
     * TESTE si les images IN ont la bonne dimension
    */
    if (n > 0) {
	for (i = 1; i <= n1; i++)
	    test = (test && c_r.image[i]->b.xsiz == c_r.ima_x[i] 
			 && c_r.image[i]->b.ysiz == c_r.ima_y[i]);
	if (!test) {
	    pcp_avertis_ico ("WARNING !","Image size problem", "with this oper", ico);
	    return FAUX;
	}
    }

    /*
     * led_busy - ATTENTION : plus de return avant la fin, sinon maj led_busy.
    */
    op->led_busy = 2; /* run */
    ico_refresh_now (ico);

    /*
     * ajustement du cadre, taille et palette des images
    */
    if (n > 0) {

	/* taille cadre a partir des images existantes */
	switch (op->val_cadre) { 
	    case IPS_BORDER_ANY   : c = c_r.image[1]->b.cmin; break;
	    case IPS_BORDER_POSIT : c = c_r.image[1]->b.cmin; if (c == 0) c = 2; break;
	    case IPS_BORDER_NOONE : c = 0; break;
	    default : c = op->val_cadre;
	}

	/* images IN */
	for (i = 1; i <= n1; i++)
	    if (c_r.image[i]->b.cmin != c) {
		if (!image_chg_cad (v[i], c, 
			c_r.image[i]->b.xsiz, c_r.image[i]->b.ysiz)) {
		    op->led_busy = 0; /* sleep */
		    ico_refresh_now (ico);
		    return FAUX;
		}
		messa_Send (
		    MESSA_FROM_ICO, v[i],
		    MESSA_TO_ICO, v[i],
		    MESSA_RESCAN, VRAI,
		    NULL);
	    }

	/* images OUT existantes */
	for (i = n1+1; i <= n; i++)
	    if (!image_props_direct (v[i], 1, c,
		c_r.ima_x[i], c_r.ima_y[i], c_r.ima_lut[i], c_r.ima_name[i])) {
		    op->led_busy = 0; /* sleep */
		    ico_refresh_now (ico);
		    return FAUX;
	    }

    } /* n > 0 */

    /*
     * creation eventuelle des resultats, et positionnement des icones
    */
    if (n < n2) {

	int pos_x = ico->xb;
	int pos_y = ico->yb;
	int lar_x = ico->xm;
	int lar_y = ico->ym;
	int mul_x = 1;
	int mul_y = 0;

	/* pas d'image pre-existante */
	if (n == 0)
	    switch (op->val_cadre) {
		case IPS_BORDER_ANY   : c = 0; break;
		case IPS_BORDER_POSIT : c = 2; break;
		case IPS_BORDER_NOONE : c = 0; break;
		default : c = op->val_cadre;
	    }

	/* calcul mul_y, mul_x suivant zone */
	else {

	    int nr = n;  /* numero de l'icone de reference / Oper */
	    int acote = FAUX;

	    /* on teste dans cet ordre l'intersection de l'icone v[nr] avec :
	     * Oper, zone a gauche, zone a droite, zone en haut, zone en bas
	    */
	    if (ico_inter (v[nr], ico)) {
		/* les valeurs par defaut sont ok */
	    } else if (ico_inter_zone (v[nr], ico->xb-50, ico->yb, 50, ico->ym)) {
		pos_y = v[n]->yb; mul_x = 1;
	    } else if (ico_inter_zone (v[nr], ico->xb+ico->xm, ico->yb, 50, ico->ym)) {
		pos_y = v[n]->yb; mul_x = -1;
	    } else if (ico_inter_zone (v[nr], ico->xb, ico->yb-50, ico->xm, 50)) {
		pos_x = v[n]->xb; mul_x = 0; mul_y = 1;
	    } else if (ico_inter_zone (v[nr], ico->xb, ico->yb+ico->ym, ico->xm, 50)) {
		pos_x = v[n]->xb; mul_x = 0; mul_y = -1;
	    } else acote = VRAI;

	    if (acote || n1 == 0) {
		mul_x = 1; mul_y = 0;
		lar_x = v[n]->xm; lar_y = v[n]->ym;
		pos_x = v[n]->xb; pos_y = v[n]->yb;
	    }

#ifdef ancienancienancien
	    int nr = (n1 == 0) ? n : n1;  /* num de l'ico de ref / ico */

	    if (v[nr]->yb + v[nr]->ym < ico->yb) mul_y = 1;
	    else if (v[nr]->yb < ico->yb + ico->ym) { mul_y = 0; pos_y = v[n]->yb; }
	    else mul_y = -1;

	    if (v[nr]->xb + v[nr]->xm < ico->xb) mul_x = 1;
	    else if (v[nr]->xb < ico->xb + ico->xm) 
	    if (mul_y == 0) mul_x = 1;
	    else { mul_x = 0; pos_x = v[n]->xb; }
	    else mul_x = -1;

	    if (n1 == 0) {
		mul_x = -mul_x; mul_y = -mul_y;
		lar_x = v[n]->xm; lar_y = v[n]->ym;
		pos_x = v[n]->xb; pos_y = v[n]->yb;
	    }
#endif
	} /* else */

	for (i = n+1; i <= n2; i++) {

	    /* New Image */
	    v[i] = ico_creat (0, 0, IPS_IMAGE);
	    image_creat ((Image *) v[i]->obj, c, c_r.ima_x[i], c_r.ima_y[i], NULL);
	    c_r.image[i] = (Image *) v[i]->obj;

	    /* teste si tout a bien pu etre cree */
	    if (c_r.image[i]->tab == NULL) {
		ico_free(v[i]); v[i] = NULL; exec_poss = FAUX; break;
	    }

	    ico_tete_pz (v[i]);
	    strncpy (c_r.image[i]->ch1, c_r.ima_name[i], IMAGE_CH1LEN);
	    image_calc_hublot (v[i]);

	    /* table des couleurs */
	    c_r.image[i]->coul_tab = glo_get_ctab_lut (c_r.ima_lut[i]);

	    /* coordonnees */
	    if (mul_x < 0) lar_x = v[i]->xm;
	    if (mul_y < 0) lar_y = v[i]->ym;
	    pos_x += mul_x * (10 + lar_x);
	    pos_y += mul_y * (10 + lar_y);
	    lar_x = v[i]->xm; lar_y = v[i]->ym;
	    v[i]->xb = pos_x; v[i]->yb = pos_y;
	    ico_douane (v[i]);

	} /* for i */

    } /* n < n2 */

    /*
     * execution
    */
    if (exec_poss) {

	double td1, td2;
	char tdc[80];

	/* preparation de Call_oper_p c_r */
	for (i = 1; i <= n2; i++) {
	    c_r.pix[i] = c_r.image[i]->tab;
	}

	if (n2 > 0) {
	    c_r.cmin  = c_r.image[1]->b.cmin;   c_r.xytot = c_r.image[1]->b.xytot;
	    c_r.xsiz  = c_r.image[1]->b.xsiz;   c_r.ysiz  = c_r.image[1]->b.ysiz;
	    c_r.xmax  = c_r.image[1]->b.xmax;   c_r.ymax  = c_r.image[1]->b.ymax;
	    c_r.xtot  = c_r.image[1]->b.xtot;   c_r.ytot  = c_r.image[1]->b.ytot;
	}      

	/* run */
	pcp_footer ("Running ", op->titre, NULL);
	pcp_gettime (&td1);
	call_oper (IPS_ACT_EXE, op->idf, &c_r);
	pcp_gettime (&td2);
	sprintf (tdc, "Time = %.3f s", td2-td1);
	pcp_footer (tdc, NULL);

    } /* if exec_poss */

    /*
     * affichage des resultats
    */
    op->led_busy = 0; /* sleep */
    ico->raf = VRAI;
    for (i = n1 + 1; i <= n2; i++) if (v[i] != NULL) {
	v[i]->raf = VRAI; method_obsolete (v[i]);
	if (op->attach_n == 0) {
	    v[i]->sel = VRAI; ico_index_selec (v[i]);
	}
    }
    ico_inter_crible ();
    ico_affi_raf ();
    for (i = n1 + 1; i <= n; i++)  if (v[i] != NULL)
	messa_Send (
	    MESSA_FROM_ICO, v[i],
	    MESSA_TO_ICO, v[i],
	    MESSA_REFRESH, VRAI,
	    NULL);

    return exec_poss;
}


/****************************************************************************************
 * FUNCTION  : oper_parbox_creat (Ico *ico);
 *
 * PURPOSE   :
 *
 * CALLED IN : ico_menu_ico_proc
 *
 ****************************************************************************************/
void   oper_parbox_creat (ico)
Ico   *ico;
{
  Oper         *op = (Oper *) ico->obj;
  Oper_parbox  *p;
  Panel          panel;
  int            mx = (int) xv_get (glo.frame_principal, XV_X) + ico->xb;
  int            my = (int) xv_get (glo.frame_principal, XV_Y) + ico->yb;
  int            i, n, n1, n2, n3, ygap, m, k;
  char           bla1[256], bla2[256];
  Panel_item     pan_inout[OPER_VECM], pan_image[OPER_VECM];

  if (op->frame_parbox != NULL)
  { 
    /*  Oper_parpox *p existe deja ; on le recupere
     */
    p = (Oper_parbox *) xv_get (op->frame_parbox, WIN_CLIENT_DATA);
  }
  else
  { 
    /*  Oper_parpox *p n'existe pas ; on le cree
     */

    p = (Oper_parbox *) pcp_alloc (sizeof(Oper_parbox));

    p->ico = ico; p->oper = op;

    op->frame_parbox = (Frame) xv_create (glo.frame_principal, FRAME_CMD,
        FRAME_LABEL,		op->titre,
	FRAME_CMD_PUSHPIN_IN,	TRUE,
        WIN_CLIENT_DATA,	p,
        FRAME_DONE_PROC,	oper_parbox_kill,
        NULL);

    panel = (Panel) xv_get (op->frame_parbox, FRAME_CMD_PANEL);

    /*  Images IN et OUT
     */
    ygap = (int) xv_get (panel, PANEL_ITEM_Y_GAP);
    xv_set (panel, PANEL_ITEM_Y_GAP, 6, NULL);

    n1 = (op->nb_image_in  > 0) ? op->nb_image_in  : 1;
    n2 = (op->nb_image_out > 0) ? op->nb_image_out : 1;
    n = n1 + n2; n3 = 1;

    for (i = 0; i < n; i++)
    {
      if (i < n1)
           if (op->nb_image_in == 0)
                { sprintf (bla1, "IN image");
                  sprintf (bla2, "no one"); n3 = 0; }
           else { sprintf (bla1, "IN image %d", i+n3); 
                  sprintf (bla2, op->lgd_images[i]); }
      else if (op->nb_image_out == 0)
                { sprintf (bla1, "OUT image");
                  sprintf (bla2, "no one"); }
           else { sprintf (bla1, "OUT image %d",i+n3); 
                  sprintf (bla2, op->lgd_images[i+n3-1]); }

      xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, NULL);
      pan_inout[i] = (Panel_item) xv_create (panel, PANEL_MESSAGE,
		PANEL_LABEL_BOLD,	TRUE,
		PANEL_LABEL_STRING,	bla1, 
		NULL);
      if (i == 0) xv_set (pan_inout[i], XV_Y, 10, NULL);
      xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);
      pan_image[i] = (Panel_item) xv_create (panel, PANEL_MESSAGE, PANEL_LABEL_STRING, bla2, NULL);
   }

    /*  Allignement images IN et OUT
     */
    n1 = 0;
    for (i = 0; i < n; i++)
    { n2 = (int) xv_get (pan_inout[i], XV_WIDTH);
      if (n2 > n1) n1 = n2;
    }
    if (n > 0) n1 += xv_get (pan_inout[0], XV_X);
    for (i = 0; i < n; i++) 
    xv_set (pan_inout[i], XV_X, n1 - xv_get (pan_inout[i], XV_WIDTH), NULL);

    n1 = 0;
    for (i = 0; i < n; i++)
    { n2 = (int) xv_get (pan_image[i], XV_X);
      if (n2 > n1) n1 = n2;
    }
    for (i = 0; i < n; i++) xv_set (pan_image[i], XV_X, n1, NULL);

    xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, PANEL_ITEM_Y_GAP, ygap-2, NULL);

    /*  Parametres
     */
    if (op->param_nb > 0) 
         p->adr = (int *) pcp_alloc (op->param_nb * sizeof(int));
    else p->adr = NULL;

    for (i = 0; i < op->param_nb; i++)
    switch (op->param_type[i])
    {
      case IPS_PAR_INT : {
          Oper_par_int *para = (Oper_par_int *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_NUMERIC_TEXT,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_MIN_VALUE,	para->inf,
		PANEL_MAX_VALUE,	para->sup,
		NULL);
        } break;

      case IPS_PAR_SLIDER : {
          Oper_par_int *para = (Oper_par_int *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_SLIDER,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_MIN_VALUE,	para->inf,
		PANEL_MAX_VALUE,	para->sup,
		PANEL_SLIDER_WIDTH,	100,
		NULL);
        } break;

      case IPS_PAR_CHK_BOX : {
          Oper_par_int *para = (Oper_par_int *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_CHECK_BOX,
		PANEL_LABEL_STRING,	para->legende,
		/*PANEL_CHOICE_STRINGS,	para->legende, NULL,*/
		PANEL_NOTIFY_PROC,	oper_parbox_par_choice_proc,
		XV_KEY_DATA,		101, p,
		XV_KEY_DATA,		102, i+1,
		NULL);
        } break;

      case IPS_PAR_REAL : {
          Oper_par_real *para = (Oper_par_real *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_TEXT,
		PANEL_LABEL_STRING,		para->legende,
		PANEL_VALUE_DISPLAY_LENGTH,	15,
		PANEL_VALUE_STORED_LENGTH,	30,
		NULL);
        } break;

      case IPS_PAR_TEXT : {
          Oper_par_text *para = (Oper_par_text *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_TEXT,
		PANEL_LABEL_STRING,		para->legende,
		PANEL_VALUE_DISPLAY_LENGTH,	25,
		PANEL_VALUE_STORED_LENGTH,	para->taille_max,
		NULL);
        } break;

      case IPS_PAR_CHOICE : {
          Oper_par_choice *para = (Oper_par_choice *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_CHOICE,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_LAYOUT,		PANEL_HORIZONTAL,
		PANEL_NOTIFY_PROC,	oper_parbox_par_choice_proc,
		XV_KEY_DATA,		101, p,
		XV_KEY_DATA,		102, i+1,
		NULL);
          for (k = 0; k < para->nchoice; k++)
          xv_set (p->adr[i], PANEL_CHOICE_STRING, k, para->choices[k], NULL);
        } break;

       case IPS_PAR_CH_ABR : {
          Oper_par_choice *para = (Oper_par_choice *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_CHOICE_STACK,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_LAYOUT,		PANEL_HORIZONTAL,
		PANEL_NOTIFY_PROC,	oper_parbox_par_choice_proc,
		XV_KEY_DATA,		101, p,
		XV_KEY_DATA,		102, i+1,
		NULL);
          for (k = 0; k < para->nchoice; k++)
          xv_set (p->adr[i], PANEL_CHOICE_STRING, k, para->choices[k], NULL);
        } break;

      default : p->adr[i] = NULL;
    }

    /*  Allignement parametres
     *
    */
    n1 = 0;
    for (i = 0; i < op->param_nb; i++)
    { n2 = (int) xv_get ((Panel_item) p->adr[i], PANEL_VALUE_X);
      if (n2 > n1) n1 = n2;
    }
    for (i = 0; i < op->param_nb; i++)
    xv_set ((Panel_item) p->adr[i], PANEL_VALUE_X, n1, NULL);

    /*  Creation des boutons
     */

    (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,	"Help",
	PANEL_NOTIFY_PROC,      oper_parbox_butt_panel_proc,
	PANEL_CLIENT_DATA,	p,
	NULL);

    xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

    if (op->param_nb > 0)
    {
      Menu menu;

      (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,	"Apply & Run",
	PANEL_NOTIFY_PROC,       oper_parbox_butt_panel_proc,
	PANEL_CLIENT_DATA,	p,
	NULL);

      menu = (Menu) xv_create (NULL, MENU,
	MENU_STRINGS,		"Apply", "Cancel", "Default", NULL,
	MENU_NOTIFY_PROC,	oper_parbox_butt_menu_proc,
	MENU_CLIENT_DATA,	p,
	NULL);

      (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,    "Apply",
	PANEL_ITEM_MENU,       menu,
	NULL);
    }
    else (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,	"Run",
	PANEL_NOTIFY_PROC,      oper_parbox_butt_panel_proc,
	PANEL_CLIENT_DATA,	p,
	NULL);


    window_fit (panel);
    window_fit (op->frame_parbox);
  }

  /*  Affectation des champs PANEL_VALUE des parametres
   */
  oper_parbox_butt_direct (p, 3);  /* 3 = bouton Cancel */

  /*  Affichage de parbox
   */
  xv_set (op->frame_parbox, XV_X, mx, XV_Y, my, XV_SHOW, TRUE, NULL);
}


/****************************************************************************************
 * FUNCTION  :
 *
 * PURPOSE   :
 *
 * CALLED IN : 
 *
 ****************************************************************************************/
int         oper_parbox_butt_panel_proc (item, event)
Panel_item  item;
Event      *event;

{
  char          *choice = (char*)         xv_get (item, PANEL_LABEL_STRING);
  Oper_parbox  *p      = (Oper_parbox*) xv_get (item, PANEL_CLIENT_DATA);
  int            value;

  if      (!strcmp(choice, "Run"         )) value = 4;
  else if (!strcmp(choice, "Apply & Run" )) value = 5;
  else if (!strcmp(choice, "Help"        )) value = 6;
  else value = 4;

# ifdef DEBUG
  printf ("oper_parbox_butt_panel_proc %s %d\n", choice, value);
# endif

  oper_parbox_butt_direct (p, value);

  return XV_OK;
}

/****************************************************************************************
 * FUNCTION  :
 *
 * PURPOSE   :
 *
 * CALLED IN : 
 *
 ****************************************************************************************/
void        oper_parbox_butt_menu_proc (menu, menu_item)
Menu menu;
Menu_item menu_item;

{
  char          *choice = (char*)         xv_get (menu_item, MENU_STRING);
  Oper_parbox  *p      = (Oper_parbox*) xv_get (menu, MENU_CLIENT_DATA);
  int            value;

  if      (!strcmp(choice, "Apply"       )) value = 1;
  else if (!strcmp(choice, "Default"     )) value = 2;
  else if (!strcmp(choice, "Cancel"      )) value = 3;
  else value = 4;

# ifdef DEBUG
  printf ("oper_parbox_butt_menu_proc %s %d\n", choice, value);
# endif

  oper_parbox_butt_direct (p, value);
}

/****************************************************************************************
 * FUNCTION  :
 *
 * PURPOSE   :
 *
 * CALLED IN : 
 *
 ****************************************************************************************/
void           oper_parbox_butt_direct (p, value)
Oper_parbox  *p;
int            value;
{
  Oper  *op = p->oper;
  int     i;
  char    bla1[256];
  int     lancer = FAUX;

  switch(value)
  {
    case 1 : /* Apply */
        break;
    case 2 : /* Default */
        break;
    case 3 : /* Cancel */
        break;
    case 4 : /* Run */
	lancer = VRAI;
        break; 
    case 5 : /* Apply & Run */
	value = 1; lancer = VRAI;
        break; 
    case 6 : /* Help */
	aide_affi_oper (op->idf);
        return; 
  }

  for (i = 0; i < op->param_nb; i++)
  switch (op->param_type[i])
  {
    case IPS_PAR_CHK_BOX :
    case IPS_PAR_SLIDER :
    case IPS_PAR_INT : {
        Oper_par_int *para = (Oper_par_int *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->defaut, NULL);
              if (op->param_type[i] == IPS_PAR_CHK_BOX)
                oper_parbox_par_choice_proc (p->adr[i], para->defaut, (Event *) NULL);
              /* no break; */
          case 1 : /* Apply */
              para->valeur = (int) xv_get (p->adr[i], PANEL_VALUE);
              break;
          case 3 : /* Cancel */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->valeur, NULL);
              if (op->param_type[i] == IPS_PAR_CHK_BOX)
                oper_parbox_par_choice_proc (p->adr[i], para->valeur, (Event *) NULL);
              break;
        }
      } break;

    case IPS_PAR_REAL : {
        Oper_par_real *para = (Oper_par_real *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              sprintf (bla1, "%f", para->defaut);
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, bla1, NULL);
              /* no break; */
          case 1 : /* Apply */
              para->valeur = atof ((char*) xv_get (p->adr[i], PANEL_VALUE));
              if (para->valeur > para->sup) 
                   para->valeur = para->sup;
              else if (para->valeur < para->inf) 
                   para->valeur = para->inf;
              else break;
          case 3 : /* Cancel */
              sprintf (bla1, "%f", para->valeur);
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, bla1, NULL);
              break;
        }
      } break;

    case IPS_PAR_TEXT : {
        Oper_par_text *para = (Oper_par_text *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->defaut, NULL);
              /* no break; */
          case 1 : /* Apply */
              strcpy (para->valeur, (char*) xv_get (p->adr[i], PANEL_VALUE));
              break;
          case 3 : /* Cancel */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->valeur, NULL);
              break;
        }
      } break;

    case IPS_PAR_CH_ABR :
    case IPS_PAR_CHOICE : {
        Oper_par_choice *para = (Oper_par_choice *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->defaut, NULL);
              oper_parbox_par_choice_proc (p->adr[i], para->defaut, (Event *) NULL);
              /* no break; */
          case 1 : /* Apply */
              para->valeur = (int) xv_get (p->adr[i], PANEL_VALUE);
              break;
          case 3 : /* Cancel */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->valeur, NULL);
              oper_parbox_par_choice_proc (p->adr[i], para->valeur, (Event *) NULL);
              break;
        }
      } break;

  }

  if (lancer) oper_check_run (p->ico);
}

/****************************************************************************************
 * FUNCTION  : oper_parbox_par_choice_proc ()
 *
 * PURPOSE   :
 *
 ****************************************************************************************/
void        oper_parbox_par_choice_proc (item, value, event)
Panel_item  item;
int         value;
Event      *event;
{
  Oper_parbox  *p  = (Oper_parbox *) xv_get (item, XV_KEY_DATA, 101);
  int            n  = (int)            xv_get (item, XV_KEY_DATA, 102);
  Oper         *op = p->oper;
  Call_oper_p    c_r;

# ifdef DEBUG
    printf ("oper_parbox_par_choice_proc  param %d  value %d\n", n, value);
# endif

  c_r.oper    = op;
  c_r.chce_num = n;
  c_r.chce_val = value;

  call_oper (IPS_ACT_CHOICE, op->idf, &c_r);
}

/****************************************************************************************
 * FUNCTION  : oper_parbox_kill (Frame frame);
 *
 * PURPOSE   : FRAME_DONE_PROC
 *
 ****************************************************************************************/
void  oper_parbox_kill (frame)
Frame frame;
{
  Oper_parbox *p = (Oper_parbox *) xv_get (frame, WIN_CLIENT_DATA);

  if (xv_destroy_safe(frame) != XV_OK) pcp_error ("oper_parbox_kill #01", NULL);

  p->oper->frame_parbox = NULL;
  if (p->adr != NULL) pcp_free (p->adr);
  pcp_free(p);
}

/****************************************************************************************
 * FUNCTION  : oper_attach (Ico *ico);
 *
 * PURPOSE   : attache une selection a l'oper
 *
 ****************************************************************************************/
int  oper_attach (ico)
Ico  *ico;
{
  Oper *op = (Oper *) ico->obj;
  Ico *p[OPER_VECM], *cur;
  int n, i;
  int n1 = op->nb_image_in;
  int n2 = op->nb_image_in + op->nb_image_out;
  int test = VRAI;

  if (op->attach_n > 0 || op->attach_v != NULL)
      pcp_error ("oper_attach #01", NULL);

  /* memorisation des ico selectes, tri et tests */
  n = 0;
  cur = glo.l_ico_a;
  while (cur != NULL) {
      if (cur->sel) 
          if (IPS_IS_DATA(cur->nat_obj)) {
              ico_tri_selec (cur, p, 0, n++);  /* tri par insertion */
              if (n > n2) break;
          }
          else if (cur->index_creat != ico->index_creat) {
	      pcp_avertis_ico ("WARNING !","Only datas",
	          "must be attached", cur);
              test = FAUX;
          }
      cur = cur->pa;
  }

  if (!test) return FAUX;

  if (n < n1 || n > n2) {
    char bla[100];
    sprintf (bla, "%d in and %d out\n", op->nb_image_in, op->nb_image_out);
    pcp_avertis_ico ("WARNING !","Number of datas needed by this oper :",bla, ico);
    return FAUX;
  }

  if (n > 0) {
    op->attach_v = (int *) pcp_alloc (n * sizeof(int));
    if (op->attach_v == NULL) return;
    for (i = 0; i < n; i++) op->attach_v[i] = p[i]->index_creat;
  }
  op->attach_n = n;
  ico_init_raf ();
  ico->sel = FAUX;
  ico->raf = VRAI;
  for (i = 0; i < n; i++) { /* on deselectionne ce qui est attache */
	p[i]->sel = FAUX; p[i]->raf = VRAI;
  }
  ico_inter_crible ();
  ico_affi_raf ();

  return VRAI;
}

/****************************************************************************************
 * FUNCTION  : oper_detach (Ico *ico);
 *
 * PURPOSE   : detache une selection a l'oper
 *
 ****************************************************************************************/
void  oper_detach (ico)
Ico  *ico;
{
  Oper *op = (Oper *) ico->obj;
  int i;
  Ico *cur;

  if (op->attach_n == 0 || op->attach_v == NULL)
      pcp_error ("oper_detach #01", NULL);

  else { 
    /* on deselectionne tout le monde et init ->raf */
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) {
	cur->raf = cur->sel; cur->sel = FAUX;
    }
    /* on reselectionne ce qui etait attache */
    for (i = 0; i < op->attach_n; i++) {
	cur = ico_get_from_index_creat (op->attach_v[i]);
	if (cur != (Ico *) NULL) {
	    cur->sel = cur->raf = VRAI;
	    ico_index_selec (cur);
	}
    }
  }

  pcp_free (op->attach_v);
  op->attach_v = NULL;
  op->attach_n = 0;
  ico->sel = FAUX;
  ico->raf = VRAI;
  ico_inter_crible ();
  ico_affi_raf ();
}

/***************************************************************************************
 * Creation du bouton et du menu "Oper" du frame_principal
 *
 ***************************************************************************************/
Panel_item oper_FramePcpButtCreat (panel)
Panel panel;
{
    menu_oper = res_lecture_mnu ( glo.chemin, "ips.oper_menu", 
	glo.frame_principal, oper_FramePcpMenuProc);

    if (menu_oper == (Menu) NULL)
	 return (Panel_item) NULL;
    else return (Panel_item) xv_create (panel, PANEL_BUTTON,   
	    PANEL_LABEL_STRING,	"Oper",
	    PANEL_ITEM_MENU,	menu_oper,
	    NULL);
}

/****************************************************************************************
 * Activation du choix dans le menu "Oper" du frame_principal
 *
 * - lecture du nom du bouton et de l'idf de l'oper selectionne
 * - creation complete de l'IPS_OPER correspondant
 *
 ****************************************************************************************/
void oper_FramePcpMenuProc (menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    char  *menu_str = (char *) xv_get (menu_item, MENU_STRING);
    char  *menu_idf = (char *) xv_get (menu_item, MENU_CLIENT_DATA);
    Ico   *clic;
    Call_oper_p  c_r;

#   ifdef DEBUG
    printf ("oper_FramePcpMenuProc %s \"%s\"\n", menu_idf, menu_str);
#   endif

    if (menu_idf == (char *) NULL) return;

    switch (aide_get_action ()) {

	case AIDE_IS_CREATE_ICON :
	    clic = ico_creat (glo.sx, glo.sy, IPS_OPER); 
	    glo.sx += 10; glo.sy += 16;
	    ico_tete_pz (clic);
	    c_r.oper = (Oper *) clic->obj;
	    oper_init (c_r.oper, menu_idf);
	    c_r.mode = IPS_MODE_CREAT;
	    call_oper (IPS_ACT_DEF, menu_idf, &c_r);
	    oper_prepare_noms (clic);
	    ico_douane (clic);
	    ico_refresh_now (clic);
	    break;

	case AIDE_IS_DISPLAY_HELP :
	    aide_affi_oper (menu_idf);
	    break;

	default :
	    aide_nothing_to_do ();
    }
}

/****************************************************************************************
 * FUNCTION  : oper_CloseAllWin (Ico *ico);
 *
 * CALLED IN : method_free
 *
 ****************************************************************************************/
void oper_CloseAllWin (ico)
Ico *ico;
{
    Oper *op = (Oper *) ico->obj;
  
    /* Boite de parametres */
    if (op->frame_parbox != NULL)
	oper_parbox_kill (op->frame_parbox);
}
