/***************************************************************************************/
/* obj_tool.c                                                 I.P.S  v 3.5  15/06/1995 */
/*                                                                                     */
/* gestion de l'objet tool (ico->nat_obj = ICO_TOOL)     Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"

#define DEBUG

#define LG_LIGNE	255+1


void tool_init ();
void tool_free ();
void tool_duplic ();
void tool_affi_ico ();
void tool_prepare_noms ();
int tool_create_tool_from_exec ();
void tool_CloseAllWin ();


/****************************************************************************************
 * FUNCTION  : tool_init ( Tool *ob, char *idf, char *nomf );
 *
 * PURPOSE   : Initialisation du tool ob
 *
 * CALLED IN : tool_create_tool_from_exec
 *
 ***************************************************************************************/
void   tool_init (ob, idf, nomf)
Tool  *ob;
char  *idf, *nomf;
{ 
    ob->idf		= stg_Cpy (idf);	/* idf de la commande lue dans menu	*/
    ob->nomf		= stg_Cpy (nomf);	/* nom du fichier executable		*/
    ob->led_attach	= 0;			/* booleen : attachement		*/
    ob->led_pourcent	= 0;			/* 0% = sleep, 1..32 busy		*/
    ob->mot_x		= 10;			/* largeur max des sous titres		*/
    ob->titre		= NULL;			/* titre du tool			*/
    ob->source		= NULL;			/* stockage de la zone source		*/
    ob->variab_len	= 0;			/* longueur de la zone variab		*/
    ob->variab		= NULL;			/* stockage de la zone variab		*/
    ob->frame_parbox	= 0;			/* Frame de la boite de dialogue 	*/
}
 

/****************************************************************************************
 * FUNCTION  : tool_free ( Tool *ob );
 *
 * PURPOSE   : Suppression du tool ob
 *
 * CALLED IN : method_free
 *
 ***************************************************************************************/
void   tool_free (ob)
Tool *ob;
{

    /* attachement */

    /* strings */
    pcp_free(ob->idf);
    pcp_free(ob->nomf);
    pcp_free(ob->titre);
    pcp_free(ob->source);
    pcp_free(ob->variab);
}


/****************************************************************************************
 * FUNCTION  : tool_duplic ( Ico *ico_new, *ico_old );
 *
 * PURPOSE   : Duplication de l'icone IPS_TOOL : ico_n := ico_o
 *
 * CALLED IN : method_duplic
 *
 ***************************************************************************************/
void  tool_duplic (ico_n, ico_o)
Ico  *ico_n, *ico_o;
{ 
    Tool *ob_o = (Tool *) ico_o->obj;
    Tool *ob_n = (Tool *) ico_n->obj;

    memcpy (ob_n, ob_o, sizeof(Tool));

    /* attachement */
    ob_n->led_attach = 0;
    ob_n->led_pourcent = 0;

    /* idf et nomf */
    ob_n->idf = stg_Cpy (ob_o->idf);
    ob_n->nomf = stg_Cpy (ob_o->nomf);

    /* titre du tool */
    ob_n->titre = stg_Cpy (ob_o->titre);

    /* zone source */
    ob_n->source = stg_Cpy (ob_o->source);

    /* zone variab */
    ob_n->variab_len = 0;
    ob_n->variab = NULL;

    /* divers */
    ob_n->frame_parbox = NULL;
}
 

/****************************************************************************************
 * FUNCTION  : tool_affi_ico ( Ico *ico );
 *
 * PURPOSE   : Afficher l'icone IPS_TOOL
 *
 * CALLED IN : method_affi_ico
 *
 ***************************************************************************************/
void  tool_affi_ico (ico)
Ico  *ico;
{
    Tool *ob = (Tool *) ico->obj;
    int    a1, a2, a3, n, x1, y1;

    XSetForeground (glo.dpy, glo.gc, glo.c[6]);

    stg_PrintCenter (ico->xb+ico->xm/2, ico->yb+2, ob->mot_x, ob->titre);

    if (ico->sel)
         { a1 = glo.c[4]; a2 = glo.c[1]; }
    else { a1 = glo.c[1]; a2 = glo.c[4]; }
    if (ob->led_attach)
	 a3 = 9;
    else a3 = 10;

    x1 = ico->xb+(ico->xm-42)/2;
    y1 = ico->yb+ico->ym-14;
    ico_affi_rect_3d (x1, y1, 42, 6, a1, a2);

    XSetForeground (glo.dpy, glo.gc, coul_pic[a3]);
    XFillRectangle (glo.dpy, glo.xwin, glo.gc, 
	x1+1, y1+1, 40, 4);
    if (ob->led_pourcent == 0) return;
    XSetForeground (glo.dpy, glo.gc, coul_pic[12]);
    XFillRectangle (glo.dpy, glo.xwin, glo.gc, 
	x1+1+32-ob->led_pourcent, y1+1, ob->led_pourcent+8, 4);
}


/****************************************************************************************
 * FUNCTION  : tool_prepare_noms ( Ico *ico, char *titre );
 *
 * PURPOSE   : Calcule les longueurs des sous titres de l'IPS_TOOL
 *
 * CALLED IN : tool_create_tool_from_exec
 *
 ****************************************************************************************/
void  tool_prepare_noms (ico, titre)
Ico  *ico;
char *titre;
{
    Tool *ob = (Tool *) ico->obj;
    char bla[500];
    int nb;

    if (titre == NULL || titre[0] == 0) 
	 stg_NormSpace ("WARNING: Tool without name", bla);
    else stg_NormSpace (titre, bla);
    pcp_free (ob->titre);
    ob->titre = stg_Cpy (bla);

    /* Repartit les mots */
    stg_RepartiMots (bla, &(ob->mot_x), &nb);

    /* calcul de ico->xm, ico->ym */
    ico->ym = 3 + nb*(glo.font_h+2) + 16+6;
    ico->xm = max (ob->mot_x * glo.font_l + 14, 16+42);
}


/****************************************************************************************
 * Creation complete d'un IPS_TOOL
 *
 * RETURN 0 on success
 ****************************************************************************************/
int tool_create_tool_from_exec  (idf, nom1)
char *idf, *nom1;
{
    char nom2[LG_LIGNE], ligne[LG_LIGNE], bla[LG_LIGNE], *source;
    FILE *f1;
    int fd, debut, fin;
    struct stat bufstat;
    Ico *clic;
    Tool *ob;

    /*
     * Test ouverture fichier
    */
    sprintf (nom2, "%s/.ips_tmp_tool", glo.home_dir);
    if ((f1 = fopen (nom2, "w")) == NULL) return -1;
    fclose (f1);

    /*
     * Appel du tool
    */
    sprintf (ligne, "%s -def >>  %s", nom1, nom2);
    if (system (ligne) != 0) return -2;
    
    /*
     * Stockage
    */
    if ((fd = open (nom2, 0)) == -1) return -3;
    if (fstat (fd, &bufstat) == -1) return -4;
    if ((source = (char *) pcp_alloc (bufstat.st_size+1)) == NULL) return -5;
    if (read (fd, source, bufstat.st_size) != bufstat.st_size) return -6;
    source[bufstat.st_size] = 0;
    close (fd);

#   ifdef DEBUG
    printf ("VOICI LE SOURCE\n%s\nFIN DU SOURCE\n", source);
#   endif
    if (itml_CtrlBasNiveau (source) < 0) { pcp_free (source); return -1; }

    /*
     * New tool
    */
    clic = ico_creat (glo.sx, glo.sy, IPS_TOOL); 
    glo.sx += 10; glo.sy += 16;
    ico_tete_pz (clic);
    ob = (Tool *) clic->obj;
    tool_init (ob, idf, nom1);
    ob->source = source;
    ob->variab = itml_CreateVariabFromBox (ob->source);
    itml_GetVariab (ob->variab, "VAL", "TITLE", bla);
    tool_prepare_noms (clic, bla);
    ico_douane (clic);
    ico_refresh_now (clic);

    return 0;
}


/****************************************************************************************
 * FUNCTION  : tool_CloseAllWin (Ico *ico);
 *
 * CALLED IN : method_free
 *
 ****************************************************************************************/
void tool_CloseAllWin (ico)
Ico *ico;
{
    Tool *ob = (Tool *) ico->obj;
  
    /* Boite de parametres */
    if (ob->frame_parbox != NULL)
	/* tool_parbox_kill (ob->frame_parbox) */ ;
}
