/***************************************************************************************/
/* obj_vor2d.c                                                I.P.S  v 3.1   31/1/1994 */
/*                                                                                     */
/* gestion de l'objet vor2d  (ico->nat_obj = ICO_VOR2D)  Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/


#include "inclus.h"

/*
 * Decoration
*/
static XPoint decoS[] =	{ {47,35}, {32,48}, {18,37}, {18,22}, {44,11}, {47,35},
			  {63,40}, {32,64}, { 0,43}, { 4, 8}, {49, 0}, {63,40},
			  {60,24}, {52,52}, {12,52}, { 4,28}, {24, 8}, {60,24},
			  {32,28} };
#define decoN 19

/*
 * Gestion de l'infini
*/
#define INFIP	1.e+05
#define INFIS	5000.


/****************************************************************************************
 * FUNCTION  : vor2d_creat (Vor2d *ob, int nb_max_point, int nb_max_sommet)
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void vor2d_creat (ob, nb_max_point, nb_max_sommet)
Vor2d *ob;
int nb_max_point, nb_max_sommet;
{
    int i, j;

    /* chaines de caracteres */
    ob->ch1[0] = ob->ch2[0] = ob->ch3[0] = ob->ch4[0] = 0; 
    ob->s_ch1 = ob->s_ch2 = ob->s_ch3 = ob->s_ch4 = ob->x_ch4 = 0;

    vor2d_creat_point (ob, nb_max_point);
    vor2d_creat_sommet(ob, nb_max_sommet);
}
 
/****************************************************************************************
 * FUNCTION  : vor2d_creat_point (Vor2d *ob, int nb_max_point)
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void vor2d_creat_point (ob, nb_max_point)
Vor2d *ob;
int nb_max_point;
{
    double rac3 = (double) sqrt ((double) 3.);
    int i;

    /* structure Delaunay */
    ob->nb_point	= 0;
    ob->tab_point_base	= (Vor2d_point *)
	pcp_alloc ((nb_max_point +3) * sizeof (Vor2d_point));

    if (ob->tab_point_base == NULL) {
	ob->nb_max_point = 0;
	ob->tab_point    = NULL;
	return;
    }
    ob->nb_max_point	= nb_max_point;
    ob->tab_point	= ob->tab_point_base  + 3;

    /* Gestion de l'infini */
    ob->tab_point[-1].point[0] =  0.;
    ob->tab_point[-1].point[1] =  INFIP;
    ob->tab_point[-2].point[0] =  INFIP*rac3/2.;
    ob->tab_point[-2].point[1] = -INFIP/2.;
    ob->tab_point[-3].point[0] = -INFIP*rac3/2.;
    ob->tab_point[-3].point[1] = -INFIP/2.; 
    ob->tab_point[-1].num_s = 
    ob->tab_point[-2].num_s =
    ob->tab_point[-3].num_s = -4; 

    /* Les client_data */
    for (i = -3; i < nb_max_point ; i++) ob->tab_point [i].client_data = 0;
}

/****************************************************************************************
 * FUNCTION  : vor2d_creat_sommet (Vor2d *ob, int nb_max_sommet)
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void vor2d_creat_sommet (ob, nb_max_sommet)
Vor2d *ob;
int nb_max_sommet;
{
    double rac3 = (double) sqrt ((double) 3.);
    int i, j;

    /* structure Voronoi */
    ob->nb_sommet	= 0;
    ob->tab_sommet_base	= (Vor2d_sommet *)
	pcp_alloc ((nb_max_sommet+4) * sizeof (Vor2d_sommet));

    if (ob->tab_sommet_base == NULL) {
	ob->nb_max_sommet= 0;
	ob->tab_sommet   = NULL;
	return;
    }
    ob->nb_max_sommet	= nb_max_sommet;
    ob->tab_sommet	= ob->tab_sommet_base + 4;

    /* Gestion de l'infini */
    ob->tab_sommet[-1].point[0] =  0.;
    ob->tab_sommet[-1].point[1] = -INFIS * INFIP;
    ob->tab_sommet[-2].point[0] = -INFIS * INFIP*rac3/2.;
    ob->tab_sommet[-2].point[1] =  INFIS * INFIP/2.;
    ob->tab_sommet[-3].point[0] =  INFIS * INFIP*rac3/2.;
    ob->tab_sommet[-3].point[1] =  INFIS * INFIP/2.; 
    ob->tab_sommet[-4].point[0] = 0.;
    ob->tab_sommet[-4].point[1] = 0.;
    for (i = -3; i < 0; i++) for (j = 0; j < 3; j++)
    	if (i == -(j+1)) {
	     ob->tab_sommet[i].num_sv[j] = -4;
	     ob->tab_sommet[i].num_gc[j] = -5;  /* pour NULL */
	} else {
	     ob->tab_sommet[i].num_sv[j] = -5;  /* pour NULL */
	     ob->tab_sommet[i].num_gc[j] = -(j+1);
	}
    for (j = 0; j < 3; j++)
	ob->tab_sommet[-4].num_sv[j] =
	ob->tab_sommet[-4].num_gc[j] = -(j+1);

    /* Les client_data */
    for (i = -4; i < nb_max_sommet; i++) ob->tab_sommet[i].client_data = 0;
}


/****************************************************************************************
 * FUNCTION  : vor2d_adapt_tab_point (Vor2d *ob, int nb_max_point)
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void vor2d_adapt_tab_point (ob, nb_max_point)
Vor2d *ob;
int nb_max_point;
{
    if (nb_max_point != ob->nb_max_point && nb_max_point >= ob->nb_point) {

	Vor2d_point *v = (Vor2d_point *) realloc ( ob->tab_point_base,
	    (nb_max_point +3) * sizeof (Vor2d_point) );
	if (v == NULL)
	    pcp_error ("vor2d_adapt_tab_point", "Realloc error", NULL);
	else {
	    ob->tab_point_base = v;
	    ob->tab_point      = ob->tab_point_base + 3;
	    ob->nb_max_point   = nb_max_point;
	}
    }
}

 /****************************************************************************************
 * FUNCTION  : vor2d_adapt_tab_sommet (Vor2d *ob, int nb_max_sommet)
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void vor2d_adapt_tab_sommet (ob, nb_max_sommet)
Vor2d *ob;
int nb_max_sommet;
{
    if (nb_max_sommet != ob->nb_max_sommet && nb_max_sommet >= ob->nb_sommet) {

	Vor2d_sommet *v = (Vor2d_sommet *) realloc ( ob->tab_sommet_base,
	    (nb_max_sommet +4) * sizeof (Vor2d_sommet) );
	if (v == NULL)
	    pcp_error ("vor2d_adapt_tab_sommet", "Realloc error", NULL);
	else {
	    ob->tab_sommet_base = v;
	    ob->tab_sommet      = ob->tab_sommet_base + 4;
	    ob->nb_max_sommet   = nb_max_sommet;
	}
    }
}
 
/****************************************************************************************
 * FUNCTION  : vor2d_free (Vor2d *ob)
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void   vor2d_free (ob)
Vor2d *ob;
{
    pcp_free (ob->tab_point_base);
    pcp_free (ob->tab_sommet_base);
}
 
/****************************************************************************************
 * FUNCTION  : vor2d_duplic ( Ico *ico_new, *ico_old );
 *
 * PURPOSE   : Duplication de l'icone IPS_VOR2D : ico_n := ico_o
 *
 * CALLED IN : method_duplic
 *
 ***************************************************************************************/
void  vor2d_duplic (ico_n, ico_o)
Ico  *ico_n, *ico_o;
{ 
    Vor2d *ob_o = (Vor2d *) ico_o->obj;
    Vor2d *ob_n = (Vor2d *) ico_n->obj;

    memcpy (ob_n, ob_o, sizeof(Vor2d));
    vor2d_prepare_noms (ico_n, ob_o->ch1);

    ob_n->tab_point_base  = (Vor2d_point *)
	pcp_alloc ((ob_o->nb_max_point +3) * sizeof (Vor2d_point));
    ob_n->tab_sommet_base = (Vor2d_sommet *)
	pcp_alloc ((ob_o->nb_max_sommet+4) * sizeof (Vor2d_sommet));

    if (ob_n->tab_point_base == NULL || ob_n->tab_sommet_base == NULL) {
	ob_n->tab_point  = NULL;
	ob_n->tab_sommet = NULL;
	return;
    }

    memcpy (ob_n->tab_point_base, ob_o->tab_point_base,
	(ob_o->nb_max_point +3) * sizeof (Vor2d_point));
    memcpy (ob_n->tab_sommet_base, ob_o->tab_sommet_base,
	(ob_o->nb_max_sommet+4) * sizeof (Vor2d_sommet));
    ob_n->tab_point  = ob_n->tab_point_base  + 3;
    ob_n->tab_sommet = ob_n->tab_sommet_base + 4;
}
 
/****************************************************************************************
 * FUNCTION  : vor2d_affi_ico ( Ico *ico );
 *
 * PURPOSE   : Afficher l'icone IPS_VOR2D
 *
 * CALLED IN : method_affi_ico
 *
 ***************************************************************************************/
void  vor2d_affi_ico (ico)
Ico  *ico;
{
    Vor2d *ob = (Vor2d *) ico->obj;
    int i, x, y;
    XPoint dS[decoN], *dA, *dB;

    /* affichage titre et numero */
    XSetForeground (glo.dpy, glo.gc, glo.c[6]);
    x = ico->xb + 5;
    y = ico->yb + glo.font_h + 2;
    XDrawString (glo.dpy, glo.xwin, glo.gc, x, y, ob->ch1, ob->s_ch1);
    y += glo.font_h + 1;
    XDrawString (glo.dpy, glo.xwin, glo.gc, x, y, ob->ch2, ob->s_ch2);

    /* prepare coords du diagramme */
    y -= 4;
    for (i = 0; i < decoN; i++) {
	dS[i].x = decoS[i].x + x;
	dS[i].y = decoS[i].y + y;
    }

    /* polygone inverse' */
    XSetForeground (glo.dpy, glo.gc, ico->sel ? glo.c[3] : glo.c[2]);
    XFillPolygon (glo.dpy, glo.xwin, glo.gc, dS, 6, Convex, CoordModeOrigin);

    /* sommets Voronoi en rouge */
    XSetForeground (glo.dpy, glo.gc, coul_pic[12]);
    for (i = 0; i < 5; i++) {
	dA = dS + i; dB = dS + i+6;
	XDrawLine (glo.dpy, glo.xwin, glo.gc, dA->x, dA->y, dB->x, dB->y );
    }
    XDrawLines (glo.dpy, glo.xwin, glo.gc, dS, 6, CoordModeOrigin);
    XDrawString (glo.dpy, glo.xwin, glo.gc,
	ico->xb+ob->x_ch4, ico->yb+ico->ym-4, ob->ch4, ob->s_ch4);

    /* points Delaunay en vert */
    XSetForeground (glo.dpy, glo.gc, coul_pic[10]);
    dA = dS + 18;
    for (i = 12; i < 18; i++) {
	dB = dS + i;
	XDrawLine (glo.dpy, glo.xwin, glo.gc, dA->x, dA->y, dB->x, dB->y );
    }
    XDrawLines (glo.dpy, glo.xwin, glo.gc, dS+12, 6, CoordModeOrigin);
    XDrawString (glo.dpy, glo.xwin, glo.gc,
	ico->xb+5, ico->yb+ico->ym-4, ob->ch3, ob->s_ch3);
}

/****************************************************************************************
 * FUNCTION  : vor2d_prepare_noms ( Ico *ico, char *name );
 *
 * PURPOSE   : Calcule les coords des sous titres de l'IPS_VOR2D
 *
 * CALLED IN : method_FramePcpMenuProc
 *
 ***************************************************************************************/
void  vor2d_prepare_noms (ico, name)
Ico  *ico;
char *name;
{ 
    Vor2d *ob = (Vor2d *) ico->obj;
    int    i, c;
    char   bla[64];
    Font_string_dims dims;

    ico->xm = 64+10;
    ico->ym = 2+glo.font_h+1+glo.font_h-4+64+5;

    /* attribution de ch1 */
    strncpy (ob->ch1, name, VOR2D_CH1LEN);
    c = VRAI; i = 0;
    while (c)
    {
      if (ob->ch1[i] == 0)
           c = FAUX;
      else {
             bla[i] = ob->ch1[i]; bla[i+1] = 0;
             (void) xv_get (glo.petite_fonte, FONT_STRING_DIMS, bla, &dims);
             if (dims.width > ico->xm-10) { c = FAUX; bla[i] = 0; }
           }
      i++;
    }

    /* attribution des ch et s_ch */
    sprintf (ob->ch2, "n.%d", ico->index_creat); 
    sprintf (ob->ch3, "%d", ob->nb_point);
    sprintf (ob->ch4, "%d", ob->nb_sommet);
    ob->s_ch1 = strlen(bla);
    ob->s_ch2 = strlen(ob->ch2);
    ob->s_ch3 = strlen(ob->ch3);
    ob->s_ch4 = strlen(ob->ch4);

    /* attribution des coord en x des ch */
    (void) xv_get (glo.petite_fonte, FONT_STRING_DIMS, ob->ch4, &dims);
    ob->x_ch4 = ico->xm - 5 - dims.width;
}
 
/****************************************************************************************
 * FUNCTION  : vor2d_CloseAllWin (Ico *ico);
 *
 * CALLED IN : method_free
 *
 ****************************************************************************************/
void vor2d_CloseAllWin (ico)
Ico *ico;
{
    Vor2d *ob = (Vor2d *) ico->obj;
  
}

#ifdef bientotpeutetre

/****************************************************************************************
 * FUNCTION  : 
 *
 * CALLED IN :
 *
 ****************************************************************************************/
void vor2d_ (ob)
Vor2d *ob;
{
    int i, na, nb;
    Vor2d_sommet *s;

    /*
     * Aretes Delaunay
    */
    for (i = 0; i < ob->nb_sommet; i++) {

	s = ob->tab_sommet + i;

	if (i < s->num_sv[0] && (na = s->num_gc[1]) >= 0 && (nb = s->num_gc[2]) >= 0)
	    affi_seg (ob->tab_point[na].point, ob->tab_point[nb].point);

	if (i < s->num_sv[1] && (na = s->num_gc[2]) >= 0 && (nb = s->num_gc[0]) >= 0)
	    affi_seg (ob->tab_point[na].point, ob->tab_point[nb].point);

	if (i < s->num_sv[2] && (na = s->num_gc[0]) >= 0 && (nb = s->num_gc[1]) >= 0)
	    affi_seg (ob->tab_point[na].point, ob->tab_point[nb].point);
    }

    /*
     * Aretes Voronoi
    */
    for (i = 0; i < ob->nb_sommet; i++) {

	s = ob->tab_sommet + i;

	if (i < s->num_sv[0] && (na = s->num_gc[1]) >= 0 && (nb = s->num_gc[2]) >= 0)
	    affi_seg (s->point, ob->tab_sommet[s->num_sv[0]].point);

	if (i < s->num_sv[1] && (na = s->num_gc[2]) >= 0 && (nb = s->num_gc[0]) >= 0)
	    affi_seg (s->point, ob->tab_sommet[s->num_sv[1]].point);

	if (i < s->num_sv[2] && (na = s->num_gc[0]) >= 0 && (nb = s->num_gc[1]) >= 0)
	    affi_seg (s->point, ob->tab_sommet[s->num_sv[2]].point);
    }

}

/****************************************************************************************
 * FUNCTION  : 
 *
 * Clipping du segment xa,ya : xb,yb dans la fenetre x1,y1 : x2,y2
 * IDEE : les coords de la fenetre sont en static global au module, et une fonction
 * est specialisee pour les changer, et donner l'adresse de la fct de dessin.
 *
 ****************************************************************************************/`
static double CWx1, CWy1, CWx2, CWy2;
static void   (*CWfunc) ();

void set_clipping_window (x1, y1, x2, y2, func)
double x1, y1, x2, y2;
void (*func) ();
{
    CWx1 = x1;  CWx2 = x2;
    CWy1 = y1;  CWy2 = y2;
    CWfunc = func;
}

void affi_seg (pA, pB)
point *pA, *pB;
{
    double xa = pA[0], ya = pA[1], xb = pB[0], yb = pB[1];

    /*
     * 1ere etape : tests en x
    */
    if ((xa < CWx1 && xb < CWx1) || (xa > CWx2 && xb > CWx2)) return;
    if (xa < CWx1)
	{ xa = CWx1; ya = (ya*xb - xa*yb + xa*(yb-ya)) / (xb-xa); }
    else if (xa > CWx2)
	{ xa = CWx2; ya = (ya*xb - xa*yb + xa*(yb-ya)) / (xb-xa); }
    if (xb < CWx1)
	{ xb = CWx1; yb = (ya*xb - xa*yb + xb*(yb-ya)) / (xb-xa); }
    else if (xb > CWx2)
	{ xb = CWx2; yb = (ya*xb - xa*yb + xb*(yb-ya)) / (xb-xa); }

    /*
     * 2eme etape : tests en y
    */
    if ((ya < CWy1 && yb < CWy1) || (ya > CWy2 && yb > CWy2)) return;
    if (ya < CWy1)
	{ ya = CWy1; xa = (xa*yb - ya*xb + ya*(xb-xa)) / (yb-ya); }
    else if (ya > CWy2)
	{ ya = CWy2; xa = (xa*yb - ya*xb + ya*(xb-xa)) / (yb-ya); }
    if (yb < CWy1)
	{ yb = CWy1; xb = (xa*yb - ya*xb + yb*(xb-xa)) / (yb-ya); }
    else if (yb > CWy2)
	{ yb = CWy2; xb = (xa*yb - ya*xb + yb*(xb-xa)) / (yb-ya); }

    /*
     * 3eme etape : affichage
    */
    (*CWfunc) (xa, ya, xb, yb);
}

#endif
