/***************************************************************************************/
/* seldrop.c                                                  I.P.S  v 3.1  18/11/1993 */
/*                                                                                     */
/* Gestion des objets DRAGDROP et REQUESTOR du server    Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

/*** ELEMENTS MIS EN PLACE ***
 *
 *  glo.server = (Server) xv_init (XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
 *
 *  glo.canvas = (Canvas) xv_create (glo.frame_principal, CANVAS,
 *	...
 *	CANVAS_RESIZE_PROC,     ico_resize,
 *	NULL);
 *
 *  glo.sel_req = xv_create (glo.canvas, SELECTION_REQUESTOR,
 *	SEL_REPLY_PROC,		sd_req_reply,
 *	NULL);
 *
 *  glo.drop_site = xv_create (canvas_paint_window (glo.canvas), DROP_SITE_ITEM,
 *	DROP_SITE_REGION,       xv_get (canvas_paint_window (glo.canvas), WIN_RECT),
 *	DROP_SITE_DEFAULT,      TRUE,
 *	NULL);
 *
 *** DANS ico_resize (canvas, ..) ***
 *
 *  xv_set (glo.drop_site, 
 *	DROP_SITE_DELETE_REGION, NULL,
 *	DROP_SITE_REGION,        xv_get (canvas_paint_window (glo.canvas), WIN_RECT),
 *	NULL);
 *
 *** DANS ico_event_proc () ***
 *
 *  switch (event_action(event)) {
 *  ...
 *  case ACTION_DRAG_COPY :
 *  case ACTION_DRAG_MOVE :
 *	sd_req_importe (event);
 *	break;
 *  }
 *
 *** POUR EXPORTER ***
 *
 *  sd_sel_exporte ();
 *
 ****/


#include "inclus.h"

#define ATOM(x,y)  (Atom) xv_get (x, SERVER_ATOM, y)

static int sel_footer_n = VRAI;
static int sel_firstTime = VRAI;
static int sel_bufDataSize = 0;
static int sel_bufPartSize = 0;
static int sel_bufPartRemain = 0;
static char *sel_bufData = (char *) NULL;
static char *sel_bufPart = (char *) NULL;

static int req_bufDataSize = 0;
static int req_bufPartSize = 0;
static char *req_bufData = (char *) NULL;


/***************************************************************************************
 * sd_sel_exporte
 *
 * Creation de l'objet DRAGDROP ; encode (ico) ; Exportation avec dnd_send_drop.
 *
 ***************************************************************************************/
void sd_sel_exporte (ico)
Ico *ico;
{
  Selection_item selItem;
  Atom selAtom;
  int send_pb;
  static char message[] = "IPS_PRIVATE drag and drop operation\n";

  /*
   * Cree l'objet a exporter
  */
  glo.dnd = xv_create (glo.p_win, DRAGDROP,
	DND_TYPE, DND_COPY,
	SEL_CONVERT_PROC, sd_sel_convert,
	NULL);

  /*
   * Prepare le buffer a envoyer (sel_BufData, sel_BufDataSize)
  */
  pcp_footer ("Building DRAGDROP ..", NULL);
  sd_sel_encode (ico);
  pcp_footer_n ();
  sel_firstTime = VRAI;
  sel_footer_n = VRAI;

  /*
   * associe du texte a l'objet
  */
  selAtom = (Atom) XA_STRING;
  selItem = xv_create (glo.dnd, SELECTION_ITEM,
	SEL_TYPE, selAtom,
	SEL_DATA, (Xv_opaque) message,
	NULL);

  /*
   * exporte l'objet
  */
  send_pb = VRAI;
  switch (dnd_send_drop (glo.dnd)) {
      case XV_OK :
          send_pb = FAUX; if (sel_footer_n) pcp_footer_n (); break;
      case DND_TIMEOUT : 
          pcp_footer ("Drag and Drop: Time Out", NULL); break;
      case DND_ILLEGAL_TARGET : 
          pcp_footer ("Drag and Drop: Illegal Target", NULL); break;
      case DND_SELECTION : 
          pcp_footer ("Drag and Drop: Bad Selection", NULL); break;
      case DND_ROOT : 
          pcp_footer ("Drag and Drop: Root Window", NULL); break;
      default : 
          pcp_footer ("Drag and Drop: Failed", NULL); break;
  }

  /*
   * Libere les donnees
  */
  if (send_pb) sd_sel_free ();
}

/***************************************************************************************
 * sd_sel_convert
 *
 * Fonction de convertion de l'objet DRAGDROP.
 *
 * Pour l'Atom IPS_PRIVATE, on decoupe le message sel_bufData en tranches, avec
 * l'Atom INCR. Le server sait qu'il n'y a plus de tranches a envoyer quand *length
 * est fixe a 0.
 ***************************************************************************************/
int sd_sel_convert (seln, type, data, length, format)
Selection_owner seln;
Atom *type;
Xv_opaque *data;
long *length;
int *format;
{
# ifdef DEBUG  
  printf ("sd_sel_convert \"%s\"\n", 
      (char *) xv_get (glo.server, SERVER_ATOM_NAME, *type));
# endif

  if (*type == ATOM (glo.server, "IPS_PRIVATE")) {
      /*
       * Envoie INCR au requestor
      */
      if (sel_firstTime) {
          pcp_footer ("Sending DRAGDROP ..", NULL);
          *type = ATOM (glo.server, "INCR");
          *data = (Xv_opaque) &sel_bufDataSize;
          *length = 1;
          *format = 32;
          sel_firstTime = FAUX;
          sel_bufPartRemain = sel_bufDataSize;
          sel_bufPartSize = 32*1024; /* < 65536 ; tester rapidite */
          return TRUE;
      }

      sel_bufPart = sel_bufData + sel_bufDataSize - sel_bufPartRemain;
      if (sel_bufPartRemain < sel_bufPartSize) {
          sel_bufPartSize = sel_bufPartRemain;
          sel_bufPartRemain = 0;
      }
      else sel_bufPartRemain -= sel_bufPartSize;
      *format = 8;
      *length = sel_bufPartSize;
      *data = (Xv_opaque) sel_bufPart;
      return TRUE;
  }
  else if (*type == ATOM (glo.server, "_SUN_SELECTION_END") ||
      *type == ATOM (glo.server, "_SUN_DRAGDROP_DONE")) {
      sd_sel_free ();
      if (sel_footer_n) pcp_footer_n ();
      return FALSE;
  }

  return (sel_convert_proc (seln, type, data, length, format));
}

/***************************************************************************************
 * sd_sel_free
 *
 * Liberation de l'objet DRAGDROP
 *
 ***************************************************************************************/
void sd_sel_free ()
{
# ifdef DEBUG  
  printf ("sd_sel_free\n");
# endif

  if (glo.dnd != NULL)
      if (xv_destroy_safe(glo.dnd) != XV_OK)
          pcp_error ("sd_sel_free #01", NULL);
  glo.dnd = NULL;

  pcp_free (sel_bufData);
  sel_bufData = (char *) NULL;
}

/***************************************************************************************
 * sd_req_importe
 *
 * Appel du REQUESTOR
 *
 ***************************************************************************************/
void sd_req_importe (event)
Event *event;
{
  if (dnd_decode_drop (glo.sel_req, event) == XV_ERROR)
       pcp_error ("sd_req_importe #01", "drop error", NULL);

  else if (dnd_is_local (event)) {   /* on evite pour le moment car bugs */
      dnd_done (glo.sel_req);
      pcp_footer ("Ignoring Drag and Drop on same window", NULL);
      sel_footer_n = FAUX;
  }

  else {
      xv_set (glo.sel_req,
          SEL_TYPE_NAMES, "IPS_PRIVATE", "FILE_NAME", NULL,
          NULL);
      sel_post_req (glo.sel_req);
  }
}

/***************************************************************************************
 * sd_req_reply
 *
 * Fonction de dialogue du REQUESTOR
 *
 * Pour l'Atom IPS_PRIVATE, on reconstitue le message req_bufData recu en tranches.
 ***************************************************************************************/
void sd_req_reply (selReq, target, type, replyBuf, length, format)
Selection_requestor selReq;
Atom target, type;
Xv_opaque replyBuf;
unsigned long length;
int format;
{
# ifdef DEBUG  
  printf ("sd_req_reply \"%s\" %d %d\n", 
      (char *) xv_get (glo.server, SERVER_ATOM_NAME, target), length, format);
# endif

  if (length == SEL_ERROR) {
#     ifdef DEBUG  
      printf ("  Error\n");
#     endif
      return;
  }

  else if (length == 0) {
#     ifdef DEBUG  
      printf ("  End of incremental data transfer.\n");
#     endif
      pcp_footer ("Decoding DRAGDROP ..", NULL);
      sd_req_decode ();
      pcp_free (req_bufData);
      req_bufData = (char *) NULL;
      dnd_done (glo.sel_req);
      pcp_footer_n ();
  }

  else if (type == ATOM (glo.server, "INCR")) {
      memcpy ((char *) &req_bufDataSize, (char *) replyBuf, sizeof (int));
#     ifdef DEBUG  
      printf ("  Get ready for INCR of size %d\n", req_bufDataSize);
#     endif
      pcp_footer ("Receiving DRAGDROP ..", NULL);
      pcp_free (req_bufData);
      req_bufPartSize = 0;
      req_bufData = (char *) pcp_alloc (req_bufDataSize);
      if (req_bufData == (char *) NULL) req_bufDataSize = 0;
  }

  else if (target == ATOM (glo.server, "IPS_PRIVATE")) {
      if (req_bufPartSize + length <= req_bufDataSize)
          memcpy (req_bufData + req_bufPartSize, replyBuf, length);
      req_bufPartSize += length;
  }

  else if ( target == ATOM (glo.server, "FILE_NAME") &&
            replyBuf != (Xv_opaque) NULL ) {
      dnd_done (glo.sel_req);
      fichier_drop (replyBuf);
  }

  if (replyBuf != (Xv_opaque) NULL) free (replyBuf);
}

/***************************************************************************************
 * sd_sel_encode
 *
 * Encodage des donnees pour creer le message sel_bufData attache a l'objet DRAGDROP.
 *
 * Si clic == NULL on encode la selection, sinon on encode clic.
 ***************************************************************************************/
void sd_sel_encode (clic)
Ico *clic;
{
  int c, d, v, nb_ico, base;
  int clic_nul = (clic == NULL);
  Ico *ico;

  /*
   * Premier passage : on calcule la taille sel_bufDataSize du message
  */
  sel_bufDataSize =
      sizeof (int) +					/* taille message total	*/
      sizeof (int);					/* nb_ico		*/
  nb_ico = 0;
  ico = (clic_nul) ? glo.l_ico_z : clic;
  while (ico != NULL) {
      if (ico->sel || !clic_nul) {

          nb_ico++;
          sel_bufDataSize +=
              sizeof(int) +			/* taille mess pr cet ico	*/
              sizeof(int) ;			/* nat_obj de cet ico		*/

          switch (ico->nat_obj) {

              case IPS_IMAGE : {
                  Image *im = (Image *) ico->obj;

                  sel_bufDataSize +=
                      strlen (im->ch1)+1 +		/* ico->ch1		*/
                      sizeof(int) +			/* IPS_COUL_		*/
                      sizeof(int)*2 +			/* xsiz,ysiz		*/
                      sizeof(Unt16)*im->b.xsiz*im->b.ysiz;	/* taille image		*/
              } break;

          } /* switch */
      } /* if */

      if (clic_nul)
           ico = ico->pz;
      else ico = NULL;
  }
  pcp_free (sel_bufData);
  sel_bufData = (char *) pcp_alloc (sel_bufDataSize + 1); /* +1 par precaution */
  if (sel_bufData == (char *) NULL) {
      sel_bufDataSize = 0; return;
  }

  /*
   * Deuxieme passage : on ecrit le message sel_bufData
  */
  c = 0;
  memcpy (sel_bufData + c, &sel_bufDataSize, d = sizeof(int)); c += d;
  memcpy (sel_bufData + c, &nb_ico, d = sizeof(int)); c += d;
  ico = (clic_nul) ? glo.l_ico_z : clic;
  while (ico != NULL) {
      if (ico->sel || !clic_nul) {

          base = c; c += sizeof(int); /* cf apres switch */
          memcpy (sel_bufData + c, &ico->nat_obj, d = sizeof(int)); c += d;

          switch (ico->nat_obj) {

              case IPS_IMAGE : {
                  Image *im = (Image *) ico->obj;
                  int x, y;

                  strcpy (sel_bufData + c, im->ch1); c += strlen (im->ch1)+1;
                  v = glo_get_attr_lut (im->coul_tab);
                  memcpy (sel_bufData + c, &v, d = sizeof(int)); c += d;
                  memcpy (sel_bufData + c, &im->b.xsiz, d = sizeof(int)); c += d;
                  memcpy (sel_bufData + c, &im->b.ysiz, d = sizeof(int)); c += d;
                  d = sizeof(Unt16);
                  for (y = im->b.cmin; y < im->b.ymax; y++)
                  for (x = im->b.cmin; x < im->b.xmax; x++) {
                      memcpy (sel_bufData + c, &im->tab[y*im->b.xtot+x], d);
                      c += d;
                  }
              } break;

          } /* switch */

          v = c - base - sizeof(int)*2; /* taille variable du mess pr ico */
          memcpy (sel_bufData + base, &v, sizeof(int));
      } /* if */

      if (clic_nul)
           ico = ico->pz;
      else ico = NULL;
  }

}

/***************************************************************************************
 * sd_req_decode
 *
 * Decodage du message req_bufData attache au REQUESTOR.
 *
 ***************************************************************************************/
void sd_req_decode ()
{
  Ico *ico;
  int d, v, n, nb_ico, nat_obj, t, tm_ico;
  int c = 0;

  /*
   * Test req_bufDataSize
  */
  if (req_bufDataSize == 0 || req_bufData == (char *) NULL)
      return;

  memcpy (&t, req_bufData + c, d = sizeof(int)); c += d;

  if (req_bufDataSize != t) {
      char bla1[80], bla2[80];
      sprintf (bla1, "%d bytes expected", t);
      sprintf (bla2, "%d bytes received", req_bufDataSize);
      pcp_error ("sd_req_decode #1", "Incomplete message :", bla1, bla2, NULL);
      return;
  }

  /*
   * Nombre d'Ico
  */
  memcpy (&nb_ico, req_bufData + c, d = sizeof(int)); c += d;

  /*
   * Creation des Ico
  */
  for (n = 0; n < nb_ico; n++) {

      /* Nature de l'Ico ; creation */
      memcpy (&tm_ico, req_bufData + c, d = sizeof(int)); c += d;   /* taille mess ico */
      memcpy (&nat_obj, req_bufData + c, d = sizeof(int)); c += d;  /* nature ico      */

      /* test si nat_obj connu */
      if (nat_obj == IPS_IMAGE) {

          ico = ico_creat (glo.sx, glo.sy, nat_obj);
          ico_tete_pz (ico);

          switch (nat_obj) {

              case IPS_IMAGE : {
                  Image *im = (Image *) ico->obj;
                  char * ch1;
                  int xsiz, ysiz, coul;

                  ch1 = req_bufData + c; c += strlen (ch1) + 1;
                  memcpy (&coul, req_bufData + c, d = sizeof (int)); c += d;
                  memcpy (&xsiz, req_bufData + c, d = sizeof (int)); c += d;
                  memcpy (&ysiz, req_bufData + c, d = sizeof (int)); c += d;

                  image_creat (im, 0, xsiz, ysiz, NULL);
                  memcpy (im->tab, req_bufData+c, d = xsiz*ysiz*sizeof(Unt16)); c += d;
                  sprintf (im->ch1, "%s", ch1);
                  image_calc_hublot (ico);
                  im->coul_tab = glo_get_ctab_lut (coul);
              } break;

          } /* switch (nat_obj) */

          /* Positionnement et affichage de l'Ico */
          if (n == 0) { 
              ico->xb -= ico->xm/2; ico->yb -= ico->ym/2;
              glo.sx = ico->xb; glo.sy = ico->yb;
          }
          ico_douane (ico);
          ico_refresh_now (ico);
          glo.sx += 10; glo.sy += 16;

      }
      else /* sauter la partie recue de taille tm_ico */
          c += tm_ico;

  } /* for n */

}


